/*
 * Name: dgsocket.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 2001-04-23
 * Tabsize: 4
 * Copyright: (c) 2001 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: dgsocket.h,v 1.3 2001/05/31 09:36:52 cs Exp $
 */

#ifndef __dgsocket_H_INCLUDED__
#define __dgsocket_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This module implements a datagram socket. The socket is kept open as long as
there are references to it.
*/

/* ------------------------------------------------------------------------- */

#define	DG_SOCKET_MAX_INTERFACES	32

typedef struct dgSocketInfo{
	btint	fd;
	btu32	localIp;
	btu32	netmask;
}dgSocketInfo_t;

typedef struct dgSocket{
	btint			localPort;
	btint			refCount;
	dgSocketInfo_t	socks[DG_SOCKET_MAX_INTERFACES + 1];	/* [0] = generic socket */
	btint			numSocks;
	struct procCond	*msgReceivedSignal;
}dgSocket_t;

typedef struct dgSocketRx{
	btint	unixError;
	btu32	remoteIp;
	btint	remotePort;
	btint	len;
	btu8	data[800];
}dgSocketRx_t;

/* ------------------------------------------------------------------------- */

dgSocket_t	*dgSocketForPort(btint localPort);
btint	dgSocketRetain(dgSocket_t *sock);
void	dgSocketRelease(dgSocket_t *sock);
void	dgSocketSend(void *caller, dgSocket_t *sock, btu32 ipAddr, btint remotePort, btu8 *data, btint len);
void	dgSocketBroadcast(void *caller, dgSocket_t *sock, btint remotePort, btu8 *data, btint len);

/* ------------------------------------------------------------------------- */

#endif /* __dgsocket_H_INCLUDED__ */
