/*
 * Name: fileattrunix.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1998-04-08
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: fileattrunix.h,v 1.1.1.1 1999/04/12 13:56:34 cs Exp $
 */

#ifndef __fileattrunix_H_INCLUDED__
#define __fileattrunix_H_INCLUDED__

#include "fileattr.h"

/*
General Description:
This is the (abstract) superclass of the file attribute classes. There is an
extra abstraction layer around attributes because this is the thing that
differs most among operating systems. Subclasses like fileAttrDOS,
fileAttrUnix and FileAttrNT are possible. The file attribute classes are
bound statically to the file class and the cache classes.
*/

/* ------------------------------------------------------------------------- */

typedef struct fileAttrUnix{
	fileAttr_t	general;
	bti64		chtime;		/* attribute change time */
    btu64		owner;		/* uid of owner */
    btu64		group;		/* gid of owner */
    btu64		nlinks;		/* number of links */
	btu32		type;		/* type of file */
	btu32		mode;		/* unix access mode */
    btu32		devMajor;	/* device number major */
    btu32		devMinor;	/* device number minor */
}fileAttrUnix_t;

/* ------------------------------------------------------------------------- */

void	*attrUnixNew(void);

/* ------------------------------------------------------------------------- */

#endif /* __fileattrunix_H_INCLUDED__ */
