/*
 * Name: licman.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1998-10-17
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: licman.h,v 1.2 1999/07/22 17:54:02 cs Exp $
 */

#ifndef __licman_H_INCLUDED__
#define __licman_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This module contains the functions used to check and read the license keys.
I won't comment too much on these because licensing is not intended to be
changed :-). You remember: You may change sharity but you must not change
the licensing mechanism or the behaviour in demo-mode.
*/

/* ------------------------------------------------------------------------- */

typedef struct licClientInfo{
	btu8			*clName;
	btu32			ipAddress;
	struct array	*clServers;	/* array of ascii strings */
}licClientInfo_t;

struct server;
struct array;
struct cifsFile;

void	licInit(void);
btu32	licSetKey(btu8 *licenseString);
void	licKeyIdentifier(btu8 licenseString[8]);
void	licRegisterServer(struct server *server, struct cifsUser *user, btError *err);
void	licRemoveServer(struct server *server);
struct server	*licLookupServer(btu8 *name);
struct array	*licOurServers(void);
void	licRequestSync(void);
btint	licNumServers(void);
btint	licNumClients(void);
struct array	*licAllServers(void);	/* array of ascii strings */
struct array	*licAllClients(void);	/* array of licClientInfo_t */
void	licRegisterFile(struct cifsFile *parent, struct cifsFile *child, btError *err);
btint	licDepthLimit(void);

/* ------------------------------------------------------------------------- */

#endif	/* __licman_H_INCLUDED__ */
