/*
 * Name: nbdgs.c
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 2001-04-22
 * Tabsize: 4
 * Copyright: (c) 2001 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: nbdgs.c,v 1.3 2001/06/16 19:37:21 cs Exp $
 */

#include "stdheaders.h"
#include "nbdgs.h"
#include "unicode.h"
#include "nbnsclient.h"
#include "logging.h"
#include "smbnumbers.h"
#include "dgsocket.h"
#include "blockpool.h"
#include "process.h"
#include "system.h"
#include "cfgparse.h"
#include "cifsmod.h"

#define	NBDGS_PORT					138
#define	LMB_BROWSE_REQUEST_ELECTION	8

/* ------------------------------------------------------------------------- */

typedef struct nbdgsCtx{
	PROC_VARSET;
	dgSocket_t	*sock;
	btint		len;
	btu8		buffer[576];
}nbdgsCtx_t;

static blockpool_t *nbdgsPool = NULL;

static void	poolFreeBlock(void *block)
{
    poolReturnBlock(nbdgsPool, block);
}

/* ------------------------------------------------------------------------- */

void	nbdgsSendToBrowseMailslotProc(void *obj, void *arg)
{
nbdgsCtx_t		*this = obj;
cfgProperty_t	*path[8];
btint			numEntries;
static btint	bindNbdgsPort;
static btu32	cfgChg = 0;

	PROC_BEGIN;
	if(CFG_DID_CHANGE(cfgChg)){
		numEntries = cifsModGlobalCfgPath(path, ELEMENTS(path));
		bindNbdgsPort = cfgIntValue(cfgQueryFromArray("bindNbdgsPort", numEntries, path), NBDGS_PORT);
	}
	this->sock = dgSocketForPort(bindNbdgsPort);
	PROC_WAITFOR_X(1, dgSocketBroadcast(this, this->sock, NBDGS_PORT, this->buffer, this->len));
	dgSocketRelease(this->sock);
	PROC_RETURN(arg);
	PROC_END;
}

void	nbdgsSendToBrowseMailslot(void *caller, btu8 *data, btsize size, btu8 *myName, btu8 *remoteName, btint remoteTag)
{
nbdgsCtx_t			*ctx;
btint				i = 0, lenPtr;
static btint		nbdgsXidCounter = 0;

    if(nbdgsPool == NULL)
        nbdgsPool = poolNew(sizeof(nbdgsCtx_t), (btu8 *)&ctx->buffer - (btu8 *)ctx);
	PROC_INIT(ctx, nbdgsSendToBrowseMailslotProc, caller, nbdgsPool, poolFreeBlock);
	memset(ctx->buffer, 0, sizeof(ctx->buffer));
	ctx->buffer[i++] = 0x11;	/* direct group datagram */
	ctx->buffer[i++] = 0x0a;	/* first, no more, M node */
	ctx->buffer[i++] = (nbdgsXidCounter >> 8) & 0xff;
	ctx->buffer[i++] = nbdgsXidCounter++ & 0xff;
	i += 4;		/* source IP address not known, leave it at 0 */
	ctx->buffer[i++] = NBDGS_PORT >> 8;
	ctx->buffer[i++] = NBDGS_PORT;
	lenPtr = i;
	i += 4;		/* space for 2 bytes length, 2 bytes packet offset */
	i += nbnsEncodeName(ctx->buffer + i, sizeof(ctx->buffer) - i, myName, 0);
	i += nbnsEncodeName(ctx->buffer + i, sizeof(ctx->buffer) - i, remoteName, remoteTag);
	/* start of Datagram content */
	ctx->buffer[i++] = 0xff;
	ctx->buffer[i++] = 'S';
	ctx->buffer[i++] = 'M';
	ctx->buffer[i++] = 'B';
	ctx->buffer[i++] = SMB_COM_TRANSACTION;
	i += 4;				/* 32 bit error */
	i += 3;				/* flags, flags2 */
	i += 12;			/* 12 bytes reserved */
	i += 8;				/* tid, pid, uid, mid */
	ctx->buffer[i++] = 0x11;	/* word count */
	i += 2;				/* total param */
	ctx->buffer[i++] = size;	/* total data */
	ctx->buffer[i++] = size >> 8;
	i += 6;				/* max param, max data, max setup */
	i += 8;				/* flags, timeout, reserved */
	i += 4;				/* param count, param offset */
	ctx->buffer[i++] = size;	/* data count */
	ctx->buffer[i++] = size >> 8;
	ctx->buffer[i++] = 0x56; i++;	/* data offset */
	ctx->buffer[i++] = 3; i++;	/* setup count */
	ctx->buffer[i++] = 1; i++;	/* setup[0] = mailslot write */
	ctx->buffer[i++] = 1; i++;	/* setup[1] = transaction priority */
	ctx->buffer[i++] = 2; i++;	/* setup[2] = mailslot class (unreliable) */
	ctx->buffer[i++] = size + 17;	/* byte count */
	ctx->buffer[i++] = (size + 17) >> 8;
	strcpy(ctx->buffer + i, "\\MAILSLOT\\BROWSE");
	i += 17;
	if(i + size > sizeof(ctx->buffer)){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "nbdgs: buffer overflow (%d bytes, buffer=%d)\n", i + size, sizeof(ctx->buffer));
		size = sizeof(ctx->buffer) - i;	/* truncate silently */
	}
	memcpy(ctx->buffer + i, data, size);
	i += size;
	ctx->buffer[lenPtr++] = i >> 8;
	ctx->buffer[lenPtr] = i;
	ctx->len = i;
    PROC_START(ctx, NULL);
}

/* ------------------------------------------------------------------------- */

void	nbdgsSendRequestBrowserElection(void *caller, btu8 *domain)
{
btu8	buffer[20], myName[256];
btint	len, i = 0;

	memset(buffer, 0, sizeof(buffer));
	buffer[i++] = LMB_BROWSE_REQUEST_ELECTION;
	buffer[i++] = 1;	/* version */
	i += 4;				/* criteria */
	i += 4;				/* TimeUp */
	i += 4;				/* Must Be Zero */
	buffer[i++] = 0;	/* ServerName */
	len = i;
	if(gethostname(myName, sizeof(myName)) != 0){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "nbdgs: gethostname(): [%d] %s\n", errno, sysStrerror(errno));
		strncpy(myName, domain, sizeof(myName));	/* we must use a name */
	}
	myName[sizeof(myName)-1] = 0;	/* ensure terminating 0 */
	/* convert to uppercase, strip domain part */
	for(i=0; myName[i] != '.' && myName[i] != 0; i++)
		myName[i] = asciiCharToupper(myName[i]);
	myName[i] = 0;
	nbdgsSendToBrowseMailslot(caller, buffer, len, myName, domain, 0x1e);
}

/* ------------------------------------------------------------------------- */
