/*
 * Name: pslnetbios.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1998-05-21
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: pslnetbios.h,v 1.2 1999/07/22 17:54:04 cs Exp $
 */

#ifndef __pslnetbios_H_INCLUDED__
#define __pslnetbios_H_INCLUDED__

#include "basictypes.h"
#include "pslayer.h"

/*
General Description:
This module implements the netbios protocol within the protocol stack. It's
usual neighbouring layers are the server object on the upper side and a
socket layer object on the lower side. The netbios socket layer implements
both interfaces: psUpInterface and psDownInterface.
*/

/* ------------------------------------------------------------------------- */

struct pslSslSocket;
struct pslSocket;
struct server;

typedef struct pslNetbios{
	int					refCount;
	psDownInterface_t	*higherI;
	void				*higherObj;
	psUpInterface_t		*lowerI;
	void				*lowerObj;
	btu8				*passPhrase;
	int					fdForSSLSocket;
	struct server		*server;		/* needed in pslsocket for timeout */
	
	btbool				requireEncryption;
	btu32				destIp;
	int					port;
	btu8				myName[16];
	btu8				remoteName[16];
	btbool				txSessionRequest;
	btbool				txSessionRqWithRealName;

	int					cstatus;
	struct procCond		*connectSignal;

	int					rxState;
	btu8				rxHeader[16];
	btsize				rxHdrPtr;
	btsize				rxHdrLen;
	btu8				*rxData;
	btsize				rxDataPtr;
	btsize				rxDataLen;
	int					rxDataAllocated;

	btcallback			rxSessionResponseCallback;
	void				*rxContext;
}pslNetbios_t;

/* ------------------------------------------------------------------------- */

pslNetbios_t	*pslNetbiosNew(char *myName, char *remoteName, btu32 destIp, int port, struct server *server);
/* server is needed by socket layer for timeout */
void	pslNetbiosSetPassPhrase(pslNetbios_t *psl, btu8 *passPhrase);
btbool	pslNetbiosConnectionEncrypted(pslNetbios_t *psl);

static inline void	pslNetbiosSetRequireEncryption(pslNetbios_t *psl, btbool reqEnc)
{
	psl->requireEncryption = reqEnc;
}

extern psUpInterface_t		pslNetbiosUpInterface;
extern psDownInterface_t	pslNetbiosDownInterface;

#define	NETB_ERR_NEED_PASSPHRASE	-10000
#define	NETB_ERR_SERVER_NOSSL		-10001
#define	NETB_ERR_CLIENT_NOSSL		-10002
#define	NETB_ERR_CLIENT_NOSSL_C		-10003

/* ------------------------------------------------------------------------- */

#endif /* __pslnetbios_H_INCLUDED__ */
