/*
 * Name: pwhash.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-12-07
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: pwhash.h,v 1.1.1.1 1999/04/12 13:56:35 cs Exp $
 */

#ifndef __pwhash_H_INCLUDED__
#define __pwhash_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This module implements all hashing functions needed to do SMB encrypted
passwords. It goes the same way as "samba" by not using a DES library
and implementing a one-way only DES, which can not be used for data
encryption to avoid troubles with US export laws. If you are looking for a
DES implementation, this is certainly not the place where to find a usable
one. In that case I would recommend that you implement one of your own
(Matthew Fischer <mfischer@blue.weeg.uiowa.edu> has posted a very good
step by step tutorial which was posted to sci.crypt and should be archived
at dejanews). Or you use one of the free libraries that can be downloaded
from non-US sites.
*/

/* ------------------------------------------------------------------------- */

void	pwhPreNtHash(btuni *passwd, int pwlen, btu8 hash[16]);
void	pwhBlockHash(btu8 block[7], btu8 hash[8]);
void	pwhNtHash(btuni *passwd, int pwlen, btu8 hash[16]);
void	pwhResponse(btu8 hash[21], btu8 challenge[8], btu8 response[24]);

/* ------------------------------------------------------------------------- */

#endif	/* __pwhash_H_INCLUDED__ */
