/*
 * Name: random.c
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-12-11
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: random.c,v 1.1.1.1 1999/04/12 13:56:34 cs Exp $
 */


#include "stdheaders.h"
#include "basictypes.h"

/*
General Description:
This program generates quasi-random data to make each compilation of
the CIFS client unique.
*/

/* ------------------------------------------------------------------------- */
/* ------------------------------- md4 hash -------------------------------- */
/* ------------------------------------------------------------------------- */

static inline btu32	md4_f(btu32 x, btu32 y, btu32 z)
{
	return (x & y) | (~x & z);
}

static inline btu32	md4_g(btu32 x, btu32 y, btu32 z)
{
	return (x & y) | (x & z) | (y & z);
}

static inline btu32	md4_h(btu32 x, btu32 y, btu32 z)
{
	return x ^ y ^ z;
}

static inline btu32	md4_rot(btu32 x, int bits)
{
	return (x << bits) | (x >> (32 - bits));
}

#define	OP1(A, B, C, D, i, s)	A = md4_rot(A + md4_f(B, C, D) + data[i], s);
#define	OP2(A, B, C, D, i, s)	A = md4_rot(A + md4_g(B, C, D) + data[i] + 0x5A827999, s);
#define	OP3(A, B, C, D, i, s)	A = md4_rot(A + md4_h(B, C, D) + data[i] + 0x6ED9EBA1, s);

static void	md4Block(btu32 context[4], btu32 data[16])
{
btu32	a, b, c, d;

	a = context[0];
	b = context[1];
	c = context[2];
	d = context[3];

	OP1(a, b, c, d,  0,  3)
	OP1(d, a, b, c,  1,  7)
	OP1(c, d, a, b,  2, 11)
	OP1(b, c, d, a,  3, 19)
	OP1(a, b, c, d,  4,  3)
	OP1(d, a, b, c,  5,  7)
	OP1(c, d, a, b,  6, 11)
	OP1(b, c, d, a,  7, 19)
	OP1(a, b, c, d,  8,  3)
	OP1(d, a, b, c,  9,  7)
	OP1(c, d, a, b, 10, 11)
	OP1(b, c, d, a, 11, 19)
	OP1(a, b, c, d, 12,  3)
	OP1(d, a, b, c, 13,  7)
	OP1(c, d, a, b, 14, 11)
	OP1(b, c, d, a, 15, 19)
	
	OP2(a, b, c, d,  0,  3)
	OP2(d, a, b, c,  4,  5)
	OP2(c, d, a, b,  8,  9)
	OP2(b, c, d, a, 12, 13)
	OP2(a, b, c, d,  1,  3)
	OP2(d, a, b, c,  5,  5)
	OP2(c, d, a, b,  9,  9)
	OP2(b, c, d, a, 13, 13)
	OP2(a, b, c, d,  2,  3)
	OP2(d, a, b, c,  6,  5)
	OP2(c, d, a, b, 10,  9)
	OP2(b, c, d, a, 14, 13)
	OP2(a, b, c, d,  3,  3)
	OP2(d, a, b, c,  7,  5)
	OP2(c, d, a, b, 11,  9)
	OP2(b, c, d, a, 15, 13)
	
	OP3(a, b, c, d,  0,  3)
	OP3(d, a, b, c,  8,  9)
	OP3(c, d, a, b,  4, 11)
	OP3(b, c, d, a, 12, 15)
	OP3(a, b, c, d,  2,  3)
	OP3(d, a, b, c, 10,  9)
	OP3(c, d, a, b,  6, 11)
	OP3(b, c, d, a, 14, 15)
	OP3(a, b, c, d,  1,  3)
	OP3(d, a, b, c,  9,  9)
	OP3(c, d, a, b,  5, 11)
	OP3(b, c, d, a, 13, 15)
	OP3(a, b, c, d,  3,  3)
	OP3(d, a, b, c, 11,  9)
	OP3(c, d, a, b,  7, 11)
	OP3(b, c, d, a, 15, 15)
	
	context[0] += a;
	context[1] += b;
	context[2] += c;
	context[3] += d;
}

/* ------------------------------------------------------------------------- */

int		main(void)
{
struct timeval	tv;
struct timezone	tz;
char			filename[256];
struct stat		sbuf;
btu32			buffer[16];
btu32			hash[4];
int				i;

	gettimeofday(&tv, &tz);
	sprintf(filename, "/tmp/$rand-%d", (int)getpid());
	close(creat(filename, 0666));
	if(stat(filename, &sbuf) != 0)
		memset(&sbuf, 0, sizeof(sbuf));
	unlink(filename);
	i = 0;
	buffer[i++] = tv.tv_sec;
	buffer[i++] = tv.tv_usec;
	if(sbuf.st_ino != 0){
		buffer[i++] = sbuf.st_ino;
	}
	hash[0] = 0x67452301;
	hash[1] = 0xefcdab89;
	hash[2] = 0x98badcfe;
	hash[3] = 0x10325476;
	md4Block(hash, buffer);
	printf("#define RND_NUMBER\t\t0x%08x%08xULL\n", hash[0], hash[1]);
	exit(0);
	return 0;
}


/* ------------------------------------------------------------------------- */

