/*
 * Name: smbnumbers.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-10-20
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: smbnumbers.h,v 1.1.1.1 1999/04/12 13:56:35 cs Exp $
 */

#ifndef __smbnumbers_H_INCLUDED__
#define __smbnumbers_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This header defines all constants needed from the CIFS specification as well
as some extensions (eg. for local error codes).
*/

/* ------------------------------------------------------------------------- */

#define	CIFS_MAX_PATHLEN	1024

/* file attributes */
#define	CIFS_ATTR_READONLY	0x01
#define	CIFS_ATTR_HIDDEN	0x02
#define	CIFS_ATTR_SYSTEM	0x04
#define	CIFS_ATTR_VOLUME	0x08
#define	CIFS_ATTR_DIR		0x10
#define	CIFS_ATTR_ARCHIVE	0x20

/* client capabilities */
#define	CIFS_CLIENT_CAP_UNICODE               0x0004
#define	CIFS_CLIENT_CAP_LARGE_FILES           0x0008
#define	CIFS_CLIENT_CAP_NT_SMBS               0x0010
#define	CIFS_CLIENT_CAP_NT_FIND               0x0200
#define	CIFS_CLIENT_CAP_STATUS32              0x0040
#define	CIFS_CLIENT_CAP_LEVEL_II_OPLOCKS      0x0080

/* server capabilites */
#define	CIFS_SERVER_CAP_RAW_MODE         0x0001
#define	CIFS_SERVER_CAP_MPX_MODE         0x0002
#define	CIFS_SERVER_CAP_UNICODE          0x0004
#define	CIFS_SERVER_CAP_LARGE_FILES      0x0008
#define	CIFS_SERVER_CAP_NT_SMBS          0x0010
#define	CIFS_SERVER_CAP_RPC_REMOTE_APIS  0x0020
#define	CIFS_SERVER_CAP_STATUS32         0x0040
#define	CIFS_SERVER_CAP_LEVEL_II_OPLOCKS 0x0080
#define	CIFS_SERVER_CAP_LOCK_AND_READ    0x0100
#define	CIFS_SERVER_CAP_NT_FIND          0x0200
#define	CIFS_SERVER_CAP_DFS              0x1000
#define	CIFS_SERVER_CAP_UNIX			 0x80000000

/* server/client flags and flags2 values */
#define	CIFS_FLAG_GUARANTEE_RXBUF		0x02
#define	CIFS_FLAG_CASE_INSENSITIVE		0x08
#define	CIFS_FLAG_EXCLUSIVE_LOCK		0x20
#define	CIFS_FLAG_BATCH_LOCK			0x40
#define	CIFS_FLAG2_LONG_NAMES			0x0001
#define	CIFS_FLAG2_EXTENDED_ATTRIBUTES	0x0002
#define	CIFS_FLAG2_UNICODE				0x8000

/* server security modes in negotiate */
#define	CIFS_SECMODE_PWENCRYPT		0x02
#define	CIFS_SECMODE_USERLEVEL		0x01

/* oplock levels */
#define	CIFS_OPLOCK_NONE		0
#define	CIFS_OPLOCK_L2			1
/* these next two are not used in the protocol but internally: */
#define	CIFS_OPLOCK_EXCLUSIVE	2
#define	CIFS_OPLOCK_BATCH		3

/* query FS information levels */
#define	SMB_INFO_ALLOCATION            1
#define	SMB_INFO_VOLUME                2
#define	SMB_QUERY_FS_VOLUME_INFO       0x102
#define	SMB_QUERY_FS_SIZE_INFO         0x103
#define	SMB_QUERY_FS_DEVICE_INFO       0x104
#define	SMB_QUERY_FS_ATTRIBUTE_INFO    0x105

/* query/find file information levels */
#define	SMB_INFO_STANDARD                1
#define	SMB_INFO_QUERY_EA_SIZE           2
#define	SMB_INFO_QUERY_EAS_FROM_LIST     3
#define	SMB_INFO_QUERY_ALL_EAS           4
#define	SMB_INFO_IS_NAME_VALID           6

/* query file information levels */
#define	SMB_QUERY_FILE_BASIC_INFO        0x101
#define	SMB_QUERY_FILE_STANDARD_INFO     0x102
#define	SMB_QUERY_FILE_EA_INFO           0x103
#define	SMB_QUERY_FILE_NAME_INFO         0x104
#define	SMB_QUERY_FILE_ALL_INFO          0x107
#define	SMB_QUERY_FILE_ALT_NAME_INFO     0x108
#define	SMB_QUERY_FILE_STREAM_INFO       0x109
#define	SMB_QUERY_FILE_COMPRESSION_INFO  0x10B
#define	SMB_QUERY_FILE_UNIX_BASIC		 0x200
#define	SMB_QUERY_FILE_UNIX_LINK		 0x201

#define	SMB_UNIX_FILETYPE_FILE			0
#define	SMB_UNIX_FILETYPE_DIR			1
#define	SMB_UNIX_FILETYPE_SYMLINK		2
#define	SMB_UNIX_FILETYPE_CHARDEV		3
#define	SMB_UNIX_FILETYPE_BLOCKDEV		4
#define	SMB_UNIX_FILETYPE_FIFO			5

/* trans2 find file modes */
#define	SMB_FIND_FILE_DIRECTORY_INFO       0x101
#define	SMB_FIND_FILE_FULL_DIRECTORY_INFO  0x102
#define	SMB_FIND_FILE_NAMES_INFO           0x103
#define	SMB_FIND_FILE_BOTH_DIRECTORY_INFO  0x104
#define	SMB_FIND_FILE_UNIX				   0x202

/* NT_CREATE parameter defines */
#define	SMB_FILE_ATTR_ARCHIVE     0x020
#define	SMB_FILE_ATTR_COMPRESSED  0x800
#define	SMB_FILE_ATTR_NORMAL      0x080
#define	SMB_FILE_ATTR_HIDDEN      0x002
#define	SMB_FILE_ATTR_READONLY    0x001
#define	SMB_FILE_ATTR_TEMPORARY   0x100
#define	SMB_FILE_ATTR_DIRECTORY   0x010
#define	SMB_FILE_ATTR_SYSTEM      0x004
#define	SMB_FILE_EA_WRITE_THROUGH    0x80000000
#define	SMB_FILE_EA_NO_BUFFERING     0x20000000
#define	SMB_FILE_EA_RANDOM_ACCESS    0x10000000
#define	SMB_FILE_EA_SEQUENTIAL_SCAN  0x08000000
#define	SMB_FILE_EA_DELETE_ON_CLOSE  0x04000000
#define	SMB_FILE_EA_BACKUP_SEMANTICS 0x02000000
#define	SMB_FILE_EA_POSIX_SEMANTICS  0x01000000

#define	SMB_NTCR_FILE_SHARE_READ			1
#define	SMB_NTCR_FILE_SHARE_WRITE			2
#define	SMB_NTCR_FILE_SHARE_DELETE			4

#define	SMB_NTCR_CREATE_NEW					1
#define	SMB_NTCR_CREATE_ALWAYS				2
#define	SMB_NTCR_OPEN_EXISTING				3
#define	SMB_NTCR_OPEN_ALWAYS				4
#define	SMB_NTCR_TRUNCATE_EXISTING			5

/* core access modes: */
#define	SMB_ACC_MASK_SHARE					0x70
#define	SMB_ACC_MASK_ACCESS					0x7
#define	SMB_ACC_SHARE_DENY_NONE				0x40
#define	SMB_ACC_READ						0
#define	SMB_ACC_RW							2

/* open functions: */
#define	SMB_OPEN_CREATE						0x10
#define	SMB_OPEN_OPEN						0x1
#define	SMB_OPEN_TRUNCATE					0x2

/* lockingAndX flags: */
#define	LOCKING_ANDX_SHARED_LOCK      0x01
#define	LOCKING_ANDX_OPLOCK_RELEASE   0x02
#define	LOCKING_ANDX_CHANGE_LOCKTYPE  0x04
#define	LOCKING_ANDX_CANCEL_LOCK      0x08
#define	LOCKING_ANDX_LARGE_FILES      0x10


/* ------------------------------------------------------------------------- */

/* SMB command codes */
#define	SMB_COM_CREATE_DIRECTORY    0x00
#define	SMB_COM_DELETE_DIRECTORY    0x01
#define	SMB_COM_OPEN                0x02
#define	SMB_COM_CREATE              0x03
#define	SMB_COM_CLOSE               0x04
#define	SMB_COM_FLUSH               0x05
#define	SMB_COM_DELETE              0x06
#define	SMB_COM_RENAME              0x07
#define	SMB_COM_QUERY_INFORMATION   0x08
#define	SMB_COM_SET_INFORMATION     0x09
#define	SMB_COM_READ                0x0A
#define	SMB_COM_WRITE               0x0B
#define	SMB_COM_LOCK_BYTE_RANGE     0x0C
#define	SMB_COM_UNLOCK_BYTE_RANGE   0x0D
#define	SMB_COM_CREATE_TEMPORARY    0x0E
#define	SMB_COM_CREATE_NEW          0x0F
#define	SMB_COM_CHECK_DIRECTORY     0x10
#define	SMB_COM_PROCESS_EXIT        0x11
#define	SMB_COM_SEEK                0x12
#define	SMB_COM_LOCK_AND_READ       0x13
#define	SMB_COM_WRITE_AND_UNLOCK    0x14
#define	SMB_COM_READ_RAW            0x1A
#define	SMB_COM_READ_MPX            0x1B
#define	SMB_COM_READ_MPX_SECONDARY  0x1C
#define	SMB_COM_WRITE_RAW           0x1D
#define	SMB_COM_WRITE_MPX           0x1E
#define	SMB_COM_WRITE_COMPLETE      0x20
#define	SMB_COM_SET_INFORMATION2    0x22
#define	SMB_COM_QUERY_INFORMATION2  0x23
#define	SMB_COM_LOCKING_ANDX        0x24
#define	SMB_COM_TRANSACTION         0x25
#define	SMB_COM_TRANSACTION_SECONDARY 0x26
#define	SMB_COM_IOCTL               0x27
#define	SMB_COM_IOCTL_SECONDARY     0x28
#define	SMB_COM_COPY                0x29
#define	SMB_COM_MOVE                0x2A
#define	SMB_COM_ECHO                0x2B
#define	SMB_COM_WRITE_AND_CLOSE     0x2C
#define	SMB_COM_OPEN_ANDX           0x2D
#define	SMB_COM_READ_ANDX           0x2E
#define	SMB_COM_WRITE_ANDX          0x2F
#define	SMB_COM_CLOSE_AND_TREE_DISC 0x31
#define	SMB_COM_TRANSACTION2        0x32
#define	SMB_COM_TRANSACTION2_SECONDARY 0x33
#define	SMB_COM_FIND_CLOSE2         0x34
#define	SMB_COM_FIND_NOTIFY_CLOSE   0x35
#define	SMB_COM_TREE_CONNECT        0x70
#define	SMB_COM_TREE_DISCONNECT     0x71
#define	SMB_COM_NEGOTIATE           0x72
#define	SMB_COM_SESSION_SETUP_ANDX  0x73
#define	SMB_COM_LOGOFF_ANDX         0x74
#define	SMB_COM_TREE_CONNECT_ANDX   0x75
#define	SMB_COM_QUERY_INFORMATION_DISK 0x80
#define	SMB_COM_SEARCH              0x81
#define	SMB_COM_FIND                0x82
#define	SMB_COM_FIND_UNIQUE         0x83
#define	SMB_COM_NT_TRANSACT         0xA0
#define	SMB_COM_NT_TRANSACT_SECONDARY 0xA1
#define	SMB_COM_NT_CREATE_ANDX      0xA2
#define	SMB_COM_NT_CANCEL           0xA4
#define	SMB_COM_OPEN_PRINT_FILE     0xC0
#define	SMB_COM_WRITE_PRINT_FILE    0xC1
#define	SMB_COM_CLOSE_PRINT_FILE    0xC2
#define	SMB_COM_GET_PRINT_QUEUE     0xC3


/* ------------------------------------------------------------------------- */

/* trans2 subcommand codes */
#define	TRANS2_OPEN2                    0x00
#define	TRANS2_FIND_FIRST2              0x01
#define	TRANS2_FIND_NEXT2               0x02
#define	TRANS2_QUERY_FS_INFORMATION     0x03
#define	TRANS2_QUERY_PATH_INFORMATION   0x05
#define	TRANS2_SET_PATH_INFORMATION     0x06
#define	TRANS2_QUERY_FILE_INFORMATION   0x07
#define	TRANS2_SET_FILE_INFORMATION     0x08
#define	TRANS2_FSCTL                    0x09
#define	TRANS2_IOCTL2                   0x0A
#define	TRANS2_FIND_NOTIFY_FIRST        0x0B
#define	TRANS2_FIND_NOTIFY_NEXT         0x0C
#define	TRANS2_CREATE_DIRECTORY         0x0D
#define	TRANS2_SESSION_SETUP            0x0E
#define	TRANS2_GET_DFS_REFERRAL         0x10
#define	TRANS2_REPORT_DFS_INCONSISTENCY 0x11

/* ------------------------------------------------------------------------- */

/* RAP subcommand codes */
#define	RAP_NET_SHARE_ENUM				0
#define	RAP_NET_SERVER_ENUM				104

/* RAP status codes */
#define RAP_OK							0
#define RAP_ERR_MORE_DATA				234

/* RAP_NET_SERVER_ENUM enumeration options: */
#define	RAP_SENUM_TYPE_SERVER			2
#define	RAP_SENUM_TYPE_DOMAIN_ENUM		0x80000000

/* ------------------------------------------------------------------------- */

/* nt transact subcommand codes */
#define	NT_TRANSACT_CREATE                 1
#define	NT_TRANSACT_IOCTL                  2
#define	NT_TRANSACT_SET_SECURITY_DESC      3
#define	NT_TRANSACT_NOTIFY_CHANGE          4
#define	NT_TRANSACT_RENAME                 5
#define	NT_TRANSACT_QUERY_SECURITY_DESC    6

/* ------------------------------------------------------------------------- */

#endif	/* __smbnumbers_H_INCLUDED__ */
