/*
 * Name: statemachine.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-04-15
 * Tabsize: 4
 * Copyright: (c) 1997, 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: statemachine.h,v 1.1.1.1 1999/04/12 13:56:35 cs Exp $
 */

#ifndef __STATEMACHINE_H_INCLUDED__
#define __STATEMACHINE_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This header defines macros that simplify the implementation of statemachines.
See "smb.c" for an example usage.
*/

/* ------------------------------------------------------------------------- */

/*
 * This version of a state machine is intended for implementing one of these
 * circle and line drawings directly in code.
 */

#define	SM_BEGIN_STATEMACHINE(do_on_error)							\
	{int _smDesiredState = (SM_STATE);								\
	char _smJustEnteredState = 0;									\
		do{															\
			switch(SM_STATE){										\
			default: do_on_error;

#define	SM_DEFINE_STATE(defined_state)								\
				goto _smBreakSwitch;								\
			case defined_state:

#define	SM_ENTRY_CODE												\
				if(_smJustEnteredState){

#define	SM_EXIT_CODE												\
					_smJustEnteredState=0;							\
					goto _smBreakSwitch;							\
				}else if(_smDesiredState!=(SM_STATE)){

#define	SM_TRANSITIONS												\
					(SM_STATE) = _smDesiredState;					\
					_smJustEnteredState = 1;						\
					goto _smBreakSwitch;							\
				}

#define	SM_GOTO(_sm_newstate)										\
				do{_smDesiredState = _sm_newstate;					\
					_smJustEnteredState=0;							\
					goto _smBreakSwitch;							\
				}while(0)

#define	SM_END_STATEMACHINE(changedVar)								\
			} _smBreakSwitch:										\
			changedVar |= _smDesiredState!=(SM_STATE);				\
		}while((SM_STATE)!=_smDesiredState || _smJustEnteredState);	\
	}

/* ------------------------------------------------------------------------- */

#endif /* __STATEMACHINE_H_INCLUDED__ */
