/*
 * Name: uirep.h
 * Project: Sharity
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1999-02-13
 * Tabsize: 4
 * Copyright: (c) 1999 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: uirep.h,v 1.3 2001/05/05 22:27:49 cs Exp $
 */

#ifndef __uirep_H_INCLUDED__
#define __uirep_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This module implements the callbacks to the user interface applications.
It first should try the specific CIFS methods to read passwords and pass
phrases and, failing that, use generic GUI methods to bring up a dialog.
*/

/* ------------------------------------------------------------------------- */

typedef struct uirepAuthData{
	btu8	*username;
	btu8	*passwd;
	btu8	*domain;
	btbool	storePasswd;
	btbool	storeAsDefault;
	btbool	allowUnencrypted;
}uirepAuthData_t;

typedef struct uirepRes{
	btError	err;
	struct ipcSrvClient	*client;	/* the client we worked with */
	btbool	isOk;					/* user clicked OK, not cancel */
}uirepRes_t;

/* ------------------------------------------------------------------------- */

void	uirepGetPassPhrase(btint uid, btu8 *srvName, uirepAuthData_t *auth, void *caller);
/* This function runs a user dialog for the user with local Unix user ID
 * 'uid' and asks for an SSL PEM pass phrase.
 * Parameters: 'uid' is the user that should be asked for the pass phrase,
 *    'srvName' is the name of the server we try to connect to, 'auth' is
 *    the destination for authentication results and 'caller' is the
 *    caller's context for callback.
 *    The caller is responsible for freeing all strings in the 'auth'
 *    structure!
 * Callback: uirepRes_t with fields 'err' and 'isOk' set. 'client' points
 *    to the UI client we talked to.
 */
void	uirepGetCredentials(btint uid, btu8 *host, uirepAuthData_t *auth, void *caller);
/* This function runs a user dialog for user 'uid' and asks for the credentials
 * on a remote machine.
 * Parameters: 'uid' is the user that should be asked for credentials, 'host'
 *    is the name of the machine where the user should be logged in, 'auth'
 *    determines the defaults presented to the user and contains the results
 *    upon callback and 'caller' is the caller's context.
 *    The caller is responsible for freeing all strings in the 'auth'
 *    structure!
 * Callback: uirepRes_t with the fields 'err' and 'isOk' set. 'client' points
 *    to the UI client we talked to.
 */
void	uirepGetSharePassword(btint uid, btuni *share, uirepAuthData_t *auth, void *caller);
/* This function runs a user dialog for user 'uid' and asks for the share
 * password of a share.
 * Parameters: 'uid' is the user that should be asked for the password, 'share'
 *    is the name of the share for which the password should be asked, 'auth'
 *    is the destination for the password obtained from the user and
 *    'caller' is the caller's context.
 *    The caller is responsible for freeing all strings in the 'auth'
 *    structure!
 * Callback: uirepRes_t with the fields 'err' and 'isOk' set. 'client'
 *    points to the UI client we talked to.
 */

/* ------------------------------------------------------------------------- */

#endif /* __uirep_H_INCLUDED__ */
