/*
 * Name: blockpool.c
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-11-05
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: blockpool.c,v 1.1.1.1 1999/04/12 13:56:37 cs Exp $
 */


#include "stdheaders.h"
#include "blockpool.h"
#include "exception.h"

/* ------------------------------------------------------------------------- */

collectionObjectI_t	poolCollectionInterface = {
    NULL,
    NULL,
    NULL,
    NULL
};

/* ------------------------------------------------------------------------- */

void	poolFree(blockpool_t *this)
{
int	i, l;
    
    if(this != NULL){
        if(this->freeBlocks != NULL){
            l = arrObjectCount(this->freeBlocks);
            for(i=0;i<l;i++){
                memFree(arrObjectAtIndex(this->freeBlocks, i));
            }
            arrFree(this->freeBlocks);
        }
        memFree(this);
    }
}

blockpool_t	*poolNew(btsize eltSize, btsize volatile clearSize)
{
blockpool_t	*volatile this = NULL;

EXC_TRY
    if(clearSize > eltSize)
        clearSize = eltSize;
    this = memCalloc(sizeof(blockpool_t));
    this->eltSize = eltSize;
    this->clearSize = clearSize;
    this->freeBlocks = arrNew(&poolCollectionInterface, 16);
EXC_CATCH
	poolFree(this);
	EXC_RETHROW();
EXC_ENDTRY
    return this;
}

/* ------------------------------------------------------------------------- */

