/*
 * Name: errcodes.h
 * Project: Sharity
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1999-02-10
 * Tabsize: 4
 * Copyright: (c) 1999 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: errcodes.h,v 1.8 2000/01/21 12:25:50 cs Exp $
 */

#ifndef __errcodes_H_INCLUDED__
#define __errcodes_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This module defines all error codes used throughout the project and a function
to translate error codes to verbose error messages. Having fixed error codes
within the foundation module contradicts modularity somewhat, but it is the
simplest solution. A later version of the framework may come up with a better
solution.
*/

/* ------------------------------------------------------------------------- */

btu8	*errDescription(btError *err);

/* ------------------------------------------------------------------------- */

/* error classes */
#define	ERRCLS_OK				0
#define	ERRCLS_CIFS_DOS			1
#define	ERRCLS_CIFS_SRV			2
#define	ERRCLS_CIFS_HARD		3

/* the following errors are outside the range of CIFS specified errors: */
#define	ERRCLS_FRAMEWORK		36250
#define	ERRCLS_CIFS_LOCAL		36251
#define	ERRCLS_UNIX				36252

/* ------------------------------------------------------------------------- */

#define	CIFS_ERR_SUCCESS		0

/* ------------------------------------------------------------------------- */

/* dos error codes */
#define	ERR_CIFS_DOSbadfunc      1
#define	ERR_CIFS_DOSbadfile      2
#define	ERR_CIFS_DOSbadpath      3
#define	ERR_CIFS_DOSnofids       4
#define	ERR_CIFS_DOSnoaccess     5
#define	ERR_CIFS_DOSbadfid       6
#define	ERR_CIFS_DOSbadmcb       7
#define	ERR_CIFS_DOSnomem        8
#define	ERR_CIFS_DOSbadmem       9
#define	ERR_CIFS_DOSbadenv       10
#define	ERR_CIFS_DOSbadformat    11
#define	ERR_CIFS_DOSbadaccess    12
#define	ERR_CIFS_DOSbaddata      13
#define	ERR_CIFS_DOSbaddrive     15
#define	ERR_CIFS_DOSremcd        16
#define	ERR_CIFS_DOSdiffdevice   17
#define	ERR_CIFS_DOSnofiles      18
#define	ERR_CIFS_DOSbadshare     32
#define	ERR_CIFS_DOSlock         33
#define ERR_CIFS_DOSunsup		 50	/* request unsupp. (according to samba) */
#define	ERR_CIFS_DOSnotshared	 67
#define	ERR_CIFS_DOSfilexists    80
#define ERR_CIFS_DOScannotopen	 110 /* cannot open the file specified */
#define ERR_CIFS_DOSunknownlevel 124
#define	ERR_CIFS_DOSnotempty	 145	/* dir not empty */
#define	ERR_CIFS_DOSdestexists	 183	/* rename: destination exists */
#define ERR_CIFS_DOSbadpipe		 230
#define ERR_CIFS_DOSpipebusy	 231
#define ERR_CIFS_DOSpipeclosing	 232	/* named pipe close in progress */
#define ERR_CIFS_DOSnotconnected 233	/* no process on other end of named pipe */
#define ERR_CIFS_DOSmoredata	 234
#define ERR_CIFS_DOSbaddirectory 267
#define ERR_CIFS_DOSeasdidntfit	 275
#define ERR_CIFS_DOSeasnotsupp	 282
#define ERR_CIFS_DOSnotifyenumdir 1022 /* buffer too small for change notify. */

/* ------------------------------------------------------------------------- */

/* server error codes */
#define	ERR_CIFS_SRVerror        1
#define	ERR_CIFS_SRVbadpw        2
#define	ERR_CIFS_SRVaccess       4
#define	ERR_CIFS_SRVinvnid       5
#define	ERR_CIFS_SRVinvnetname   6
#define	ERR_CIFS_SRVinvdevice    7
#define ERR_CIFS_SRVunknownsmb	 22 /* according to samba */
#define	ERR_CIFS_SRVqfull        49
#define	ERR_CIFS_SRVqtoobig      50
#define	ERR_CIFS_SRVqeof         51
#define	ERR_CIFS_SRVinvpfid      52
#define	ERR_CIFS_SRVsmbcmd       64
#define	ERR_CIFS_SRVsrverror     65
#define	ERR_CIFS_SRVfilespecs    67
#define	ERR_CIFS_SRVbadpermits   69
#define	ERR_CIFS_SRVsetattrmode  71
#define	ERR_CIFS_SRVpaused       81
#define	ERR_CIFS_SRVmsgoff       82
#define	ERR_CIFS_SRVnoroom       83
#define	ERR_CIFS_SRVrmuns        87
#define	ERR_CIFS_SRVtimeout      88
#define	ERR_CIFS_SRVnoresource   89
#define	ERR_CIFS_SRVtoomanyuids  90
#define	ERR_CIFS_SRVbaduid       91
#define	ERR_CIFS_SRVusempx       250
#define	ERR_CIFS_SRVusestd       251
#define	ERR_CIFS_SRVcontmpx      252
#define	ERR_CIFS_SRVpwexpired    2242
#define	ERR_CIFS_SRVnosupport    65535

/* ------------------------------------------------------------------------- */

/* hard error codes */
#define	ERR_CIFS_HRDnowrite      19
#define	ERR_CIFS_HRDbadunit      20
#define	ERR_CIFS_HRDnotready     21
#define	ERR_CIFS_HRDbadcmd       22
#define	ERR_CIFS_HRDdata         23
#define	ERR_CIFS_HRDbadreq       24
#define	ERR_CIFS_HRDseek         25
#define	ERR_CIFS_HRDbadmedia     26
#define	ERR_CIFS_HRDbadsector    27
#define	ERR_CIFS_HRDnopaper      28
#define	ERR_CIFS_HRDwrite        29
#define	ERR_CIFS_HRDread         30
#define	ERR_CIFS_HRDgeneral      31
#define	ERR_CIFS_HRDbadshare     32
#define	ERR_CIFS_HRDlock         33
#define	ERR_CIFS_HRDwrongdisk    34
#define	ERR_CIFS_HRDFCBUnavail   35
#define	ERR_CIFS_HRDsharebufexc  36

/* ------------------------------------------------------------------------- */

/* local errors */
#define	ERR_CIFS_LOCconnlost	1	/* connection to server lost */
#define	ERR_CIFS_LOCtimeout		2	/* operation timed out */
#define	ERR_CIFS_LOCbogusdata	3	/* server sent data not according to spec */
#define	ERR_CIFS_LOCstale		4	/* stale NFS file handle */
#define	ERR_CIFS_LOCpwencrypt	5	/* server can't use encrypted passwd */
#define	ERR_CIFS_LOClookup		6	/* netbios name lookup failed */
#define	ERR_CIFS_LOClicclients	7	/* license limit exceeded: clients */
#define	ERR_CIFS_LOClicservers	8	/* license limit exceeded: servers */
#define	ERR_CIFS_LOClicense		9	/* license string is invalid */
#define	ERR_CIFS_LOCsrvconn		10	/* servers connected when setting license */
#define	ERR_CIFS_LOCwrongphrase	11	/* pass phrase not accepted */
#define	ERR_CIFS_LOCneedpassphrase	12	/* SSL needs pass phrase */
#define	ERR_CIFS_LOCnetbiosrefused	13	/* netbios connection refused */
#define	ERR_CIFS_LOChavenossl	14	/* no SSL support compiled in */
#define	ERR_CIFS_LOCnosslserver	15	/* server can't do SSL */
#define	ERR_CIFS_LOCsslerror	16	/* an SSL error occured, see syslog */
#define	ERR_CIFS_LOCnomountdir	17	/* mount area does not exist */
#define	ERR_CIFS_LOCnobrowser	18	/* browser does not exist */
#define	ERR_CIFS_LOCotherlogin	19	/* not owner of login */
#define	ERR_CIFS_LOCsslserver	20	/* server requests SSL */
#define	ERR_CIFS_LOCsrvnotconn	21	/* server not connected */
#define	ERR_CIFS_LOCusrnotconn	22	/* user not connected */
#define	ERR_CIFS_LOCnosharegiven 23	/* no share name given */

/* ------------------------------------------------------------------------- */

/* framework errors: */
#define	ERR_FRWmodulenotfound	1	/* module not found */
#define	ERR_FRWmoduletypeerr	2	/* wrong module type */
#define	ERR_FRWalreadymounted	3	/* already something mounted here */
#define	ERR_FRWipcparameters	4	/* error in IPC parameters */
#define	ERR_FRWipcversion		5	/* IPC version mismatch */
#define	ERR_FRWnomem			6	/* memory allocation error */
#define	ERR_FRWnosuchobj		7	/* GUI dsource object does not exist */
#define	ERR_FRWnosuchmethod		8	/* no such GUI dsource method */
#define	ERR_FRWumountperm		9	/* no permission to unmount */
#define	ERR_FRWclientcomm		10	/* client communication error */
#define	ERR_FRWnosuchmnt		11	/* mountpoint does not exist */
#define	ERR_FRWmpaccessperm		12	/* not owner of mount, operation not permitted */
#define	ERR_FRWautomounted		13	/* operation not perm. on automounted mounts */
#define	ERR_FRWipcnotimpl		14	/* IPC function not implemented */
#define	ERR_FRWusrabort			15	/* aborted by user */
#define	ERR_FRWnotauth			16	/* user not authorized to perform operation */
#define	ERR_FRWnomountpoint		17	/* Mountpoint does not exist */
#define	ERR_FRWstoredisabled	18	/* Storing mounts/logins is disabled */
#define	ERR_FRWnotownerofmp		19	/* Not owner of mount point */

/* ------------------------------------------------------------------------- */

#endif /* __errcodes_H_INCLUDED__ */
