/*
 * Name: hashtable.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-10-24
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: hashtable.h,v 1.1.1.1 1999/04/12 13:56:38 cs Exp $
 */

#ifndef __hashtable_H_INCLUDED__
#define __hashtable_H_INCLUDED__

#include "basictypes.h"
#include "array.h"

/*
General Description:
This module implements a hashtable of objects. There is no separation into
keys and values as it's usual in the implementation of hashtables. Wich part
of the object should be regarded as key and wich as value is defined by the
implementation of the 'isEqualTo()' and 'hashValue()' functions (see "array.h"
for more details). Most of the things said for the array apply to the hastable
accordingly. Objects which should be added to a hashtable must, however,
implement one more interface function: 'hashValue()'. This function must
return a "digest" of the key part of the object. Objects with equal keys
must have the same 'hashValue()' return value.
*/

/* ------------------------------------------------------------------------- */

typedef struct hashNode{
    struct hashNode	*next;
    void			*obj;
}hashNode_t;

typedef struct hashtable{
    collectionObjectI_t	objectInterface;
    hashNode_t			*table;
    btu32				tableSize;
    btu32				objectCount;
    btu32				maxFillPercentage;
}hashtable_t;

hashtable_t	*htabNew(const collectionObjectI_t *contentObjectInterface, btu32 initialSize, btu32 maxFillPercentage);
void		htabFree(hashtable_t *this);
void		htabAddObject(hashtable_t *this, void *obj);
btbool		htabRemoveObjectEqualTo(hashtable_t *this, void *obj);
/* returns '1' if object has been removed, '0' otherwise */
void		*htabFindObjectEqualTo(hashtable_t *this, void *obj);
array_t		*htabAllObjects(hashtable_t *this);

static inline btu32	htabObjectCount(hashtable_t *this)
{
    return this->objectCount;
}


/* ------------------------------------------------------------------------- */

#endif	/* __hashtable_H_INCLUDED__ */
