/*
 * Name: logging.c
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-10-14
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: logging.c,v 1.2 2002/04/05 17:53:49 cs Exp $
 */

#include "stdheaders.h"
#include "logging.h"

/* ------------------------------------------------------------------------- */

bti32	logModeMask = (1 << LOGM_ERROR) | (1 << LOGM_INFO);
int		logDestination = LOG_DEST_CMDLINE;

/* ------------------------------------------------------------------------- */

static void	logVprintf(int mode, const char *format, va_list vlist)
{
static char		buffer[2048];
bti32			sec, sec1000;
struct timeval	tv;
struct timezone	tz;

	vsprintf(buffer, format, vlist);
	if(strlen(buffer) >= sizeof(buffer)){
		if(logDestination == LOG_DEST_SYSLOG){
			syslog(LOG_ERR, "*** Sharity fatal error: logging buffer overflow!\n");
		}else{
			fprintf(stderr, "*** Sharity fatal error: logging buffer overflow!\n");
		}
		exit(1);
	}
	switch(logDestination){
	case LOG_DEST_SYSLOG:
		syslog(LOG_ERR, "[%d] %s", mode, buffer);
		break;
	case LOG_DEST_STDERR:
		gettimeofday(&tv, &tz);
		sec = tv.tv_sec % 1000;
		sec1000 = tv.tv_usec / 1000U;
		fprintf(stderr, "%03d.%03d [%d] %s", sec, sec1000, mode, buffer);
		fflush(stderr);
		fsync(fileno(stderr));
		break;
	case LOG_DEST_CMDLINE:
		fprintf(stderr, "%s", buffer);
		fflush(stderr);
		fsync(fileno(stderr));
		break;
	}
}

/* ------------------------------------------------------------------------- */

void	logPrintf(int mode, const char *format, ...)
{
va_list		vlist;

	va_start(vlist, format);
	logVprintf(mode, format, vlist);
	va_end(vlist);
}

/* ------------------------------------------------------------------------- */

