/*
 * Name: logging.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-10-14
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: logging.h,v 1.4 2001/10/13 12:37:24 cs Exp $
 */

#ifndef __LOGGING_H_INCLUDED__
#define __LOGGING_H_INCLUDED__

#include "basictypes.h"
#include "exception.h"

/*
General Description:
This module implements logging of informational, debug and error messages
to various facilities. Logging can currently go to stdout or to the
syslog. The logged messages may be enabled or disabled in a non-hierarchic
way by means of the bitmap 'logModeMask'. Use the macros 'LOG_SET_MODE()'
and 'LOG_CLEAR_MODE()' to enable or disable specific messages. If you are
a user: 'logModeMask' is initialized from the configuration file. You need
not modify the source to change the logging behaviour.
*/

/* ------------------------------------------------------------------------- */

/* the following defines must match the 'loglevels' array in main.c */
#define	LOGM_INFO			0	/* informational messages */
#define	LOGM_ERROR			1	/* general error */
#define	LOGM_DEBUG			2
#define	LOGM_RESOURCE		3	/* resource allocation */
#define	LOGM_NETBIOSERR		4	/* errors in netbios layer */
#define	LOGM_NETBIOSDEB		5	/* debug infos from netbios layer */
#define	LOGM_NETBTRACE		6	/* rx and tx message traces */
#define	LOGM_NFSTRACE		7	/* nfs command traces */
#define	LOGM_RARE			8	/* things that should happen infrequently */
#define	LOGM_CACHEDEB		9	/* debug messages from cache code */
#define	LOGM_CIFSTRACE		10	/* trace about cifs RPCs */
#define	LOGM_OPLOCK			11	/* debug oplocks */
#define	LOGM_WARN			12	/* warnings about config file */
#define	LOGM_SERVER_SMB		13	/* SMB sequence number arbitration */
#define	LOGM_DEBUG_ATTR		14	/* attribute modification code debugging */
#define	LOGM_DEBUG_SSL		15	/* SSL debugging */
#define	LOGM_SMB_CONNECT	16	/* connect/disconnect messages for netbios */
#define	LOGM_UITRACE		17	/* ui socket communication trace */
#define	LOGM_NBNS			18	/* netbios name service trace */
#define	LOGM_DISKARB		19	/* netbios name service trace */


#define	LOG_MODE_IS_ACTIVE(mode)	((logModeMask & (1<<(mode))) != 0)
#define	LOG_SET_MODE(mode)			(logModeMask |= 1<<(mode))
#define	LOG_CLEAR_MODE(mode)		(logModeMask &= ~(1<<(mode)))

#define	LOG_DEST_STDERR		0
#define	LOG_DEST_SYSLOG		1
#define	LOG_DEST_CMDLINE	2	/* same as stderr, but without any prefix */

/* ------------------------------------------------------------------------- */

extern bti32	logModeMask;
/* This global variable holds a bitmaks representing which topics should be
 * logged.
 */
/* ------------------------------------------------------------------------- */
extern int		logDestination;
/* This global variable determines the destination of logging information. It
 * may be either 'LOG_DEST_STDERR' or 'LOG_DEST_SYSLOG'.
 */
/* ------------------------------------------------------------------------- */
void	logPrintf(int mode, const char *format, ...);
/* This printf()-like function writes the string 'format' after replacing
 * all escape sequences (starting with '%') to the current logging facility.
 * The string may be prefixed with the logging mode and time and date.
 */

/* ------------------------------------------------------------------------- */

#endif	/* __LOGGING_H_INCLUDED__ */
