/*
 * Name: memalloc.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-10-12
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: memalloc.h,v 1.1.1.1 1999/04/12 13:56:38 cs Exp $
 */

#ifndef __MEMALLOC_H_INCLUDED__
#define __MEMALLOC_H_INCLUDED__

#include "basictypes.h"
#include "exception.h"

/*
General Description:
A library of memory allocation functions used throughout the project instead
of the malloc library. It uses exception handling to simplify error handling.
*/

/* ------------------------------ exceptions ------------------------------- */

EXC_DECLARE(memExcNomem);
/* This exception is raised if the requested amount of memory is not
 * available.
 */

/* -------------------------- function prototypes -------------------------- */

void	*memAlloc(btsize size);
/* This function is a kind of malloc() with exception handling. It is used
 * throughout this project instead of malloc()/calloc().
 */
/* ------------------------------------------------------------------------- */
void	*memCalloc(btsize size);
/* This function is equivalent to memAlloc() except that it sets the allocated
 * storage to zero.
 */
/* ------------------------------------------------------------------------- */
void	*memRealloc(void *oldbase, btsize newsize);
/* This function is a kind of realloc() with exception handling. It's used
 * throughout this project instead of realloc().
 */
/* ------------------------------------------------------------------------- */
void	memFree(void *base);
/* This function is equivalent to free(). It complements memAlloc() and
 * memRealloc() in this module.
 */

/* ------------------------------------------------------------------------- */

#endif	/* __MEMALLOC_H_INCLUDED__ */
