/*
 * Name: system.c
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-10-16
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: system.c,v 1.37 2002/05/06 15:06:12 cs Exp $
 */
 
/*
Remarks:
This file is there to cope with the real world: things that are different
on various unix systems. Therefore it behaves like real world items do:
It's a real mess and tends to get out of control. But that is the price
for isolating the rest of the code from the Evil Real World :-)
*/

#if defined __osf__
#define	_SOCKADDR_LEN	/* This looks like a hack and it probably is... */
#endif

#define	NFSCLIENT
#include "stdheaders.h"
#include "system.h"
#include "logging.h"
#include "util.h"
#include "checks/checks.h"

/* ------------------------------------------------------------------------- */
/* ----------------------------- sysStrerror() ----------------------------- */
/* ------------------------------------------------------------------------- */

#if HAVE_SYS_ERRLIST
char	*sysStrerror(int errcode)
{
static char	numericOutput[64];

	if(errcode < 0 || errcode >= sys_nerr || sys_errlist[errcode] == NULL){
		sprintf(numericOutput, "Error %d", errcode);
		return numericOutput;
	}else{
		return (char *)sys_errlist[errcode];	/* discard possible const */
	}
}
#else /* no sys_errlist global array */
/* UnixWare requires error text is accessed via a function. Allows proper
 * language support...
 */
char	*sysStrerror(int errcode)
{
	return (char *)strerror(errcode);       /* discard possible const */
}
#endif /* UNIXWARE */

/* ------------------------------------------------------------------------- */
/* --------------------------- sysSetNonblock() ---------------------------- */
/* ------------------------------------------------------------------------- */

int	sysSetNonblock(int fd)
{
#if defined SOLARIS2 || defined UNIXWARE
	return fcntl(fd, F_SETFL, O_NDELAY | O_NONBLOCK);
#else
	return fcntl(fd, F_SETFL, FNDELAY);
#endif
}

/* ------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------- */

static char	*sysGetwd(char *buffer)
{
#if defined NeXT || defined SUNOS
	return getwd(buffer);
#else
	return getcwd(buffer, MAXPATHLEN - 1);
#endif
}

/* ------------------------------------------------------------------------- */
/* ------------------------- sysSetCorefileLimit() ------------------------- */
/* ------------------------------------------------------------------------- */

void	sysSetCorefileLimit(btint limit)
{
struct rlimit	rlim;

	if(getrlimit(RLIMIT_CORE, &rlim) == -1){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "system: can\'t enable corefile: getrlimit: [%d] %s\n", errno, sysStrerror(errno));
		return;
	}
	if(limit > rlim.rlim_max){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "system: requested corefile limit %d too big, limiting to %d\n", limit, rlim.rlim_max);
		limit = rlim.rlim_max;
	}
	rlim.rlim_cur = limit;
	if(setrlimit(RLIMIT_CORE, &rlim) == -1){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "system: can\'t enable corefile: setrlimit: [%d] %s\n", errno, sysStrerror(errno));
	}
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- sysDetach() ----------------------------- */
/* ------------------------------------------------------------------------- */

void	sysDetach(void)
{
btint	rval, i;

/*	chdir("/"); do not chdir -- startup script can determine dir for corefiles */
	if((rval = fork()) != 0){	/* we are the parent */
		if(rval == -1)
			perror("fork");
		_exit(0);
	}
	for(i=0;i<3;i++)
		close(i);		/* detach from terminal */
	/* they say it might be a good idea to have fds 0 to 2: */
	for(i=0;i<3;i++){
		if(open("/dev/null", O_WRONLY, 0) != i)
			break;	/* something went wrong, stop opening fds */
	}
#if defined NeXT
	setpgrp(0, getpid());
#elif defined SUNOS
	setpgrp(getpid(), 0);
#else
	setsid();
#endif
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- sysRealpath() ----------------------------- */
/* ------------------------------------------------------------------------- */

#if HAVE_REALPATH
char	*sysRealpath(char *path, char *real)
{
char	pathdir[MAXPATHLEN], *p, *q;
int		len;
/*
 * Implementations of realpath() may not do what we desire. It's a good
 * idea to prepend the current directory to relative pathes.
 */
	if(path[0] != '/'){	/* not an absolute path */
		if(sysGetwd(pathdir) == NULL){
			return NULL;
		}
		len = strlen(pathdir);
		if(len + strlen(path) + 2 > MAXPATHLEN){
			errno = ENAMETOOLONG;
			return NULL;
		}
		if(pathdir[len-1] != '/'){
			pathdir[len++] = '/';	/* ensure trailing slash */
		}
		strcpy(pathdir + len, path);
	}else{
		strncpy(pathdir, path, MAXPATHLEN);
		pathdir[MAXPATHLEN - 1] = 0;
	}
	if(realpath(pathdir, real) != NULL){
		return real;
	}
	p = strrchr(pathdir, '/');
	if(p != NULL){
		*p++ = 0;	/* try it without last path component */
		if(realpath(pathdir, real) != NULL){
			q = real + strlen(real);
			if(q - real < MAXPATHLEN){
				*q++ = '/';
				strncpy(q, p, MAXPATHLEN - (q - real));
			}
			return real;
		}
	}
	return NULL;
}
#else
/* this is not the real thing, but a sufficient dummy implementation for OSF */
char	*sysRealpath(char *path, char *real)
{
btsize	len;
char	*p = real;

	len = strlen(path);
	if(path[len-1] == '/')	/* don't need trailing slashes */
		len--;
	if(path[0] != '/'){	/* relative path */
		if(sysGetwd(p) == NULL)
			return NULL;
		p += strlen(p);
		if(p[-1] != '/')
			*p++ = '/';		/* ensure slash */
	}
	if(p - real + len >= MAXPATHLEN)
		len = MAXPATHLEN - 1 - (p - real);
	if(len > 0){
		memcpy(p, path, len);
		p += len;
	}
	*p++ = 0;
	return real;
}
#endif

/* ------------------------------------------------------------------------- */
/* --------------------------- sysNonblockWait() --------------------------- */
/* ------------------------------------------------------------------------- */

#if defined NeXT 
int	sysNonblockWait(int *rval, btint waitForPid)
{
union wait	status;
int			pid;

	memset(&status, 0, sizeof(status));
    pid = wait4(waitForPid, &status, WNOHANG, NULL);
    *rval = status.w_retcode;
    if(pid <= 0)
        return 0;
    return pid;
}
#else /* NeXT */
int	sysNonblockWait(int *rval, btint waitForPid)
{
int		status = 0;
int		pid;

    pid = waitpid(waitForPid, &status, WNOHANG);
    *rval = WEXITSTATUS(status);
    if(pid <= 0)
        return 0;
    return pid;
}
#endif /* NeXT */

/* ------------------------------------------------------------------------- */
/* ----------------------------- sysGetPass() ------------------------------ */
/* ------------------------------------------------------------------------- */

#if HAVE_TERMIOS
#include <termios.h>
/* termios version */
static void	setEcho(int fd, btbool isOn)
{
struct termios			ttyset;
static struct termios	ttystore;

	if(!isOn){
		tcgetattr(fd, &ttyset);
		memcpy(&ttystore, &ttyset, sizeof(struct termios));
		ttyset.c_lflag &= ~(ECHO|ISIG);
		tcsetattr(fd, TCSANOW, &ttyset);
	}else{
		tcsetattr(fd, TCSANOW, &ttystore);	/* restore original settings */
	}
}
#define	ECHO_ONOFF_DEFINED
#elif HAVE_SGTTY
#include <sgtty.h>
/* sgtty version */
static void	setEcho(int fd, btbool isOn)
{
struct sgttyb			ttyset;
static struct sgttyb	ttystore;

	if(!isOn){
		ioctl(fd, TIOCGETP, &ttyset);
		memcpy(&ttystore, &ttyset, sizeof(struct sgttyb));
		ttyset.sg_flags &= ~ECHO;
		ttyset.sg_flags |= CBREAK;
		ioctl(fd, TIOCSETP, &ttyset);
	}else{
		ioctl(fd, TIOCSETP, &ttystore);	/* restore original settings */
	}
}
#define	ECHO_ONOFF_DEFINED
#endif
/* ------------------------------------------------------------------------- */

#ifdef ECHO_ONOFF_DEFINED

static char	sysGetChar(int fd)
{
char	c = 0;

	if(read(fd, &c, 1) != 1)
		return 0;
	return c;
}

char *sysGetPass(char *prompt, char *buffer, int bufferLen)
{
int		fd;
char	c, *p;

	fd = open("/dev/tty", O_RDWR);	/* read from tty, whatever stdin may be */
	if(fd < 0)	/* if /dev/tty can't be opened, use stdin */
		fd = 0;
	setEcho(fd, 0);
	fflush(stdout);
	fprintf(stderr, "%s", prompt);
	fflush(stderr);
	p = buffer;
	for(;;){
		c = sysGetChar(fd);
		if(c == 3 || c == 4){	/* ^C or ^D */
			p = NULL;
			break;
		}
		if(c == '\r' || c == '\n' || c == 0)	/* end of line */
			break;
		if(c == 8 || c == 0x7f){	/* backspace or delete */
			if(p > buffer)
				p--;
		}
		if(p - buffer < bufferLen - 1)
			*p++ = c;
	}
	fprintf(stderr,"\r\n");
	fflush(stderr);
	setEcho(fd, 1);
	if(fd != 0)
		close(fd);
	if(p == NULL)
		return NULL;
	*p = 0;
	return buffer;
}

#else

/* Generic version limited to getpass() capabilities. Use this if you can't
 * get one of the previous versions to compile.
 */
char	*sysGetPass(char *prompt, char *buffer, int bufferLen)
{
char	*pw;

	pw = getpass(prompt);
	if(pw != NULL && *pw == 0)
		return NULL;
	if(pw != NULL){
		strncpy(buffer, pw, bufferLen);
		pw = buffer;
	}
	return pw;
}
#endif

/* ------------------------------------------------------------------------- */
/* ----------------------- virtual memory allocation ----------------------- */
/* ------------------------------------------------------------------------- */

/* sysValloc() is expected to return "virtual" memory. It is used with
 * rather large arguments (in the 1MB range). The result should be
 * page-aligned. If valloc() is not present, we allocate it using malloc().
 * Sharity does not rely on the allocated area to be zero-filled.
 */

#if HAVE_VALLOC

void	*sysValloc(int bytes)
{
void	*base;

	base = valloc(bytes);
	return base;
}

void	sysVfree(void *base)
{
	free(base);
}

#else

#define	SYS_PAGESIZE	4096	/* the system's page size */

void	*sysValloc(int bytes)
{
btu8	*base = malloc(bytes + SYS_PAGESIZE);
void	**aligned;

	if(base == NULL)
		return NULL;
	aligned = (void **)((int)(base + SYS_PAGESIZE) & ~(SYS_PAGESIZE - 1));
/* the memory position before 'aligned' must be free for at least one pointer
 * because malloc() returns at least pointer-aligned memory.
 */
	aligned[-1] = base;	/* store the original base for free() */
	return (void *)aligned;
}

void	sysVfree(void *base)
{
void	**aligned = (void **)base;

	free(aligned[-1]);	/* that's where the original base was stored */
}

#endif

/* ------------------------------------------------------------------------- */
/* ------------------ getting list of network interfaces ------------------- */
/* ------------------------------------------------------------------------- */

#include <net/if.h>		/* struct ifreq, struct ifconf */
#if HAVE_SOCKIO_H
#include <sys/sockio.h>	/* socket ioctl()s */
#endif

#ifdef _SIZEOF_ADDR_IFREQ	/* if the OS has a function, use it */
#	define	IFREQ_ENTRY_SIZE(ifr)	_SIZEOF_ADDR_IFREQ(ifr)
#elif HAVE_SOCKADDR_SA_LEN
#	define IFREQ_ENTRY_SIZE(ifr)	((ifr).ifr_addr.sa_len > sizeof(struct sockaddr) ? \
	 (sizeof(ifr) - sizeof(struct sockaddr) + (ifr).ifr_addr.sa_len) : sizeof(struct ifreq))
#else	/* if we don't have socket address length, use structure size */
#	define	IFREQ_ENTRY_SIZE(ifr)	sizeof(struct sockaddr)
#endif

btint	sysNetworkInterfaces(sysNetIf_t *ifArray, btint maxEntries)
{
struct ifconf	ifconf;
btu8			buffer[4096], *errSpec;
btint			fd, i, j, nextIf, entryLen;
struct ifreq	*ifp;

	if((fd = socket(AF_INET, SOCK_DGRAM, 0))< 0){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "sysNetworkInterfaces: socket(): [%d] %s\n", errno, sysStrerror(errno));
		return 0;
	}
	ifconf.ifc_len = sizeof(buffer);
	ifconf.ifc_buf = buffer;
	if(ioctl(fd, SIOCGIFCONF, &ifconf) != 0){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "sysNetworkInterfaces: ioctl(SIOCGIFCONF): [%d] %s\n", errno, sysStrerror(errno));
		close(fd);
		return 0;
	}
	for(i=nextIf=0;i<ifconf.ifc_len;i+=entryLen){
		ifp = (struct ifreq *)(buffer + i);
		entryLen = IFREQ_ENTRY_SIZE(*ifp);
		for(j=0;j<nextIf;j++){
			if(strcmp(ifArray[j].name, ifp->ifr_name) == 0)
				break;		/* we already have this interface */
		}
		if(j >= nextIf){	/* not break -> new interface */
			errSpec = "SIOCGIFFLAGS";
			if(ioctl(fd, SIOCGIFFLAGS, ifp) != 0)
				goto errorOccured;
			if((~ifp->ifr_flags & (IFF_UP|IFF_BROADCAST)) != 0)	/* IF must be capable of broadcasts */
				continue;
			errSpec = "SIOCGIFADDR";
			if(ioctl(fd, SIOCGIFADDR, ifp) != 0)
				goto errorOccured;
			strncpy(ifArray[nextIf].name, ifp->ifr_name, sizeof(ifArray->name) - 1);
			ifArray[nextIf].name[sizeof(ifArray->name) - 1] = 0;	/* force 0-termination of string */
			ifArray[nextIf].ifAddr = ntohl(((struct sockaddr_in *)&ifp->ifr_addr)->sin_addr.s_addr);
			errSpec = "SIOCGIFNETMASK";
			if(ioctl(fd, SIOCGIFNETMASK, ifp) != 0)
				goto errorOccured;
			ifArray[nextIf].ifNetmask = ntohl(((struct sockaddr_in *)&ifp->ifr_addr)->sin_addr.s_addr);
			ifArray[nextIf].ifBroadcast = ifArray[nextIf].ifAddr | ~ifArray[nextIf].ifNetmask;
			if(++nextIf < maxEntries)
				continue;
			break;
errorOccured:
			if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))		/* print errors, IF name must be valid */
				logPrintf(LOGM_ERROR, "sysNetworkInterfaces: ioctl(%s, %s): [%d] %s\n", ifp->ifr_name, errSpec, errno, sysStrerror(errno));
		}
	}
	close(fd);
	return nextIf;
}

/* ------------------------------------------------------------------------- */
/* ---------------------- dynamic loading and linking ---------------------- */
/* ------------------------------------------------------------------------- */

/* We'll want the following implementation for Linux and probably Solaris.
 * Don't use it now because it requires an additional library and the
 * -rdynamic flag to the linker which makes Makefile.config even more
 * platform dependent. There's currently no requirement for dynamically linked
 * modules.
 */
#if HAVE_DLOPEN
#include <dlfcn.h>

btu8	*sysLoadModule(btu8 *modulePath)
{
void	*dlhandle;

	dlhandle = dlopen(modulePath, RTLD_NOW);
	if(dlhandle == NULL)
		return dlerror();
	return NULL;
}

#else	/* the dummy implementation: */

btu8	*sysLoadModule(btu8 *modulePath)
{
	return "Dynamic loading not implemented";
}

#endif

/* ------------------------------------------------------------------------- */
/* ------------------ utility functions for mtab handling ------------------ */
/* ------------------------------------------------------------------------- */

#if defined NeXT || defined linux || defined HP || defined __sgi || (defined SOLARIS2 && !defined SOLARIS8) || defined SUNOS || defined UNIXWARE

#if defined SOLARIS2 || defined UNIXWARE
#	include <sys/mnttab.h>
#	define MTAB_LOCK	"/etc/.mnttab.lock"
#	define MTAB_FILE	MNTTAB
/* some translations for Solaris name conventions: */
#	define mntent		mnttab
#	define setmntent	fopen
#	define endmntent	fclose
#	define addmntent	putmntent
	static struct mnttab		staticMntent;
#	define readmntent(fp)	(getmntent(fp, &staticMntent) == 0 ? &staticMntent : NULL)
#	define mnt_fsname	mnt_special
#	define mnt_dir		mnt_mountp
#	define mnt_type		mnt_fstype
#	define mnt_opts		mnt_mntopts
#else
#	include <mntent.h>
#	define	readmntent	getmntent
#	define MTAB_FILE	MOUNTED
#	if defined linux
#		define MTAB_LOCK	MTAB_FILE "~"
#	endif
#endif

#define	MTAB_TMP	(MTAB_FILE ".tmp")


#if defined linux

#define MTAB_LOCK_LINK	MTAB_LOCK "~"

static btbool	mtabSetLock(btbool state)
{
btint			retryCount = 3, fd, storedErrno, rval;
struct flock	fl;
static btbool	weLockedMtab = 0;

	if(state){
		for(;;){
			fd = open(MTAB_LOCK_LINK, O_WRONLY|O_CREAT, 0);
			if(fd < 0)
				return 0;
			close(fd);
			rval = link(MTAB_LOCK_LINK, MTAB_LOCK);
			storedErrno = errno;
			unlink(MTAB_LOCK_LINK);
			if(rval < 0 && storedErrno != EEXIST)
				return 0;
			fd = open(MTAB_LOCK, O_WRONLY);
			if(fd < 0){
				if(errno == ENOENT && retryCount-- > 0)
					continue;
				return 0;
			}
			memset(&fl, 0, sizeof(fl));
			fl.l_type = F_WRLCK;
			fl.l_whence = SEEK_SET;
			if(rval == 0) {
				/* We created the link, lock the file */
				fcntl(fd, F_SETLK, &fl);
				weLockedMtab = 1;
				close(fd);
				break;
			}else{
				/* Wait for the lockfile to become free */
				if(fcntl(fd, F_SETLK, &fl) == 0){	/* owned by nobody... */
					close(fd);
					unlink(MTAB_LOCK);
					if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
						logPrintf(LOGM_ERROR, "system: removed stale lock file\n");
					if(retryCount-- < 0)
						return 0;
					continue;
				}
				rval = fcntl(fd, F_SETLK, &fl);
				close(fd);
				if(rval == -1)
					return 0;
			}
		}
	}else{
		if(weLockedMtab)
			unlink(MTAB_LOCK);
		weLockedMtab = 0;
	}
	return 1;
}

#else	/* not linux */

static btbool	mtabSetLock(btbool state)
{
#ifdef MTAB_LOCK	/* only if we have a lockfile defined */
static int		lockMtabFd = -1;
int				fd;
struct flock	fl;
btbool			lockOK = 0;

	if(state){
		if(lockMtabFd == -1){
			fd = open(MTAB_LOCK, O_RDWR | O_CREAT | O_TRUNC, 0644);
			if(fd < 0){
				if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
					logPrintf(LOGM_ERROR, "system: error opening %s: [%d] %s\n", MTAB_LOCK, errno, sysStrerror(errno));
			}else{
				memset(&fl, 0, sizeof(fl));
				fl.l_type = F_WRLCK;
				fl.l_whence = SEEK_CUR;
				if(fcntl(fd, F_SETLKW, &fl) == -1) {
					if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
						logPrintf(LOGM_ERROR, "system: error locking %s: [%d] %s\n", MTAB_LOCK, errno, sysStrerror(errno));
					close(fd);
				}else{
					lockMtabFd = fd;
					lockOK = 1;
				}
			}
		}
		if(!lockOK && LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "system: could not lock %s\n", MTAB_LOCK);
	}else{
		if(lockMtabFd != -1){
			close(lockMtabFd);
#if 0	/* must not be unlinked for 'du' to work, at least on Solaris */
			unlink(MTAB_LOCK);
#endif
			lockMtabFd = -1;
			lockOK = 1;	/* unlock OK */
		}
	}
	return lockOK;
#else	/* MTAB_LOCK */
	return 1;	/* dummy implementation */
#endif	/* MTAB_LOCK */
}

#endif /* linux */

static int	addMtabEntry(char *mountpoint, char *fsname)
{
FILE			*fp;
struct mntent	mnt;
#if defined SOLARIS2 || defined UNIXWARE
char			mnttime[20];
#endif

	memset(&mnt, 0, sizeof(mnt));
	mnt.mnt_fsname = fsname;
	mnt.mnt_dir = mountpoint;
	mnt.mnt_type = "nfs";
	mnt.mnt_opts ="soft,noquota";
#if defined HP
	mnt.mnt_time = time(NULL);
#endif
#if defined SOLARIS2 || defined UNIXWARE
	sprintf(mnttime, "%ld", time(NULL));
	mnt.mnt_time = mnttime;	/* the small differences make life so interesting.... */
#else
	mnt.mnt_freq = mnt.mnt_passno = 0;
#endif
	if(!mtabSetLock(1)){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "system: error locking %s\n", MTAB_FILE);
		return 0;	/* ignore mtab errors, mounting may still succeed */
	}
	if((fp = setmntent(MTAB_FILE, "a")))
		addmntent(fp, &mnt);
	else{
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "system: error opening %s: [%d] %s\n", MTAB_FILE, errno, sysStrerror(errno));
		return 0;	/* ignore mtab errors, mounting may still succeed */
	}
	endmntent(fp);
	mtabSetLock(0);
	return 0;
}

int	removeMntEntry(char *mountpoint)
{
FILE			*fpout, *fpin;
struct mntent	*ent;
int				storedErrno;

	if(!mtabSetLock(1))
		return -1;
	if((fpout = setmntent(MTAB_TMP, "w")) == NULL){
		storedErrno = errno;
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "system: error creating %s: [%d] %s\n", MTAB_TMP, storedErrno, sysStrerror(storedErrno));
		goto errorOccured;
	}else if((fpin = setmntent(MTAB_FILE, "r")) == NULL){
		storedErrno = errno;
		endmntent(fpout);
		unlink(MTAB_TMP);
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "system: error opening %s: [%d] %s\n", MTAB_FILE, storedErrno, sysStrerror(storedErrno));
		goto errorOccured;
	}else{
		while((ent = readmntent(fpin)) != NULL){
			if(strcmp(ent->mnt_dir, mountpoint) != 0)
				addmntent(fpout, ent);
		}
		endmntent(fpin);
		endmntent(fpout);
		if(rename(MTAB_TMP, MTAB_FILE) != 0){
			storedErrno = errno;
			unlink(MTAB_TMP);
			if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
				logPrintf(LOGM_ERROR, "system: error renaming %s to %s: [%d] %s\n", MTAB_TMP, MTAB_FILE, storedErrno, sysStrerror(storedErrno));
			goto errorOccured;
		}
		chmod(MTAB_FILE, 0644);
	}
	mtabSetLock(0);
	return 0;
errorOccured:
	mtabSetLock(0);
	errno = storedErrno;
	return 0;	/* ignore mtab errors, unmounting may still succeed */
}
#endif	/* defined NeXT || defined linux || defined HP || defined __sgi || defined SOLARIS2  || defined SUNOS */


/* ------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------- */
/* ------------------ the mount and umount system calls: ------------------- */
/* ------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------- */

/* we can't rely on nfs_fh3 being defined in any system header: */
typedef struct sysNfsFh3{
	btu32	len;
	btu8	data[64];
}sysNfsFh3_t;

/* ------------------------------------------------------------------------- */
/* ------------------- NEXTSTEP, OPENSTEP/Mach and SunOS ------------------- */
/* ------------------------------------------------------------------------- */

#if defined NeXT || defined SUNOS
#include <sys/mount.h>
#ifdef NeXT
#	include <nfs/nfs_mount.h>
#else
#	undef MOUNT_NFS
#	define MOUNT_NFS	"nfs"
#endif

int	sysMount(char *mpAscii, void *rootFh, int sockFd, struct sockaddr_in *sockAddr, char *serverName, char *nfsPath, btint nfsTimeout, btint nfsTransferSize, btbool readOnly, btint nfsRetransmit, btint kernelCacheTime, btbool useNfs3)
{
struct nfs_args		nfs_args;
int					rval, flags = 0;

	if(nfsRetransmit < 0){
		nfsRetransmit = 3;
	}else{
		flags |= NFSMNT_SOFT | NFSMNT_RETRANS;
	}
	if(useNfs3){	/* These old OSes don't support NFS3 */
		errno = EOPNOTSUPP;
		return -1;
	}
	if(kernelCacheTime == 0){
		flags |= NFSMNT_NOAC;
	}else{
		flags |= NFSMNT_ACREGMIN | NFSMNT_ACREGMAX | NFSMNT_ACDIRMIN | NFSMNT_ACDIRMAX;
	}
	memset(&nfs_args, 0, sizeof(nfs_args));
	nfs_args.wsize		= nfsTransferSize;
	nfs_args.rsize		= nfsTransferSize;
	nfs_args.retrans	= nfsRetransmit;
	nfs_args.timeo		= nfsTimeout;	/* timeout */
	nfs_args.acregmin = nfs_args.acregmax = nfs_args.acdirmin = nfs_args.acdirmax = kernelCacheTime;	/* in seconds */
	nfs_args.addr		= sockAddr;
	nfs_args.flags		= flags | NFSMNT_HOSTNAME | NFSMNT_TIMEO | NFSMNT_WSIZE | NFSMNT_RSIZE;
	nfs_args.fh			= (char *)rootFh;
	nfs_args.hostname	= serverName;
	nfs_args.netname	= "net";
	if((rval = mount(MOUNT_NFS, mpAscii, readOnly ? M_RDONLY : 0, (caddr_t)&nfs_args)) != 0)
        return rval;
	return addMtabEntry(mpAscii, nfsPath);
}

int	sysUmount(char *mpAscii)
{
int		rval;

	if((rval = unmount(mpAscii)) != 0)
        return rval;
	return removeMntEntry(mpAscii);
}

#endif /* NeXT */

/* ------------------------------------------------------------------------- */
/* --------------------------------- Linux --------------------------------- */
/* ------------------------------------------------------------------------- */

#ifdef linux
/* Prevent inclusion of <linux/in.h> by nfs_mount.h. We already have
 * netinet/in.h included and many definitions are conflicting. This is
 * probably a hack...
 */
#define _LINUX_IN_H
/* The following lines are a hack -- definitely. Unfortunately, the NFS
 * structures have been moved around among system headers for the 2.4
 * kernel.
 */
#if 0
/* I don't think we need this anywhere now... */
		#include <linux/version.h>
		#ifndef KERNEL_VERSION	/* make sure to compile old-kernel branch */
		#	define	KERNEL_VERSION(a, b, c)	0x7ffffff0
		#endif
		#if defined LINUX_VERSION_CODE && LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
		#	define __KERNEL__
		#	define _LINUX_SUNRPC_MSGPROT_H_
		#	include <linux/nfs.h>
		#	undef __KERNEL__
		#	include <linux/nfs2.h>
		#else
			struct nfs_fh {
				char data[32];
			};
		#endif
#endif
#include <linux/nfs_mount.h>
#include <sys/mount.h>
#ifndef MS_RDONLY
#	include <linux/fs.h>
#endif

int	sysMount(char *mpAscii, void *rootFh, int sockFd, struct sockaddr_in *sockAddr, char *serverName, char *nfsPath, btint nfsTimeout, btint nfsTransferSize, btbool readOnly, btint nfsRetransmit, btint kernelCacheTime, btbool useNfs3)
{
struct nfs_mount_data	nfsArgs;
int						ksock, rval, flags = 0;
struct sockaddr_in		kaddr;

	if(nfsRetransmit < 0){
		nfsRetransmit = 10000;	/* try forever... */
	}else{
		flags |= NFS_MOUNT_SOFT;
	}
	if(useNfs3){
#ifdef NFS_MOUNT_VER3
#define	HAVE_NFS3	1
		flags |= NFS_MOUNT_VER3;
#else
		errno = EOPNOTSUPP;
		return -1;
#endif
	}
#ifdef NFS_MOUNT_NONLM
	flags |= NFS_MOUNT_NONLM;	/* Sharity can't do locking anyway */
#endif
/*
 * We have to do the connect for the kernel before we can call mount().
 */
	ksock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if(ksock < 0){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
			logPrintf(LOGM_ERROR, "system: creating kernel socket: [%d] %s\n", errno, sysStrerror(errno));
		return -1;
	}
    memset(&kaddr, 0, sizeof(kaddr));
	kaddr.sin_family = AF_INET;
	kaddr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
	if(bind(ksock, (struct sockaddr *) &kaddr, sizeof(kaddr)) < 0){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
        	logPrintf(LOGM_ERROR, "system: binding kernel socket: [%d] %s\n", errno, sysStrerror(errno));
        return -1;
    }
	memset(&nfsArgs, 0, sizeof(nfsArgs));
	nfsArgs.version	= NFS_MOUNT_VERSION;
	nfsArgs.fd			= ksock;
	nfsArgs.flags		= flags;
	nfsArgs.wsize		= nfsTransferSize;
	nfsArgs.rsize		= nfsTransferSize;
	nfsArgs.retrans		= nfsRetransmit;
	nfsArgs.timeo		= nfsTimeout;	/* timeout */
	nfsArgs.acregmin = nfsArgs.acregmax = nfsArgs.acdirmin = nfsArgs.acdirmax	= kernelCacheTime;		/* in seconds */
	nfsArgs.addr		= *sockAddr;
#if NFS_MOUNT_VERSION >= 4
	nfsArgs.root.size = 32;
	memcpy(&nfsArgs.root.data, rootFh, nfsArgs.root.size);
#else
	memcpy(&nfsArgs.root, rootFh, 32);
#endif
	strncpy(nfsArgs.hostname, serverName, sizeof(nfsArgs.hostname));

	if(connect(ksock, (struct sockaddr *)&nfsArgs.addr, sizeof(nfsArgs.addr)) < 0){
		if(LOG_MODE_IS_ACTIVE(LOGM_ERROR))
        	logPrintf(LOGM_ERROR, "system: connecting kernel socket: [%d] %s\n", errno, sysStrerror(errno));
        return -1;
	}
	if((rval = mount(nfsPath, mpAscii, "nfs", 0xc0ed0000 | (readOnly ? MS_RDONLY : 0), &nfsArgs)) < 0)
		return rval;
	return addMtabEntry(mpAscii, nfsPath);
}

int	sysUmount(char *mpAscii)
{
int		rval;

	if((rval = umount(mpAscii)) != 0)
        return rval;
	return removeMntEntry(mpAscii);
}

#endif	/* linux */

/* ------------------------------------------------------------------------- */
/* ------------------------------- SGI/IRIX -------------------------------- */
/* ------------------------------------------------------------------------- */

#ifdef __sgi
#include <sys/mount.h>
#include <sys/fs/nfs.h>
#include <sys/fs/nfs_clnt.h>
#include <sys/fsid.h>

int	sysMount(char *mpAscii, void *rootFh, int sockFd, struct sockaddr_in *sockAddr, char *serverName, char *nfsPath, btint nfsTimeout, btint nfsTransferSize, btbool readOnly, btint nfsRetransmit, btint kernelCacheTime, btbool useNfs3)
{
struct nfs_args	nfs_args;
int				findex, rval, flags = 0;
	
	if(nfsRetransmit < 0){
		nfsRetransmit = 3;
	}else{
		flags |= NFSMNT_SOFT | NFSMNT_RETRANS;
	}
	if(kernelCacheTime == 0){
		flags |= NFSMNT_NOAC;
	}else{
		flags |= NFSMNT_ACREGMIN | NFSMNT_ACREGMAX | NFSMNT_ACDIRMIN | NFSMNT_ACDIRMAX;
	}
	if(useNfs3){
#ifdef FSID_NFS3
#define	HAVE_NFS3	1
		findex = sysfs(GETFSIND, FSID_NFS3);
#else
		errno = EOPNOTSUPP;
		return -1;
#endif
	}else{
		findex = sysfs(GETFSIND, FSID_NFS);	/* this is the default NFS2 file system */
	}
	memset(&nfs_args, 0, sizeof(nfs_args));
	nfs_args.wsize		= nfsTransferSize;
	nfs_args.rsize		= nfsTransferSize;
	nfs_args.retrans	= nfsRetransmit;
	nfs_args.timeo		= nfsTimeout;  /* timeout */
	nfs_args.addr		= sockAddr;
	nfs_args.acregmin = nfs_args.acregmax = nfs_args.acdirmin = nfs_args.acdirmax = kernelCacheTime;		/* in seconds */
	nfs_args.flags		= flags | NFSMNT_HOSTNAME | NFSMNT_TIMEO | NFSMNT_WSIZE | NFSMNT_RSIZE;
	nfs_args.fh			= (fhandle_t *)rootFh;
#ifdef FSID_NFS3
	nfs_args.fh_len = 32;
#endif
	nfs_args.hostname	= serverName;
	if((rval = mount(mpAscii, mpAscii, MS_FSS | MS_DATA | (readOnly ? MS_RDONLY : 0), findex, &nfs_args, sizeof(struct nfs_args))) < 0)
		return rval;
	return addMtabEntry(mpAscii, nfsPath);
}

int	sysUmount(char *mpAscii)
{
int		rval;

	if((rval = umount(mpAscii)) != 0)
        return rval;
	return removeMntEntry(mpAscii);
}

#endif	/* __sgi */

/* ------------------------------------------------------------------------- */
/* ---------------------------- HP-UX 9 and 10 ----------------------------- */
/* ------------------------------------------------------------------------- */

#if defined HP && !defined HP11
#include <sys/mount.h>
#include <nfs/nfs.h>

struct nfs_fh3 {	/* why is this in no system header? */
	uint32_t fh3_length;
	union nfs_fh3_u {
		char data[64];
	} fh3_u;
};

int	sysMount(char *mpAscii, void *rootFh, int sockFd, struct sockaddr_in *sockAddr, char *serverName, char *nfsPath, btint nfsTimeout, btint nfsTransferSize, btbool readOnly, btint nfsRetransmit, btint kernelCacheTime, btbool useNfs3)
{
struct nfs_args	nfs_args;
int				rval, flags = 0;
char			*fstype = MNTTYPE_NFS;
sysNfsFh3_t		fh3;

	if(nfsRetransmit < 0){
		nfsRetransmit = 3;
	}else{
		flags |= NFSMNT_INT | NFSMNT_RETRANS;
	}
	if(useNfs3){
#ifdef MNTTYPE_NFS3
#define	HAVE_NFS3	1
		fstype = MNTTYPE_NFS3;
#else
		errno = EOPNOTSUPP;
		return -1;
#endif
	}
	if(kernelCacheTime == 0){
		flags |= NFSMNT_NOAC;
	}else{
		flags |= NFSMNT_ACREGMIN | NFSMNT_ACREGMAX | NFSMNT_ACDIRMIN | NFSMNT_ACDIRMAX;
	}
	memset(&nfs_args, 0, sizeof(nfs_args));
	nfs_args.wsize		= nfsTransferSize;
	nfs_args.rsize		= nfsTransferSize;
	nfs_args.retrans	= nfsRetransmit;
	nfs_args.timeo		= nfsTimeout;	/* timeout */
	nfs_args.acregmin = nfs_args.acregmax = nfs_args.acdirmin = nfs_args.acdirmax = kernelCacheTime;	/* in seconds */
	nfs_args.addr		= sockAddr;
	nfs_args.flags		= flags | NFSMNT_HOSTNAME | NFSMNT_TIMEO | NFSMNT_WSIZE | NFSMNT_RSIZE;
	if(useNfs3){
		fh3.len = 32;
		memcpy(&fh3.data, rootFh, 32);
		nfs_args.fh = (void *)&fh3;
	}else{
		nfs_args.fh = (void *)rootFh;
	}
	nfs_args.hostname	= serverName;
#ifdef NFSMNT_FSNAME
	nfs_args.flags		|= NFSMNT_FSNAME;
	nfs_args.fsname		= nfsPath;
#endif
#if 0 /* old style mount call does not support NFS3 */
	if((rval = vfsmount(MOUNT_NFS, mpAscii, readOnly ? M_RDONLY : 0, (caddr_t)&nfs_args)) < 0)
#endif
	if((rval = mount(nfsPath, mpAscii, MS_DATA | (readOnly ? MS_RDONLY : 0), fstype, (char *)&nfs_args, sizeof(nfs_args))) < 0)
		return rval;
	return addMtabEntry(mpAscii, nfsPath);
}

int	sysUmount(char *mpAscii)
{
int		rval;

	if((rval = umount(mpAscii)) != 0)
        return rval;
	return removeMntEntry(mpAscii);
}

#endif /* HP */

/* ------------------------------------------------------------------------- */
/* ---------------------------------- AIX ---------------------------------- */
/* ------------------------------------------------------------------------- */

#ifdef _AIX
#include <sys/vmount.h>

struct nfs_args {
	struct sockaddr_in	addr;		/* file server address */
	fhandle_t			fh;			/* File handle to be mounted */
	int					flags;		/* flags */
	int					wsize;		/* write size in bytes */
	int					rsize;		/* read size in bytes */
	int					timeo;		/* initial timeout in .1 secs */
	int					retrans;	/* times to retry send */
	char				*hostname;	/* server's hostname */
	int					acregmin;	/* attr cache file min secs */
	int					acregmax;	/* attr cache file max secs */
	int					acdirmin;	/* attr cache dir min secs */
	int					acdirmax;	/* attr cache dir max secs */
	char				*netname;	/* server's netname */
	int					biods;		/* number of BIODS */
};

struct nfs_args3 {
  struct sockaddr_in	addr;		/* server socket addr */
  caddr_t				unknown;
  unsigned long			proto;		/* IPPROTO_TCP or IPPROTO_UDP */
  char					*hostname;
  char					*netname;
  caddr_t				fh;			/* points to sysAixNfsFh3_t */
  unsigned long			flags;
  unsigned long			wsize;
  unsigned long			rsize;
  unsigned long			timeo;
  unsigned long			retrans;
  unsigned long			acregmin;
  unsigned long			acregmax;
  unsigned long			acdirmin;
  unsigned long			acdirmax;
  unsigned long			unknown1;
  struct pathcnf		*pathconf;
};

typedef struct sysAixNfsFh3{
  btu32	len;
  btu8	*data;
}sysAixNfsFh3_t;


#define NFSMNT_SOFT		0x001	/* soft mount (hard is default) */
#define NFSMNT_WSIZE	0x002	/* set write size */
#define NFSMNT_RSIZE	0x004	/* set read size */
#define NFSMNT_TIMEO	0x008	/* set initial timeout */
#define NFSMNT_RETRANS	0x010	/* set number of request retrys */
#define NFSMNT_HOSTNAME	0x020	/* set hostname for error printf */
#define NFSMNT_INT		0x040	/* allow interrupts on hard mount */
#define	NFSMNT_NOAC		0x080	/* don't cache attributes */
#define	NFSMNT_ACREGMIN	0x0100	/* set min secs for file attr cache */
#define	NFSMNT_ACREGMAX	0x0200	/* set max secs for file attr cache */
#define	NFSMNT_ACDIRMIN	0x0400	/* set min secs for dir attr cache */
#define	NFSMNT_ACDIRMAX	0x0800	/* set max secs for dir attr cache */
#define NFSMNT_SECURE	0x1000	/* secure mount */
#define NFSMNT_BIODS	0x10000	/* Number of biods for the file system */

#define ADD_ENTRY(index, data, size)				\
	v->vmt_data[index].vmt_off = p - buffer;		\
	v->vmt_data[index].vmt_size = size;				\
	if(p + size - buffer > sizeof(longbuf))			\
		goto overflowError;							\
	memcpy(p, data, size);							\
	p += (4 * ((size + 3) / 4))	/* align to 32 bit boundary */

int	sysMount(char *mpAscii, void *rootFh, int sockFd, struct sockaddr_in *sockAddr, char *serverName, char *nfsPath, btint nfsTimeout, btint nfsTransferSize, btbool readOnly, btint nfsRetransmit, btint kernelCacheTime, btbool useNfs3)
{
btu32				longbuf[2048];		/* buffer should be 32 bit aligned */
btu8				*p, *buffer = (void *)longbuf;
struct vmount		*v = (void *)buffer;
struct nfs_args		nfs_args;
struct nfs_args3	nfs_args3;
sysAixNfsFh3_t		fh3;
void				*args;
btint				argsLen;
int					rval, flags = 0;

	memset(buffer, 0, sizeof(buffer));
	if(nfsRetransmit < 0){
		nfsRetransmit = 3;
	}else{
		flags |= NFSMNT_SOFT | NFSMNT_RETRANS;
	}
	if(kernelCacheTime == 0){
		flags |= NFSMNT_NOAC;
	}else{
		flags |= NFSMNT_ACREGMIN | NFSMNT_ACREGMAX | NFSMNT_ACDIRMIN | NFSMNT_ACDIRMAX;
	}
	v->vmt_revision = VMT_REVISION;
	v->vmt_flags = readOnly ? MNT_READONLY : 0;
#ifdef MNT_NFS3
#define	HAVE_NFS3	1
	v->vmt_gfstype = useNfs3 ? MNT_NFS3 : MNT_NFS;
#else
	if(useNfs3){
		errno = EOPNOTSUPP;
		return -1;
	}
	v->vmt_gfstype = MNT_NFS;
#endif
	if(useNfs3){
		memset(&nfs_args3, 0, sizeof(nfs_args3));
		fh3.len = 32;
		fh3.data = rootFh;
		nfs_args3.addr		= *sockAddr;
		nfs_args3.proto		= IPPROTO_UDP;
		nfs_args3.fh		= (void *)&fh3;
		nfs_args3.wsize		= nfsTransferSize;
		nfs_args3.rsize		= nfsTransferSize;
		nfs_args3.retrans	= nfsRetransmit;
		nfs_args3.timeo		= nfsTimeout;	/* timeout */
		nfs_args3.acregmin = nfs_args.acregmax = nfs_args.acdirmin = nfs_args.acdirmax = kernelCacheTime;	/* in seconds */
		nfs_args3.hostname	= serverName;
		nfs_args3.flags		= flags | NFSMNT_HOSTNAME | NFSMNT_TIMEO | NFSMNT_WSIZE | NFSMNT_RSIZE;
		args = &nfs_args3;
		argsLen = sizeof(nfs_args3);
	}else{
		memset(&nfs_args, 0, sizeof(nfs_args));
		nfs_args.addr		= *sockAddr;
		nfs_args.fh			= *(fhandle_t *)rootFh;
		nfs_args.wsize		= nfsTransferSize;
		nfs_args.rsize		= nfsTransferSize;
		nfs_args.retrans	= nfsRetransmit;
		nfs_args.biods		= 4;
		nfs_args.timeo		= nfsTimeout;	/* timeout */
		nfs_args.acregmin = nfs_args.acregmax = nfs_args.acdirmin = nfs_args.acdirmax = kernelCacheTime;	/* in seconds */
		nfs_args.hostname	= serverName;
		nfs_args.flags		= flags | NFSMNT_HOSTNAME | NFSMNT_TIMEO | NFSMNT_WSIZE | NFSMNT_RSIZE | NFSMNT_BIODS;
		args = &nfs_args;
		argsLen = sizeof(nfs_args);
	}
	p = buffer + sizeof(struct vmount);
	ADD_ENTRY(VMT_OBJECT, "", 1);								/* path at server */
	ADD_ENTRY(VMT_STUB, mpAscii, strlen(mpAscii) + 1);			/* mountpoint */
	ADD_ENTRY(VMT_HOST, serverName, strlen(serverName) + 1);	/* hostname (?) */
	ADD_ENTRY(VMT_HOSTNAME, serverName, strlen(serverName) + 1);/* hostname */
	ADD_ENTRY(VMT_INFO, args, argsLen);							/* arguments for NFS client */
	ADD_ENTRY(VMT_ARGS, nfsPath, strlen(nfsPath) + 1);			/* arguments of mount */
	v->vmt_length = p - buffer;
	if((rval = vmount(buffer, v->vmt_length)) < 0)
		return rval;
	return 0;
overflowError:
	errno = EINVAL;
	return -1;
}

int	sysUmount(char *mpAscii)
{
struct stat	statbuf;

/* stat() may be done here although it is forbidden in the rest of the server
 * code because sysMount() and sysUmount() are called from a separate
 * (forked) process.
 */
	if(stat(mpAscii, &statbuf)){
		return -1;
	}
	return uvmount(statbuf.st_vfs, UVMNT_FORCE);
}

#endif	/* _AIX */

/* ------------------------------------------------------------------------- */
/* -------------------- SOLARIS, UNIXWARE and HP-UX 11 --------------------- */
/* ------------------------------------------------------------------------- */

#if defined SOLARIS2 || defined UNIXWARE || defined HP11
#include <sys/mount.h>
#include <netconfig.h>
#include <nfs/nfs.h>

#if !defined HP11
#	include <nfs/mount.h>
#endif

#if defined UNIXWARE
#	include <sys/fstyp.h>
#	include <sys/fsid.h>
#endif

#if defined HP11
/* Is it possible that HP forgot to include the following in their system
 * header files? Anyway, thanks to Uli Betzler who found the striking
 * similarity to Solaris!
 */
struct nfs_args {
	struct netbuf		*addr;		/* file server address */
	struct netbuf		*syncaddr;	/* secure NFS time sync addr */
	struct knetconfig	*knconf;	/* transport netconfig struct */
	char				*hostname;	/* server's hostname */
	char				*netname;	/* server's netname */
	void				*fh;		/* File handle to be mounted */
	uint32_t			flags;		/* flags */
	int32_t				wsize;		/* write size in bytes */
	int32_t				rsize;		/* read size in bytes */
	int32_t				timeo;		/* initial timeout in .1 secs */
	int32_t				retrans;	/* times to retry send */
	int32_t				acregmin;	/* attr cache file min secs */
	int32_t				acregmax;	/* attr cache file max secs */
	int32_t				acdirmin;	/* attr cache dir min secs */
	int32_t				acdirmax;	/* attr cache dir max secs */
/* Added for HP-UX */
	char				*fsname;	/* server's fs path name */
};

/*
 * NFS mount option flags
 */
#define	NFSMNT_SOFT			0x001	/* soft mount (hard is default) */
#define	NFSMNT_WSIZE		0x002	/* set write size */
#define	NFSMNT_RSIZE		0x004	/* set read size */
#define	NFSMNT_TIMEO		0x008	/* set initial timeout */
#define	NFSMNT_RETRANS		0x010	/* set number of request retrys */
#define	NFSMNT_HOSTNAME		0x020	/* set hostname for error printf */
#define	NFSMNT_INT			0x040	/* allow interrupts on hard mount */
#define	NFSMNT_NOAC			0x080	/* don't cache attributes */
#define	NFSMNT_ACREGMIN		0x0100	/* set min secs for file attr cache */
#define	NFSMNT_ACREGMAX		0x0200	/* set max secs for file attr cache */
#define	NFSMNT_ACDIRMIN		0x0400	/* set min secs for dir attr cache */
#define	NFSMNT_ACDIRMAX		0x0800	/* set max secs for dir attr cache */
#define	NFSMNT_KNCONF		0x4000	/* transport's knetconfig structure */
#endif


int	sysMount(char *mpAscii, void *rootFh, int sockFd, struct sockaddr_in *sockAddr, char *serverName, char *nfsPath, btint nfsTimeout, btint nfsTransferSize, btbool readOnly, btint nfsRetransmit, btint kernelCacheTime, btbool useNfs3)
{
struct nfs_args		nfs_args;
struct netbuf		nb;
struct netconfig	*nc;
struct knetconfig	kn;
struct stat			stb;
sysNfsFh3_t			fh3;
int					rval = 0, flags = 0;
btu8				*fstype;

	if(nfsRetransmit < 0){
		nfsRetransmit = 3;
	}else{
		flags |= NFSMNT_SOFT | NFSMNT_RETRANS;
	}
	if(kernelCacheTime == 0){
		flags |= NFSMNT_NOAC;
	}else{
		flags |= NFSMNT_ACREGMIN | NFSMNT_ACREGMAX | NFSMNT_ACDIRMIN | NFSMNT_ACDIRMAX;
	}
#define	HAVE_NFS3	1
	fstype = useNfs3 ? "nfs3" : "nfs";
	nb.buf = (char*)sockAddr;
	nb.maxlen = nb.len = sizeof(struct sockaddr_in);

	nc = getnetconfigent("udp");
	if(nc == NULL)
		return -1;

	memset(&kn, 0, sizeof(kn));
	kn.knc_semantics = nc->nc_semantics;
	kn.knc_protofmly = nc->nc_protofmly;
	kn.knc_proto = nc->nc_proto;
/* stat() may be done here although it is forbidden in the rest of the server
 * code because sysMount() and sysUmount() are called from a separate
 * (forked) process.
 */
	if(stat(nc->nc_device, &stb) == -1)
		return -1;
	kn.knc_rdev = stb.st_rdev;

	memset(&nfs_args, 0, sizeof(nfs_args));
	nfs_args.wsize		= nfsTransferSize;
	nfs_args.rsize		= nfsTransferSize;
	nfs_args.retrans	= nfsRetransmit;
	nfs_args.timeo		= nfsTimeout;	/* timeout */
	nfs_args.acregmin = nfs_args.acregmax = nfs_args.acdirmin = nfs_args.acdirmax = kernelCacheTime;	/* in seconds */
	nfs_args.addr		= &nb;
	nfs_args.knconf		= &kn;
	nfs_args.flags		= flags | NFSMNT_HOSTNAME | NFSMNT_TIMEO | NFSMNT_WSIZE | NFSMNT_RSIZE | NFSMNT_KNCONF;
	if(useNfs3){
		fh3.len = 32;
		memcpy(&fh3.data, rootFh, 32);
		nfs_args.fh = (char *)&fh3;
	}else{
		nfs_args.fh = (char *)rootFh;
	}
	nfs_args.hostname	= serverName;
	nfs_args.netname	= "net";
#if defined SOLARIS8
	return mount(nfsPath, mpAscii, MS_DATA | (readOnly ? MS_RDONLY : 0), fstype, (void *)&nfs_args, sizeof(nfs_args));
#else
	if((rval = mount("", mpAscii, MS_DATA | (readOnly ? MS_RDONLY : 0), fstype, (void *)&nfs_args, sizeof(nfs_args))) != 0)
        return rval;
	return addMtabEntry(mpAscii, nfsPath);
#endif
}

int	sysUmount(char *mpAscii)
{
int		rval;

	if((rval = umount(mpAscii)) != 0)
        return rval;
#if defined SOLARIS8
	return rval;
#else
	return removeMntEntry(mpAscii);
#endif
}

#endif /* SOLARIS2 */

/* ------------------------------------------------------------------------- */
/* ------------------------------- DEC/OSF1 -------------------------------- */
/* ------------------------------------------------------------------------- */

#ifdef __osf__
#include <sys/mount.h>
#include <nfs/nfs.h>

int	sysMount(char *mpAscii, void *rootFh, int sockFd, struct sockaddr_in *sockAddr, char *serverName, char *nfsPath, btint nfsTimeout, btint nfsTransferSize, btbool readOnly, btint nfsRetransmit, btint kernelCacheTime, btbool useNfs3)
{
struct nfs_args		nfs_args;
btint				flags = 0, fstype = MOUNT_NFS;
struct nfs_fh3		fh3;

	if(nfsRetransmit < 0){
		nfsRetransmit = 3;
	}else{
		flags |= NFSMNT_SOFT | NFSMNT_RETRANS;
	}
	if(useNfs3){
#ifdef MOUNT_NFS3
#define	HAVE_NFS3	1
		fstype = MOUNT_NFS3;
#else
		errno = EOPNOTSUPP;
		return -1;
#endif
	}
	if(kernelCacheTime == 0){
		flags |= NFSMNT_NOAC;
	}else{
		flags |= NFSMNT_ACREGMIN | NFSMNT_ACREGMAX | NFSMNT_ACDIRMIN | NFSMNT_ACDIRMAX;
	}
	sockAddr->sin_len = sizeof(*sockAddr);
	sockAddr->sin_family = AF_INET;
	memset(&nfs_args, 0, sizeof(nfs_args));
	nfs_args.wsize		= nfsTransferSize;
	nfs_args.rsize		= nfsTransferSize;
	nfs_args.retrans	= nfsRetransmit;
	nfs_args.timeo		= nfsTimeout;	/* timeout */
	nfs_args.acregmin = nfs_args.acregmax = nfs_args.acdirmin = nfs_args.acdirmax = kernelCacheTime;	/* in seconds */
	nfs_args.addr		= sockAddr;
	nfs_args.flags		= flags | NFSMNT_HOSTNAME | NFSMNT_TIMEO | NFSMNT_WSIZE | NFSMNT_RSIZE;
	if(useNfs3){
		fh3.fh3_length = 32;
		memcpy(&fh3.fh3_u.data, rootFh, 32);
		nfs_args.fh = (char *)&fh3;
	}else{
		nfs_args.fh = (char *)rootFh;
	}
	nfs_args.hostname	= serverName;
	nfs_args.netname	= "net";
	return mount(MOUNT_NFS, mpAscii, readOnly ? M_RDONLY : 0, (caddr_t)&nfs_args);
}

int	sysUmount(char *mpAscii)
{
	return umount(mpAscii, MNT_FORCE);
}

#endif /* __osf__ */

/* ------------------------------------------------------------------------- */
/* ---------------------------------- BSD ---------------------------------- */
/* ------------------------------------------------------------------------- */

#if defined __bsdi__ || defined __NetBSD__ || defined __FreeBSD__ || defined __OpenBSD__ || defined __APPLE__

#define NFS
#include <sys/mount.h>
#if defined __bsdi__ && !defined BSDI3
#	include <nfs/nfsv2.h>
#else
#	include <nfs/nfsproto.h>
#	include <nfs/rpcv2.h>
#endif
#if defined __NetBSD__
#	include <nfs/nfsmount.h>
#endif
#include <nfs/nfs.h>
#if !defined MOUNT_NFS || __FreeBSD__ == 3
#	define MOUNT_NFS	"nfs"
#endif

int	sysMount(char *mpAscii, void *rootFh, int sockFd, struct sockaddr_in *sockAddr, char *serverName, char *nfsPath, btint nfsTimeout, btint nfsTransferSize, btbool readOnly, btint nfsRetransmit, btint kernelCacheTime, btbool useNfs3)
{
struct nfs_args		nfs_args;
btint				rval, flags = 0;

	if(nfsRetransmit < 0){
		nfsRetransmit = 3;
	}else{
		flags |= NFSMNT_SOFT | NFSMNT_RETRANS;
	}
	if(useNfs3){
#ifdef NFSMNT_NFSV3
#define	HAVE_NFS3	1
		flags |= NFSMNT_NFSV3;
#else
		errno = EOPNOTSUPP;
		return -1;
#endif
	}
#ifdef NFSMNT_NOAC
	if(kernelCacheTime == 0){
		flags |= NFSMNT_NOAC;
	}else{
		flags |= NFSMNT_ACREGMIN | NFSMNT_ACREGMAX | NFSMNT_ACDIRMIN | NFSMNT_ACDIRMAX;
	}
#elif defined NFSMNT_ACREGMIN
	flags |= NFSMNT_ACREGMIN | NFSMNT_ACREGMAX | NFSMNT_ACDIRMIN | NFSMNT_ACDIRMAX;
#endif
	sockAddr->sin_len = sizeof(*sockAddr);
	sockAddr->sin_family = AF_INET;
	memset(&nfs_args, 0, sizeof(nfs_args));
#ifdef NFS_ARGSVERSION
	nfs_args.version	= NFS_ARGSVERSION;
#endif
	nfs_args.wsize		= nfsTransferSize;
	nfs_args.rsize		= nfsTransferSize;
	nfs_args.retrans	= nfsRetransmit;
	nfs_args.timeo		= nfsTimeout;	/* timeout */
#ifdef NFSMNT_ACREGMIN
	nfs_args.acregmin = nfs_args.acregmax = nfs_args.acdirmin = nfs_args.acdirmax = kernelCacheTime;	/* in seconds */
#endif
	nfs_args.addr		= (struct sockaddr *)sockAddr;
	nfs_args.addrlen	= sizeof(*sockAddr);
	nfs_args.sotype		= SOCK_DGRAM;
	nfs_args.flags		= flags | NFSMNT_TIMEO | NFSMNT_WSIZE | NFSMNT_RSIZE;
	nfs_args.fh			= rootFh;
#if !defined __bsdi__ || defined BSDI3
	nfs_args.fhsize		= 32;
#endif
	nfs_args.hostname	= nfsPath;
	rval = mount(MOUNT_NFS, mpAscii, readOnly ? MNT_RDONLY : 0, (caddr_t)&nfs_args);
	return rval;
}

int	sysUmount(char *mpAscii)
{
	return unmount(mpAscii, MNT_FORCE);
}

#endif	/* defined __bsdi__ || defined __NetBSD__ || defined __FreeBSD__ || defined __OpenBSD__ || defined __APPLE__ */

#ifndef HAVE_NFS3
#	define	HAVE_NFS3	0
#endif

btbool	sysHaveNfs3(void)
{
	return HAVE_NFS3;
}

/* ------------------------------------------------------------------------- */

