/*
 * Name: system.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1997-10-16
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: system.h,v 1.6 2001/11/06 09:50:35 cs Exp $
 */

#ifndef __SYSTEM_H_INCLUDED__
#define __SYSTEM_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This module contains wrappers for all system calls that may be different on
various systems.
*/

/* ------------------------------------------------------------------------- */

struct sockaddr_in;

typedef struct sysNetIf{
	btu8	name[16];		/* name of interface */
	btu32	ifAddr;			/* IP address of interface */
	btu32	ifNetmask;		/* netmask */
	btu32	ifBroadcast;	/* broadcast address */
}sysNetIf_t;

char	*sysStrerror(int errcode);
int	sysSetNonblock(int fd);
int	sysNonblockWait(int *rval, btint waitForPid);
void	sysDetach(void);
void	sysSetCorefileLimit(btint limit);
char	*sysRealpath(char *path, char *real);
char	*sysGetPass(char *prompt, char *buffer, int bufferLen);
/* 'buffer' may be the same as 'prompt', returns the password if given,
 * NULL if password input aborted.
 */
void	*sysValloc(int bytes);
/* This function returns a block of virtual memory. It's called with rather
 * large block sizes which are multiples of the pagesize.
 */
void	sysVfree(void *base);

btint	sysNetworkInterfaces(sysNetIf_t *ifArray, btint maxEntries);
/* This function returns all network interfaces which are capable of
 * broadcasts in the location pointed to by 'ifArray' (at maximum 'maxEntries'
 * entries). It returns the number of network interfaces found.
 */
btu8	*sysLoadModule(btu8 *modulePath);
/* This function dynamically loads and links a module to the current process.
 * Loading should call a system-dependent initialisation function such as
 * _init(), where the module can add itself to the framework's table of
 * modules.
 * Parameters: 'modulePath' is the full filesystem path to the module to load.
 * Returns: A pointer to a static error string if an error occured or NULL on
 *    success.
 */

int	sysMount(char *mountpoint, void *rootFh, int sockFd, struct sockaddr_in *sockAddr, char *nfsServer, char *mtabServiceName, btint nfsTimeout, btint nfsTransferSize, btbool readOnly, btint nfsRetransmit, btint kernelCacheTime, btbool useNfs3);

int	sysUmount(char *mountpoint);

btbool	sysHaveNfs3(void);

/* ------------------------------------------------------------------------- */

#endif	/* __SYSTEM_H_INCLUDED__ */
