/*
 * Name: childtable.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1998-04-30
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: childtable.h,v 1.1.1.1 1999/04/12 13:56:45 cs Exp $
 */

#ifndef __childtable_H_INCLUDED__
#define __childtable_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This module contains an array-like container type that's held within
blockAllocator blocks. It is used for directory content lists. The container
object is defined by the allocator context and the index of the object's
block. Operations that resize the object may move it in memory. These
operations return the new location of the object as its index within the
blockAllocator context.
*/

/* ------------------------------------------------------------------------- */

struct blockAllocator;

btu32	ctabObjectCount(struct blockAllocator *ba, btu32 table);
btu32	ctabObjectAtIndex(struct blockAllocator *ba, btu32 table, btu32 index);
void	ctabInsertObjectAtIndex(struct blockAllocator *ba, btu32 *tablep, btu32 index, btu32 object);
void	ctabRemoveObjectAtIndex(struct blockAllocator *ba, btu32 *tablep, btu32 index);
void	ctabRemoveAllObjects(struct blockAllocator *ba, btu32 *tablep);

/* ------------------------------------------------------------------------- */

#endif	/* __childtable_H_INCLUDED__ */
