/*
 * Name: nfs3_prot.h
 * Project: Sharity
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 2001-10-24
 * Tabsize: 4
 * Copyright: (c) 2001 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: nfs3_prot.h,v 1.1 2001/11/06 09:50:35 cs Exp $
 */

#ifndef __nfs3_prot_H_INCLUDED__
#define __nfs3_prot_H_INCLUDED__

#include "basictypes.h"

/*
General Description:
This header defines the NFS3 data types and prototypes.
*/

#define	NFS3_PROGRAM	100003
#define	NFS3_VERSION	3

#define	NFS3_OK				0
#define	NFS3ERR_PERM		1
#define	NFS3ERR_NOENT		2
#define	NFS3ERR_IO			5
#define	NFS3ERR_NXIO		6
#define	NFS3ERR_ACCES		13
#define	NFS3ERR_EXIST		17
#define	NFS3ERR_XDEV		18
#define	NFS3ERR_NODEV		19
#define	NFS3ERR_NOTDIR		20
#define	NFS3ERR_ISDIR		21
#define	NFS3ERR_INVAL		22
#define	NFS3ERR_FBIG		27
#define	NFS3ERR_NOSPC		28
#define	NFS3ERR_ROFS		30
#define	NFS3ERR_MLINK		31
#define	NFS3ERR_NAMETOOLONG	63
#define	NFS3ERR_NOTEMPTY	66
#define	NFS3ERR_DQUOT		69
#define	NFS3ERR_STALE		70
#define	NFS3ERR_REMOTE		71
#define	NFS3ERR_BADHANDLE	10001
#define	NFS3ERR_NOT_SYNC	10002
#define	NFS3ERR_BAD_COOKIE	10003
#define	NFS3ERR_NOTSUPP		10004
#define	NFS3ERR_TOOSMALL	10005
#define	NFS3ERR_SERVERFAULT	10006
#define	NFS3ERR_BADTYPE		10007
#define	NFS3ERR_JUKEBOX		10008


#define	NFS3_TYPE_REG	1
#define	NFS3_TYPE_DIR	2
#define	NFS3_TYPE_BLK	3
#define	NFS3_TYPE_CHR	4
#define	NFS3_TYPE_LNK	5
#define	NFS3_TYPE_SOCK	6
#define	NFS3_TYPE_FIFO	7


#define	NFS3_STABLE_UNSTABLE	0
#define	NFS3_STABLE_DATA_SYNC	1
#define	NFS3_STABLE_FILE_SYNC	2


#define	NFS3_CREATE_UNCHECKED	0
#define	NFS3_CREATE_GUARDED		1
#define	NFS3_CREATE_EXCLUSIVE	2


#define	NFS3_TIMESET_DONT_CHANGE		0
#define	NFS3_TIMESET_SET_TO_SERVER_TIME	1
#define	NFS3_TIMESET_SET_TO_CLIENT_TIME	2

#define	NFS3_PROPERTY_LINK			0x0001
#define	NFS3_PROPERTY_SYMLINK		0x0002
#define	NFS3_PROPERTY_HOMOGENEOUS	0x0008
#define	NFS3_PROPERTY_CANSETTIME	0x0010

#define	NFS3_FEATURE_LINK			0x0001
#define	NFS3_FEATURE_SYMLINK		0x0002
#define	NFS3_FEATURE_HOMOGENEOUS	0x0008
#define	NFS3_FEATURE_CANSETTIME		0x0010


#define	NFS3_WRITEVERFSIZE	8
#define	NFS3_CREATEVERFSIZE	8
#define	NFS3_COOKIEVERFSIZE	8

/* ------------------------------------------------------------------------- */

struct nfs3_opaque{
	btint	len;
	btu8	*data;
};

#define nfs_fh3	nfs3_opaque

struct specdata3{
	btu32	specdata1;
	btu32	specdata2;
};

struct nfstime3{
	btu32	seconds;
	btu32	nseconds;
};

struct fattr3{
	btint				type;
	btu32				mode;
	btu32				nlink;
	btu32				uid;
	btu32				gid;
	btu64				size;
	btu64				used;
	struct specdata3	rdev;
	btu64				fsid;
	btu64				fileid;
	struct nfstime3		atime;
	struct nfstime3		mtime;
	struct nfstime3		ctime;
};

struct diropargs3{
	struct nfs_fh3	dir;
	btu8			*name;
};

/* ------------------------------------------------------------------------- */

struct set_btu32{
	btbool	set;
	btu32	val;	/* only valid if set */
};

struct set_btu64{
	btbool	set;
	btu64	val;	/* only valid if set */
};

struct set_nfstime3{
	btint			mode;
	struct nfstime3	time;	/* only valid if set */
};

struct sattr3{
	struct set_btu32	mode;
	struct set_btu32	uid;
	struct set_btu32	gid;
	struct set_btu64	size;
	struct set_nfstime3	atime;
	struct set_nfstime3	mtime;
};

struct sattrguard3{
	btbool			check;
	struct nfstime3	obj_ctime;	/* only valid if check */
};

struct wcc_attr{
	btu64			size;
	struct nfstime3	mtime;
	struct nfstime3	ctime;
};

struct pre_op_attr{
	btbool			attributes_follow;
	struct wcc_attr	attributes;	/* only valid if attributes_follow */
};

struct post_op_attr{
	btbool			attributes_follow;
	struct fattr3	attributes;	/* only valid if attributes_follow */
};

struct wcc_data{
	struct pre_op_attr	before;
	struct post_op_attr	after;
};

struct post_op_fh3{
	btbool			handle_follows;
	struct nfs_fh3	handle;	/* only valid if handle_follows */
};

/* ------------------------------------------------------------------------- */

struct GETATTR3res{
	btint			status;
	struct fattr3	attributes;	/* only valid if OK */
};

struct SETATTR3args{
	struct nfs_fh3		object;
	struct sattr3		new_attributes;
	struct sattrguard3	guard;
};

struct SETATTR3res{
	btint			status;
	struct wcc_data	obj_wcc;	/* valid on success AND failure */
};

struct LOOKUP3res{
	btint	status;
	struct post_op_attr	dir_attributes;	/* valid on OK AND failure */
	struct nfs_fh3		object;				/* valid only if OK */
	struct post_op_attr	obj_attributes;	/* valid only if OK */
};

struct ACCESS3args{
	struct nfs_fh3	object;
	btu32			access;
};

struct ACCESS3res{
	btint				status;
	struct post_op_attr	obj_attributes;	/* valid on OK AND failure */
	btu32				access;			/* only valid if OK */
};

struct READLINK3res{
	btint				status;
	struct post_op_attr	symlink_attributes;	/* valid on OK AND failure */
	btu8				*data;				/* only valid if OK */
};

struct READ3args{
	struct nfs_fh3	file;
	btu64			offset;
	btu32			count;
};

struct READ3res{
	btint				status;
	struct post_op_attr	file_attributes;	/* valid on OK AND failure */
	btu32				count;				/* rest of struct only valid if OK */
	btbool				eof;
	struct nfs3_opaque	data;
};

struct WRITE3args{
	struct nfs_fh3		file;
	btu64				offset;
	btu32				count;
	btint				stable;
	struct nfs3_opaque	data;
};

struct WRITE3res{
	btint			status;
	struct wcc_data	file_wcc;	/* valid on OK AND failure */
	btu32			count;		/* rest only valid if OK */
	btint			committed;
	btu8			verf[NFS3_WRITEVERFSIZE];
};

struct CREATE3args{
	struct diropargs3	where;
	btint				mode;
	union{
		struct{
			struct sattr3	obj_attributes;
		}guarded_or_unchecked;
		struct{
			btu8			verf[NFS3_CREATEVERFSIZE];
		}exclusive;
	}u;
};

struct CREATE3res{
	btint				status;
	struct wcc_data		dir_wcc;	/* valid on OK AND failure */
	struct post_op_fh3	obj;		/* rest only valid if OK */
	struct post_op_attr	obj_attributes;
};

struct MKDIR3args{
	struct diropargs3	where;
	struct sattr3		attributes;
};

#define	MKDIR3res	CREATE3res

struct SYMLINK3args{
	struct diropargs3	where;
	struct sattr3		symlink_attributes;
	btu8				*symlink_data;
};

#define	SYMLINK3res	CREATE3res

struct MKNOD3args{
	struct diropargs3	where;
	btint				type;	/* discriminator for following: */
	union{
		struct{
			struct sattr3		dev_attributes;
			struct specdata3	spec;
		}char_or_blk;
		struct{
			struct sattr3		pipe_attributes;
		}sock_or_fifo;
	}u;
};

#define	MKNOD3res	CREATE3res

struct REMOVE3res{
	btint			status;
	struct wcc_data	dir_wcc;	/* valid on OK AND failure */
};

#define	RMDIR3res	REMOVE3res

struct RENAME3args{
	struct diropargs3	from;
	struct diropargs3	to;
};

struct RENAME3res{
	btint			status;
	struct wcc_data	fromdir_wcc;	/* valid on OK AND failure */
	struct wcc_data	todir_wcc;		/* valid on OK AND failure */
};

struct LINK3args {
	struct nfs_fh3		file;
	struct diropargs3	link;
};

struct LINK3res{
	btint				status;
	struct post_op_attr	file_attributes;	/* valid on OK AND failure */
	struct wcc_data		linkdir_wcc;		/* valid on OK AND failure */
};


struct READDIR3args{
	struct nfs_fh3	dir;
	btu64			cookie;
	btu8			cookieverf[NFS3_COOKIEVERFSIZE];
	btu32			count;
};

struct entry3{
	btu64			fileid;
	btu8			*name;
	btu64			cookie;
	struct entry3	*nextentry;
};

struct READDIR3res{
	btint				status;
	struct post_op_attr	dir_attributes;	/* valid on OK AND failure */
	btu8				cookieverf[NFS3_COOKIEVERFSIZE];	/* rest only valid if OK */
	struct entry3		*entries;
	btbool				eof;
};

struct READDIRPLUS3args{
	struct nfs_fh3	dir;
	btu64			cookie;
	btu8			cookieverf[NFS3_COOKIEVERFSIZE];
	btu32			dircount;
	btu32			maxcount;
};

struct entryplus3{
	btu64				fileid;
	btu8				*name;
	btu64				cookie;
	struct post_op_attr	name_attributes;
	struct post_op_fh3	name_handle;
	struct entryplus3	*nextentry;
};

struct READDIRPLUS3res{
	btint				status;
	struct post_op_attr	dir_attributes;	/* valid on OK AND failure */
	btu8				cookieverf[NFS3_COOKIEVERFSIZE];	/* rest only valid if OK */
	struct entryplus3	*entries;
	btbool				eof;
};

struct FSSTAT3res{
	btint				status;
	struct post_op_attr	obj_attributes;	/* valid on OK AND failure */
	btu64				tbytes;			/* rest only valid if OK */
	btu64				fbytes;
	btu64				abytes;
	btu64				tfiles;
	btu64				ffiles;
	btu64				afiles;
	btu32				invarsec;
};

struct FSINFO3res{
	btint				status;
	struct post_op_attr	obj_attributes;	/* valid on OK AND failure */
	btu32				rtmax;			/* rest only valid if OK */
	btu32				rtpref;
	btu32				rtmult;
	btu32				wtmax;
	btu32				wtpref;
	btu32				wtmult;
	btu32				dtpref;
	btu64				maxfilesize;
	struct nfstime3		time_delta;
	btu32				properties;
};

struct PATHCONF3res{
	btint				status;
	struct post_op_attr	obj_attributes;	/* valid on OK AND failure */
	btu32				linkmax;		/* rest only valid if OK */
	btu32				name_max;
	btbool				no_trunc;
	btbool				chown_restricted;
	btbool				case_insensitive;
	btbool				case_preserving;
};

struct COMMIT3args{
	struct nfs_fh3	file;
	btu64			offset;
	btu32			count;
};

struct COMMIT3res{
	btint			status;
	struct wcc_data	file_wcc;	/* valid on OK AND failure */
	btu8			verf[NFS3_WRITEVERFSIZE];	/* rest only valid if OK */
};

/* ------------------------------------------------------------------------- */
	
#define NFS3_PROC_NULL			0	/* f(void) = void */
#define NFS3_PROC_GETATTR		1	/* f(nfs_fh3) = GETATTR3res */
#define NFS3_PROC_SETATTR		2	/* f(SETATTR3args) = SETATTR3res */
#define NFS3_PROC_LOOKUP		3	/* f(diropargs3) = LOOKUP3res */
#define NFS3_PROC_ACCESS		4	/* f(ACCESS3args) = ACCESS3res */
#define NFS3_PROC_READLINK		5	/* f(nfs_fh3) = READLINK3res */
#define NFS3_PROC_READ			6	/* f(READ3args) = READ3res */
#define NFS3_PROC_WRITE			7	/* f(WRITE3args) = WRITE3res */
#define NFS3_PROC_CREATE		8	/* f(CREATE3args) = CREATE3res */
#define NFS3_PROC_MKDIR			9	/* f(MKDIR3args) = MKDIR3res */
#define NFS3_PROC_SYMLINK		10	/* f(SYMLINK3args) = SYMLINK3res */
#define NFS3_PROC_MKNOD			11	/* f(MKNOD3args) = MKNOD3res */
#define NFS3_PROC_REMOVE		12	/* f(diropargs3) = REMOVE3res */
#define NFS3_PROC_RMDIR			13	/* f(diropargs3) = RMDIR3res */
#define NFS3_PROC_RENAME		14	/* f(RENAME3args) = RENAME3res */
#define NFS3_PROC_LINK			15	/* f(LINK3args) = LINK3res */
#define NFS3_PROC_READDIR		16	/* f(READDIR3args) = READDIR3res */
#define NFS3_PROC_READDIRPLUS	17	/* f(READDIRPLUS3args) = READDIRPLUS3res */
#define NFS3_PROC_FSSTAT		18	/* f(nfs_fh3) = FSSTAT3res */
#define NFS3_PROC_FSINFO		19	/* f(nfs_fh3) = FSINFO3res */
#define NFS3_PROC_PATHCONF		20	/* f(nfs_fh3) = PATHCONF3res */
#define NFS3_PROC_COMMIT		21	/* f(COMMIT3args) = COMMIT3res */

/* ------------------------------------------------------------------------- */

typedef union nfs3Arg{
	struct nfs_fh3			getattr;
	struct SETATTR3args		setattr;
	struct diropargs3		lookup;
	struct ACCESS3args		access;
	struct nfs_fh3			readlink;
	struct READ3args		read;
	struct WRITE3args		write;
	struct CREATE3args		create;
	struct MKDIR3args		mkdir;
	struct SYMLINK3args		symlink;
	struct MKNOD3args		mknod;
	struct diropargs3		remove;
	struct diropargs3		rmdir;
	struct RENAME3args		rename;
	struct LINK3args		link;
	struct READDIR3args		readdir;
	struct READDIRPLUS3args	readdirplus;
	struct nfs_fh3			fsstat;
	struct nfs_fh3			fsinfo;
	struct nfs_fh3			pathconf;
	struct COMMIT3args		commit;
}nfs3Arg_t;

typedef union nfs3Res{
	struct GETATTR3res		getattr;
	struct SETATTR3res		setattr;
	struct LOOKUP3res		lookup;
	struct ACCESS3res		access;
	struct READLINK3res		readlink;
	struct READ3res			read;
	struct WRITE3res		write;
	struct CREATE3res		create;
	struct MKDIR3res		mkdir;
	struct SYMLINK3res		symlink;
	struct MKNOD3res		mknod;
	struct REMOVE3res		remove;
	struct RMDIR3res		rmdir;
	struct RENAME3res		rename;
	struct LINK3res			link;
	struct READDIR3res		readdir;
	struct READDIRPLUS3res	readdirplus;
	struct FSSTAT3res		fsstat;
	struct FSINFO3res		fsinfo;
	struct PATHCONF3res		pathconf;
	struct COMMIT3res		commit;
}nfs3Res_t;

/* ------------------------------------------------------------------------- */

#endif /* __nfs3_prot_H_INCLUDED__ */
