/*
 * Name: nfs3print.c
 * Project: Sharity
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 2001-10-25
 * Tabsize: 4
 * Copyright: (c) 2001 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: nfs3print.c,v 1.3 2002/02/18 12:37:39 cs Exp $
 */

/*
General Description:
This module implements printing functions for NFS3 data types.
*/

#include "stdheaders.h"
#include "nfs3print.h"
#include "logging.h"

/* ------------------------------------------------------------------------- */

void	nfs3PrintStatus(btint val)
{
btu8	*s = NULL;

	switch(val){
		case NFS3_OK:				s = "NFS3_OK";				break;
		case NFS3ERR_PERM:			s = "NFS3ERR_PERM";			break;
		case NFS3ERR_NOENT:			s = "NFS3ERR_NOENT";		break;
		case NFS3ERR_IO:			s = "NFS3ERR_IO";			break;
		case NFS3ERR_NXIO:			s = "NFS3ERR_NXIO";			break;
		case NFS3ERR_ACCES:			s = "NFS3ERR_ACCES";		break;
		case NFS3ERR_EXIST:			s = "NFS3ERR_EXIST";		break;
		case NFS3ERR_XDEV:			s = "NFS3ERR_XDEV";			break;
		case NFS3ERR_NODEV:			s = "NFS3ERR_NODEV";		break;
		case NFS3ERR_NOTDIR:		s = "NFS3ERR_NOTDIR";		break;
		case NFS3ERR_ISDIR:			s = "NFS3ERR_ISDIR";		break;
		case NFS3ERR_INVAL:			s = "NFS3ERR_INVAL";		break;
		case NFS3ERR_FBIG:			s = "NFS3ERR_FBIG";			break;
		case NFS3ERR_NOSPC:			s = "NFS3ERR_NOSPC";		break;
		case NFS3ERR_ROFS:			s = "NFS3ERR_ROFS";			break;
		case NFS3ERR_MLINK:			s = "NFS3ERR_MLINK";		break;
		case NFS3ERR_NAMETOOLONG:	s = "NFS3ERR_NAMETOOLONG";	break;
		case NFS3ERR_NOTEMPTY:		s = "NFS3ERR_NOTEMPTY";		break;
		case NFS3ERR_DQUOT:			s = "NFS3ERR_DQUOT";		break;
		case NFS3ERR_STALE:			s = "NFS3ERR_STALE";		break;
		case NFS3ERR_REMOTE:		s = "NFS3ERR_REMOTE";		break;
		case NFS3ERR_BADHANDLE:		s = "NFS3ERR_BADHANDLE";	break;
		case NFS3ERR_NOT_SYNC:		s = "NFS3ERR_NOT_SYNC";		break;
		case NFS3ERR_BAD_COOKIE:	s = "NFS3ERR_BAD_COOKIE";	break;
		case NFS3ERR_NOTSUPP:		s = "NFS3ERR_NOTSUPP";		break;
		case NFS3ERR_TOOSMALL:		s = "NFS3ERR_TOOSMALL";		break;
		case NFS3ERR_SERVERFAULT:	s = "NFS3ERR_SERVERFAULT";	break;
		case NFS3ERR_BADTYPE:		s = "NFS3ERR_BADTYPE";		break;
		case NFS3ERR_JUKEBOX:		s = "NFS3ERR_JUKEBOX";		break;
	}
	if(s){
		logPrintf(LOGM_NFSTRACE, "status: %s\n", s);
	}else{
		logPrintf(LOGM_NFSTRACE, "status: *** unknown NFS status %d\n", val);
	}
}

void	nfs3PrintType(btint val)
{
btu8	*s = NULL;

	switch(val){
		case NFS3_TYPE_REG:		s = "NFS3_TYPE_REG";	break;
		case NFS3_TYPE_DIR:		s = "NFS3_TYPE_DIR";	break;
		case NFS3_TYPE_BLK:		s = "NFS3_TYPE_BLK";	break;
		case NFS3_TYPE_CHR:		s = "NFS3_TYPE_CHR";	break;
		case NFS3_TYPE_LNK:		s = "NFS3_TYPE_LNK";	break;
		case NFS3_TYPE_SOCK:	s = "NFS3_TYPE_SOCK";	break;
		case NFS3_TYPE_FIFO:	s = "NFS3_TYPE_FIFO";	break;
	}
	if(s){
		logPrintf(LOGM_NFSTRACE, "file type: %s\n", s);
	}else{
		logPrintf(LOGM_NFSTRACE, "file type: *** unknown NFS type %d\n", val);
	}
}

void	nfs3PrintStable(btint val)
{
btu8	*s = NULL;

	switch(val){
		case NFS3_STABLE_UNSTABLE:	s = "NFS3_STABLE_UNSTABLE";		break;
		case NFS3_STABLE_DATA_SYNC:	s = "NFS3_STABLE_DATA_SYNC";	break;
		case NFS3_STABLE_FILE_SYNC:	s = "NFS3_STABLE_FILE_SYNC";	break;
	}
	if(s){
		logPrintf(LOGM_NFSTRACE, "stable: %s\n", s);
	}else{
		logPrintf(LOGM_NFSTRACE, "stable: *** unknown NFS stable %d\n", val);
	}
}

void	nfs3PrintCreateMode(btint val)
{
btu8	*s = NULL;

	switch(val){
		case NFS3_CREATE_UNCHECKED:	s = "NFS3_CREATE_UNCHECKED";	break;
		case NFS3_CREATE_GUARDED:	s = "NFS3_CREATE_GUARDED";		break;
		case NFS3_CREATE_EXCLUSIVE:	s = "NFS3_CREATE_EXCLUSIVE";	break;
	}
	if(s){
		logPrintf(LOGM_NFSTRACE, "create mode: %s\n", s);
	}else{
		logPrintf(LOGM_NFSTRACE, "create mode: *** unknown create mode %d\n", val);
	}
}

void	nfs3PrintTimesetMode(btint val)
{
btu8	*s = NULL;

	switch(val){
		case NFS3_TIMESET_DONT_CHANGE:			s = "NFS3_TIMESET_DONT_CHANGE";			break;
		case NFS3_TIMESET_SET_TO_SERVER_TIME:	s = "NFS3_TIMESET_SET_TO_SERVER_TIME";	break;
		case NFS3_TIMESET_SET_TO_CLIENT_TIME:	s = "NFS3_TIMESET_SET_TO_CLIENT_TIME";	break;
	}
	if(s){
		logPrintf(LOGM_NFSTRACE, "set time mode: %s\n", s);
	}else{
		logPrintf(LOGM_NFSTRACE, "set time mode: *** unknown mode %d\n", val);
	}
}

static btu8	*strvalBtu64(btu64 val)
{
static btu8	buffer[32];

	sprintf(buffer, "0x%x%08x", (btu32)(val>>32), (btu32)val);
	return buffer;
}

static void	printOpaque(btu8 *title, struct nfs3_opaque *val, btint maxlen)
{
btint	i, len = val->len;
btu8	*data = val->data, buffer[256];
btbool	truncated = 0;

	sprintf(buffer, "%s: len=%d, data=", title, len);
	if(maxlen != 0 && len > maxlen){
		len = maxlen;
		truncated = 1;
	}
	while(len > 0){
		for(i=0;i<16 && len>0;i++){
			sprintf(buffer + strlen(buffer), " %02x", *data++);
			len--;
		}
		logPrintf(LOGM_NFSTRACE, "%s\n", buffer);
		strcpy(buffer, "...data=");
	}
	if(truncated)
		logPrintf(LOGM_NFSTRACE, "<data truncated>\n", buffer);
}

static void	printFixedBlock(btu8 *title, btu8 *data, btint len)
{
struct nfs3_opaque	val[1];

	val->data = data;
	val->len = len;
	printOpaque(title, val, 0);
}

void	nfs3PrintFh(struct nfs3_opaque *val)
{
	printOpaque("nfs3fh", val, 128);
}

void	nfs3PrintSpecdata(struct specdata3 *val)
{
	logPrintf(LOGM_NFSTRACE, "specdata: d1=0x%x, d2=0x%x\n", val->specdata1, val->specdata2);
}

static void	printNfstime(btu8 *title, struct nfstime3 *val)
{
btu8		buffer[128];
struct tm	*tm;
time_t		timeval = val->seconds;

	tm = gmtime(&timeval);
	strftime(buffer, sizeof(buffer), "%Y-%m-%d %H:%M:%S", tm);
	sprintf(buffer + strlen(buffer), ".%09d", val->nseconds);
	logPrintf(LOGM_NFSTRACE, "%s: %s\n", title, buffer);
}

void	nfs3PrintFattr(struct fattr3 *val)
{
	nfs3PrintType(val->type);
	logPrintf(LOGM_NFSTRACE, "mode:  0%o\n", val->mode);
	logPrintf(LOGM_NFSTRACE, "nlink: %d\n", val->nlink);
	logPrintf(LOGM_NFSTRACE, "uid:   %d\n", val->uid);
	logPrintf(LOGM_NFSTRACE, "gid:   %d\n", val->gid);
	logPrintf(LOGM_NFSTRACE, "size:  %s\n", strvalBtu64(val->size));
	logPrintf(LOGM_NFSTRACE, "used:  %s\n", strvalBtu64(val->used));
	nfs3PrintSpecdata(&val->rdev);
	logPrintf(LOGM_NFSTRACE, "fsid:  %s\n", strvalBtu64(val->fsid));
	logPrintf(LOGM_NFSTRACE, "fileid:  %s\n", strvalBtu64(val->fileid));
	printNfstime("atime", &val->atime);
	printNfstime("mtime", &val->mtime);
	printNfstime("ctime", &val->ctime);
}

void	nfs3PrintDiropargs(struct diropargs3 *val)
{
	nfs3PrintFh(&val->dir);
	logPrintf(LOGM_NFSTRACE, "name: %s\n", val->name);
}

static void	printSetBtu32(btu8 *title, struct set_btu32 *val, btbool isOctal)
{
	if(val->set){
		if(isOctal){
			logPrintf(LOGM_NFSTRACE, "%s: 0%o\n", title, val->val);
		}else{
			logPrintf(LOGM_NFSTRACE, "%s: %d\n", title, val->val);
		}
	}else{
		logPrintf(LOGM_NFSTRACE, "%s: <not set>\n", title);
	}
}

static void	printSetBtu64(btu8 *title, struct set_btu64 *val)
{
	if(val->set){
		logPrintf(LOGM_NFSTRACE, "%s: %s\n", title, strvalBtu64(val->val));
	}else{
		logPrintf(LOGM_NFSTRACE, "%s: <not set>\n", title);
	}
}

static void	printSetNfstime(btu8 *title, struct set_nfstime3 *val)
{
	nfs3PrintTimesetMode(val->mode);
	if(val->mode == NFS3_TIMESET_SET_TO_CLIENT_TIME){
		printNfstime(title, &val->time);
	}
}

void	nfs3PrintSattr(struct sattr3 *val)
{
	printSetBtu32("mode", &val->mode, 1);
	printSetBtu32("uid", &val->uid, 0);
	printSetBtu32("gid", &val->gid, 0);
	printSetBtu64("size", &val->size);
	printSetNfstime("atime", &val->atime);
	printSetNfstime("mtime", &val->mtime);
}

void	nfs3PrintSattrguard(struct sattrguard3 *val)
{
	if(val->check){
		printNfstime("guard check time", &val->obj_ctime);
	}else{
		logPrintf(LOGM_NFSTRACE, "guard check time not set\n");
	}
}

void	nfs3PrintWccAttr(struct wcc_attr *val)
{
	logPrintf(LOGM_NFSTRACE, "wcc: size: %s\n", strvalBtu64(val->size));
	printNfstime("wcc: mtime", &val->mtime);
	printNfstime("wcc: ctime", &val->ctime);
}

void	nfs3PrintPreOpAttr(struct pre_op_attr *val)
{
	if(val->attributes_follow){
		nfs3PrintWccAttr(&val->attributes);
	}else{
		logPrintf(LOGM_NFSTRACE, "preop: no wcc attributes given\n");
	}
}

void	nfs3PrintPostOpAttr(struct post_op_attr *val)
{
	if(val->attributes_follow){
		nfs3PrintFattr(&val->attributes);
	}else{
		logPrintf(LOGM_NFSTRACE, "postop: no attributes given\n");
	}
}

void	nfs3PrintWccData(struct wcc_data *val)
{
	nfs3PrintPreOpAttr(&val->before);
	nfs3PrintPostOpAttr(&val->after);
}

void	nfs3PrintPostOpFh(struct post_op_fh3 *val)
{
	if(val->handle_follows){
		nfs3PrintFh(&val->handle);
	}else{
		logPrintf(LOGM_NFSTRACE, "No post_op_fh given.\n");
	}
}

void	nfs3PrintGetattrRes(struct GETATTR3res *val)
{
	nfs3PrintStatus(val->status);
	if(val->status == NFS3_OK){
		nfs3PrintFattr(&val->attributes);
	}
}

void	nfs3PrintSetattrArgs(struct SETATTR3args *val)
{
	nfs3PrintFh(&val->object);
	nfs3PrintSattr(&val->new_attributes);
	nfs3PrintSattrguard(&val->guard);
}

void	nfs3PrintSetattrRes(struct SETATTR3res *val)
{
	nfs3PrintStatus(val->status);
	nfs3PrintWccData(&val->obj_wcc);
}

void	nfs3PrintLookupRes(struct LOOKUP3res *val)
{
	nfs3PrintStatus(val->status);
	logPrintf(LOGM_NFSTRACE, "directory attributes:\n");
	nfs3PrintPostOpAttr(&val->dir_attributes);
	if(val->status == NFS3_OK){
		logPrintf(LOGM_NFSTRACE, "object attributes:\n");
		nfs3PrintFh(&val->object);
		nfs3PrintPostOpAttr(&val->obj_attributes);
	}
}

void	nfs3PrintAccessArgs(struct ACCESS3args *val)
{
	nfs3PrintFh(&val->object);
	logPrintf(LOGM_NFSTRACE, "access: 0x%x\n", val->access);
}

void	nfs3PrintAccessRes(struct ACCESS3res *val)
{
	nfs3PrintStatus(val->status);
	nfs3PrintPostOpAttr(&val->obj_attributes);
	if(val->status == NFS3_OK){
		logPrintf(LOGM_NFSTRACE, "access: 0x%x\n", val->access);
	}
}

void	nfs3PrintReadlinkRes(struct READLINK3res *val)
{
	nfs3PrintStatus(val->status);
	nfs3PrintPostOpAttr(&val->symlink_attributes);
	if(val->status == NFS3_OK){
		logPrintf(LOGM_NFSTRACE, "data: ->%s<-\n", val->data);
	}
}

void	nfs3PrintReadArgs(struct READ3args *val)
{
	nfs3PrintFh(&val->file);
	logPrintf(LOGM_NFSTRACE, "offset: %s\n", strvalBtu64(val->offset));
	logPrintf(LOGM_NFSTRACE, "count: %d\n", val->count);
}

void	nfs3PrintReadRes(struct READ3res *val)
{
	nfs3PrintStatus(val->status);
	nfs3PrintPostOpAttr(&val->file_attributes);
	if(val->status == NFS3_OK){
		logPrintf(LOGM_NFSTRACE, "count: %d\n", val->count);
		logPrintf(LOGM_NFSTRACE, "EOF: %d\n", val->eof);
		printOpaque("data", &val->data, 16);
	}
}

void	nfs3PrintWriteArgs(struct WRITE3args *val)
{
	nfs3PrintFh(&val->file);
	logPrintf(LOGM_NFSTRACE, "offset: %s\n", strvalBtu64(val->offset));
	logPrintf(LOGM_NFSTRACE, "count: %d\n", val->count);
	nfs3PrintStable(val->stable);
	printOpaque("data", &val->data, 16);
}

void	nfs3PrintWriteRes(struct WRITE3res *val)
{
	nfs3PrintStatus(val->status);
	nfs3PrintWccData(&val->file_wcc);
	if(val->status == NFS3_OK){
		logPrintf(LOGM_NFSTRACE, "count: %d\n", val->count);
		nfs3PrintStable(val->committed);
		printFixedBlock("verf", val->verf, NFS3_WRITEVERFSIZE);
	}
}

void	nfs3PrintCreateArgs(struct CREATE3args *val)
{
	nfs3PrintDiropargs(&val->where);
	nfs3PrintCreateMode(val->mode);
	if(val->mode == NFS3_CREATE_EXCLUSIVE){
		printFixedBlock("verf", val->u.exclusive.verf, NFS3_CREATEVERFSIZE);
	}else if(val->mode == NFS3_CREATE_GUARDED || val->mode == NFS3_CREATE_UNCHECKED){
		nfs3PrintSattr(&val->u.guarded_or_unchecked.obj_attributes);
	}
}

void	nfs3PrintCreateRes(struct CREATE3res *val)
{
	nfs3PrintStatus(val->status);
	nfs3PrintWccData(&val->dir_wcc);
	if(val->status == NFS3_OK){
		nfs3PrintPostOpFh(&val->obj);
		nfs3PrintPostOpAttr(&val->obj_attributes);
	}
}

void	nfs3PrintMkdirArgs(struct MKDIR3args *val)
{
	nfs3PrintDiropargs(&val->where);
	nfs3PrintSattr(&val->attributes);
}

void	nfs3PrintSymlinkArgs(struct SYMLINK3args *val)
{
	nfs3PrintDiropargs(&val->where);
	nfs3PrintSattr(&val->symlink_attributes);
	logPrintf(LOGM_NFSTRACE, "symlink_data: ->%s<-\n", val->symlink_data);
}

void	nfs3PrintMknodArgs(struct MKNOD3args *val)
{
	nfs3PrintDiropargs(&val->where);
	nfs3PrintType(val->type);
	if(val->type == NFS3_TYPE_CHR || val->type == NFS3_TYPE_BLK){
		nfs3PrintSattr(&val->u.char_or_blk.dev_attributes);
		nfs3PrintSpecdata(&val->u.char_or_blk.spec);
	}else if(val->type == NFS3_TYPE_SOCK || val->type == NFS3_TYPE_FIFO){
		nfs3PrintSattr(&val->u.sock_or_fifo.pipe_attributes);
	}
}

void	nfs3PrintRemoveRes(struct REMOVE3res *val)
{
	nfs3PrintStatus(val->status);
	nfs3PrintWccData(&val->dir_wcc);
}

void	nfs3PrintRenameArgs(struct RENAME3args *val)
{
	nfs3PrintDiropargs(&val->from);
	nfs3PrintDiropargs(&val->to);
}

void	nfs3PrintRenameRes(struct RENAME3res *val)
{
	nfs3PrintStatus(val->status);
	nfs3PrintWccData(&val->fromdir_wcc);
	nfs3PrintWccData(&val->todir_wcc);
}

void	nfs3PrintLinkArgs(struct LINK3args *val)
{
	nfs3PrintFh(&val->file);
	nfs3PrintDiropargs(&val->link);
}

void	nfs3PrintLinkRes(struct LINK3res *val)
{
	nfs3PrintStatus(val->status);
	nfs3PrintPostOpAttr(&val->file_attributes);
	nfs3PrintWccData(&val->linkdir_wcc);
}

void	nfs3PrintReaddirArgs(struct READDIR3args *val)
{
	nfs3PrintFh(&val->dir);
	logPrintf(LOGM_NFSTRACE, "cookie: %s\n", strvalBtu64(val->cookie));
	printFixedBlock("cookieverf", val->cookieverf, NFS3_COOKIEVERFSIZE);
	logPrintf(LOGM_NFSTRACE, "count: %d\n", val->count);
}

void	nfs3PrintEntries(struct entry3 *val)
{
btint	i = 0;

	while(val){
		logPrintf(LOGM_NFSTRACE, "entry %d:\n", i++);
		logPrintf(LOGM_NFSTRACE, "  fileid: %s\n", strvalBtu64(val->fileid));
		logPrintf(LOGM_NFSTRACE, "  name: ->%s<-\n", val->name);
		logPrintf(LOGM_NFSTRACE, "  cookie: %s\n", strvalBtu64(val->cookie));
		val = val->nextentry;
	}
}

void	nfs3PrintReaddirRes(struct READDIR3res *val)
{
	nfs3PrintStatus(val->status);
	nfs3PrintPostOpAttr(&val->dir_attributes);
	if(val->status == NFS3_OK){
		printFixedBlock("cookieverf", val->cookieverf, NFS3_COOKIEVERFSIZE);
		nfs3PrintEntries(val->entries);
		logPrintf(LOGM_NFSTRACE, "eof: %d\n", val->eof);
	}
}

void	nfs3PrintReaddirplusArgs(struct READDIRPLUS3args *val)
{
	nfs3PrintFh(&val->dir);
	logPrintf(LOGM_NFSTRACE, "cookie: %s\n", strvalBtu64(val->cookie));
	printFixedBlock("cookieverf", val->cookieverf, NFS3_COOKIEVERFSIZE);
	logPrintf(LOGM_NFSTRACE, "dircount: %d\n", val->dircount);
	logPrintf(LOGM_NFSTRACE, "maxcount: %d\n", val->maxcount);
}

void	nfs3PrintEntriesplus(struct entryplus3 *val)
{
btint	i = 0;

	while(val != NULL){
		logPrintf(LOGM_NFSTRACE, "entryplus %d:\n", i++);
		logPrintf(LOGM_NFSTRACE, "fileid: %s\n", strvalBtu64(val->fileid));
		logPrintf(LOGM_NFSTRACE, "name: ->%s<-\n", val->name);
		logPrintf(LOGM_NFSTRACE, "cookie: %s\n", strvalBtu64(val->cookie));
		nfs3PrintPostOpAttr(&val->name_attributes);
		nfs3PrintPostOpFh(&val->name_handle);
		val = val->nextentry;
	}
}

void	nfs3PrintReaddirplusRes(struct READDIRPLUS3res *val)
{
	nfs3PrintStatus(val->status);
	nfs3PrintPostOpAttr(&val->dir_attributes);
	if(val->status == NFS3_OK){
		printFixedBlock("cookieverf", val->cookieverf, NFS3_COOKIEVERFSIZE);
		nfs3PrintEntriesplus(val->entries);
		logPrintf(LOGM_NFSTRACE, "eof: %d\n", val->eof);
	}
}

void	nfs3PrintFsstatRes(struct FSSTAT3res *val)
{
	nfs3PrintStatus(val->status);
	nfs3PrintPostOpAttr(&val->obj_attributes);
	if(val->status == NFS3_OK){
		logPrintf(LOGM_NFSTRACE, "tbytes: %s\n", strvalBtu64(val->tbytes));
		logPrintf(LOGM_NFSTRACE, "fbytes: %s\n", strvalBtu64(val->fbytes));
		logPrintf(LOGM_NFSTRACE, "abytes: %s\n", strvalBtu64(val->abytes));
		logPrintf(LOGM_NFSTRACE, "tfiles: %s\n", strvalBtu64(val->tfiles));
		logPrintf(LOGM_NFSTRACE, "ffiles: %s\n", strvalBtu64(val->ffiles));
		logPrintf(LOGM_NFSTRACE, "afiles: %s\n", strvalBtu64(val->afiles));
		logPrintf(LOGM_NFSTRACE, "invarsec: %d\n", val->invarsec);
	}
}

void	nfs3PrintFsinfoRes(struct FSINFO3res *val)
{
	nfs3PrintStatus(val->status);
	nfs3PrintPostOpAttr(&val->obj_attributes);
	if(val->status == NFS3_OK){
		logPrintf(LOGM_NFSTRACE, "rtmax: %d\n", val->rtmax);
		logPrintf(LOGM_NFSTRACE, "rtpref: %d\n", val->rtpref);
		logPrintf(LOGM_NFSTRACE, "rtmult: %d\n", val->rtmult);
		logPrintf(LOGM_NFSTRACE, "wtmax: %d\n", val->wtmax);
		logPrintf(LOGM_NFSTRACE, "wtpref: %d\n", val->wtpref);
		logPrintf(LOGM_NFSTRACE, "wtmult: %d\n", val->wtmult);
		logPrintf(LOGM_NFSTRACE, "dtpref: %d\n", val->dtpref);
		logPrintf(LOGM_NFSTRACE, "maxfilesize: %s\n", strvalBtu64(val->maxfilesize));
		printNfstime("time_delta", &val->time_delta);
		logPrintf(LOGM_NFSTRACE, "properties: 0x%x\n", val->properties);
	}
}

void	nfs3PrintPathconfRes(struct PATHCONF3res *val)
{
	nfs3PrintStatus(val->status);
	nfs3PrintPostOpAttr(&val->obj_attributes);
	if(val->status == NFS3_OK){
		logPrintf(LOGM_NFSTRACE, "linkmax: %d\n", val->linkmax);
		logPrintf(LOGM_NFSTRACE, "name_max: %d\n", val->name_max);
		logPrintf(LOGM_NFSTRACE, "no_trunc: %d\n", val->no_trunc);
		logPrintf(LOGM_NFSTRACE, "chown_restricted: %d\n", val->chown_restricted);
		logPrintf(LOGM_NFSTRACE, "case_insensitive: %d\n", val->case_insensitive);
		logPrintf(LOGM_NFSTRACE, "case_preserving: %d\n", val->case_preserving);
	}
}

void	nfs3PrintCommitArgs(struct COMMIT3args *val)
{
	nfs3PrintFh(&val->file);
	logPrintf(LOGM_NFSTRACE, "offset: %s\n", strvalBtu64(val->offset));
	logPrintf(LOGM_NFSTRACE, "count: %d\n", val->count);
}

void	nfs3PrintCommitRes(struct COMMIT3res *val)
{
	nfs3PrintStatus(val->status);
	nfs3PrintWccData(&val->file_wcc);
	if(val->status == NFS3_OK)
		printFixedBlock("verf", val->verf, NFS3_WRITEVERFSIZE);
}

/* ------------------------------------------------------------------------- */
