/*
 * Name: nfs3xdr.h
 * Project: CIFS Client
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 2001-10-25
 * Tabsize: 4
 * Copyright: (c) 2001 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: nfs3xdr.h,v 1.1 2001/11/06 09:50:36 cs Exp $
 */

#ifndef __nfs3xdr_H_INCLUDED__
#define __nfs3xdr_H_INCLUDED__

#include "basictypes.h"
#include "nfs3_prot.h"

/*
General Description:
This module contains marshalling and unmarshalling of the NFS 3 RPC protocol,
and (necessarily) basic data types according to the RFC1014 XDR specification.
*/

/* ------------------------------------------------------------------------- */

struct srpcMsg;

void	nfs3DecodeFh(struct srpcMsg *msg, struct nfs_fh3 *val);
void	nfs3EncodeFh(struct srpcMsg *msg, struct nfs_fh3 *val);
void	nfs3DecodeDiropargs(struct srpcMsg *msg, struct diropargs3 *val);
void	nfs3EncodeGetattrRes(struct srpcMsg *msg, struct GETATTR3res *val);
void	nfs3DecodeSetattrArgs(struct srpcMsg *msg, struct SETATTR3args *val);
void	nfs3EncodeSetattrRes(struct srpcMsg *msg, struct SETATTR3res *val);
void	nfs3EncodeLookupRes(struct srpcMsg *msg, struct LOOKUP3res *val);
void	nfs3DecodeAccessArgs(struct srpcMsg *msg, struct ACCESS3args *val);
void	nfs3EncodeAccessRes(struct srpcMsg *msg, struct ACCESS3res *val);
void	nfs3EncodeReadlinkRes(struct srpcMsg *msg, struct READLINK3res *val);
void	nfs3DecodeReadArgs(struct srpcMsg *msg, struct READ3args *val);
void	nfs3EncodeReadRes(struct srpcMsg *msg,struct READ3res *val);
void	nfs3DecodeWriteArgs(struct srpcMsg *msg, struct WRITE3args *val);
void	nfs3EncodeWriteRes(struct srpcMsg *msg, struct WRITE3res *val);
void	nfs3DecodeCreateArgs(struct srpcMsg *msg, struct CREATE3args *val);
void	nfs3EncodeCreateRes(struct srpcMsg *msg, struct CREATE3res *val);
void	nfs3DecodeMkdirArgs(struct srpcMsg *msg, struct MKDIR3args *val);
#define	nfs3EncodeMkdirRes	nfs3EncodeCreateRes
void	nfs3DecodeSymlinkArgs(struct srpcMsg *msg, struct SYMLINK3args *val);
#define	nfs3EncodeSymlinkRes	nfs3EncodeCreateRes
void	nfs3DecodeMknodArgs(struct srpcMsg *msg, struct MKNOD3args *val);
#define	nfs3EncodeMknodRes	nfs3EncodeCreateRes
void	nfs3EncodeRemoveRes(struct srpcMsg *msg, struct REMOVE3res *val);
#define	nfs3EncodeRmdirRes	nfs3EncodeRemoveRes
void	nfs3DecodeRenameArgs(struct srpcMsg *msg, struct RENAME3args *val);
void	nfs3EncodeRenameRes(struct srpcMsg *msg, struct RENAME3res *val);
void	nfs3DecodeLinkArgs(struct srpcMsg *msg, struct LINK3args *val);
void	nfs3EncodeLinkRes(struct srpcMsg *msg, struct LINK3res *val);
void	nfs3DecodeReaddirArgs(struct srpcMsg *msg, struct READDIR3args *val);
void	nfs3EncodeReaddirRes(struct srpcMsg *msg, struct READDIR3res *val);
void	nfs3DecodeReaddirplusArgs(struct srpcMsg *msg, struct READDIRPLUS3args *val);
void	nfs3EncodeReaddirplusRes(struct srpcMsg *msg, struct READDIRPLUS3res *val);
void	nfs3EncodeFsstatRes(struct srpcMsg *msg, struct FSSTAT3res *val);
void	nfs3EncodeFsinfoRes(struct srpcMsg *msg, struct FSINFO3res *val);
void	nfs3EncodePathconfRes(struct srpcMsg *msg, struct PATHCONF3res *val);
void	nfs3DecodeCommitArgs(struct srpcMsg *msg, struct COMMIT3args *val);
void	nfs3EncodeCommitRes(struct srpcMsg *msg, struct COMMIT3res *val);

/* ------------------------------------------------------------------------- */

#endif /* __nfs3xdr_H_INCLUDED__ */
