/*
 * Name: guimanager.c
 * Project: Sharity
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1999-03-01
 * Tabsize: 4
 * Copyright: (c) 1999 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: guimanager.c,v 1.6 2000/01/21 12:26:46 cs Exp $
 */

#include "stdheaders.h"
#include "array.h"
#include "cfgparse.h"
#include "guimanager.h"
#include "mainmodule.h"
#include "ipc.h"
#include "../uiLib/maininterface.h"
#include "errcodes.h"
#include "memalloc.h"
#include "main.h"
#include "blockpool.h"
#include "process.h"
#include "ipcserver.h"
#include "util.h"
#include "module.h"
#include "evntloop.h"

/* ------------------------------------------------------------------------- */

static void	marshalError(ipcMsg_t *reply, btint class, btint code)
{
btError	err;

	ipcAddInt(reply, class);
	ipcAddInt(reply, code);
	err.class = class;
	err.code = code;
	ipcAddAscii(reply, errDescription(&err));
}

/* ------------------------------------------------------------------------- */

btbool	guimanAllowAccess(btint permFlags, btint uid)
{
cfgProperty_t	*path[8];
btint			numElements;
static btbool	usersMayConfigure;
static btint	usersMayConfigureChg = 0;

	if(CFG_DID_CHANGE(usersMayConfigureChg)){
		numElements = mainModCfgPath(path, ELEMENTS(path));
		usersMayConfigure = cfgEnumValue(cfgQueryFromArray("usersMayConfigure", numElements, path), cfgYesNoEnum, 0);
	}
	if((permFlags & MAIN_GUI_PERM_ANYBODY) != 0)
		return 1;
	if((permFlags & MAIN_GUI_PERM_AUTHORIZED) != 0){
		if(usersMayConfigure || uid == 0)
			return 1;
	}
	if((permFlags & MAIN_GUI_PERM_ROOT) != 0){
		if(uid == 0)
			return 1;
	}
	return 0;
}

/* ------------------------------------------------------------------------- */

void	guimanGetValues(btu8 ***cfgPath, btu8 ***hierarchicalPath, struct array *guiEntries, struct ipcMsg *rq, struct ipcMsg *reply)
{	/* cfgPath[whichPath][whichComponent], all NULL terminated */
btbool				getDefaults;
btint				i, j, cnt, file, num, whichPath, numEntries = 0;
guiEntry_t			*entry;
cfgProperty_t		*p, **path;
btu8				*s, *hitems[2];
guiTypedesEnum_t	*enumDes;

	if(hierarchicalPath != NULL){
		hitems[0] = ipcGetAscii(rq);
		hitems[1] = GUIMAN_DEFAULT_ENTRY;
	}
	num = 0;
	for(i=0;cfgPath[i]!=NULL;i++)
		num++;
	if(hierarchicalPath != NULL)
		for(i=0;hierarchicalPath[i]!=NULL;i++)
			num += 2;
	path = memAlloc(2 * num * sizeof(cfgProperty_t *));
	getDefaults = ipcGetInt(rq);
	if(hierarchicalPath != NULL){
		for(j=0;j<2;j++){	/* first check hierarchical component in db and cfg, then check hierarchical ".default" */
			for(file=(getDefaults ? 1 : 0);file<2;file++){
				for(whichPath=0; hierarchicalPath[whichPath]!=NULL;whichPath++){
					p = cfgFilePlist(file == 0 ? gdata.dbFile : gdata.cfgFile);
					for(i=0; hierarchicalPath[whichPath][i]!=NULL;i++){
						p = cfgQueryWithName(hierarchicalPath[whichPath][i], 1, p);
					}
					p = cfgQueryWithName(hitems[j], 1, p);
					if(p != NULL)
						path[numEntries++] = p;
				}
			}
		}
	}
	for(file=(getDefaults ? 1 : 0);file<2;file++){	/* and finally plain cfg */
		for(whichPath=0;cfgPath[whichPath]!=NULL;whichPath++){
			p = cfgFilePlist(file == 0 ? gdata.dbFile : gdata.cfgFile);
			for(i=0;cfgPath[whichPath][i]!=NULL;i++){
				p = cfgQueryWithName(cfgPath[whichPath][i], 1, p);
			}
			if(p != NULL)
				path[numEntries++] = p;
		}
	}
	marshalError(reply, 0, 0);
	cnt = arrObjectCount(guiEntries);
	for(i=0;i<cnt;i++){
		entry = arrObjectAtIndex(guiEntries, i);
		switch(entry->type){
		case MAIN_GUI_ENTRY_STRING:
		case MAIN_GUI_ENTRY_PASSWORD:
		case MAIN_GUI_ENTRY_FILE:
			s = cfgStringValue(cfgQueryFromArray(entry->extraInfo, numEntries, path), "");
			ipcAddAscii(reply, s);
			break;
		case MAIN_GUI_ENTRY_ENUM:
			enumDes = entry->typeDescriptor;
			num = cfgEnumValue(cfgQueryFromArray(entry->extraInfo, numEntries, path), enumDes->internalNames, 0);
			ipcAddInt(reply, num);
			break;
		case MAIN_GUI_ENTRY_NUMBER:
			num = cfgIntValue(cfgQueryFromArray(entry->extraInfo, numEntries, path), 0);
			ipcAddInt(reply, num);
			break;
		case MAIN_GUI_ENTRY_BOOL:
			num = cfgEnumValue(cfgQueryFromArray(entry->extraInfo, numEntries, path), cfgYesNoEnum, 1);
			ipcAddInt(reply, num);
			break;
		case MAIN_GUI_ENTRY_SEPARATOR:
			/* no data */
			break;
		}
	}
	memFree(path);
}

/* ------------------------------------------------------------------------- */

void	guimanSetValues(btbool isHierarchical, btu8 **cfgPath, struct array *guiEntries, struct ipcMsg *rq, struct ipcMsg *reply, btint uid)
{	/* cfgPath[whichComponent] */
btint				i, num, cnt;
guiEntry_t			*entry;
cfgProperty_t		*sect, *p;
btu8				*s, *item = NULL;
guiTypedesEnum_t	*enumDes;
btbool				isDirty, allowAccess;

	if(isHierarchical)
		item = ipcGetAscii(rq);
	sect = cfgFilePlist(gdata.dbFile);
	for(i=0;cfgPath[i]!=NULL;i++){
		p = cfgQueryWithName(cfgPath[i], 1, sect);
		if(p == NULL){
			p = cfgBuildDictProperty();
			cfgSetProperty(cfgPath[i], sect, p);
		}
		sect = p;
	}
	if(item != NULL){
		p = cfgQueryWithName(item, 1, sect);
		if(p == NULL){
			p = cfgBuildDictProperty();
			cfgSetProperty(item, sect, p);
		}
		sect = p;
	}
	cnt = arrObjectCount(guiEntries);
	for(i=0;i<cnt;i++){
		entry = arrObjectAtIndex(guiEntries, i);
		allowAccess = guimanAllowAccess(entry->accessPermissions, uid);
		isDirty = ipcGetInt(rq);
		switch(entry->type){
		case MAIN_GUI_ENTRY_STRING:
		case MAIN_GUI_ENTRY_PASSWORD:
		case MAIN_GUI_ENTRY_FILE:
			s = ipcGetAscii(rq);
			if(isDirty && allowAccess){
				if(s == NULL)
					s = "";
				if(s[0] == 0 && !entry->mayBeNull){
					cfgSetProperty(entry->extraInfo, sect, NULL);
				}else{
					p = cfgBuildStringProperty(s);
					cfgSetProperty(entry->extraInfo, sect, p);
				}
			}
			break;
		case MAIN_GUI_ENTRY_ENUM:
			num = ipcGetInt(rq);
			if(isDirty && allowAccess){
				enumDes = entry->typeDescriptor;
				p = cfgBuildEnumProperty(num, enumDes->internalNames);
				cfgSetProperty(entry->extraInfo, sect, p);
			}
			break;
		case MAIN_GUI_ENTRY_NUMBER:
			num = ipcGetInt(rq);
			if(isDirty && allowAccess){
				p = cfgBuildIntProperty(num);
				cfgSetProperty(entry->extraInfo, sect, p);
			}
			break;
		case MAIN_GUI_ENTRY_BOOL:
			num = ipcGetInt(rq);
			if(isDirty && allowAccess){
				p = cfgBuildEnumProperty(num, cfgYesNoEnum);
				cfgSetProperty(entry->extraInfo, sect, p);
			}
			break;
		case MAIN_GUI_ENTRY_SEPARATOR:
			/* no data */
			break;
		}
	}
	marshalError(reply, 0, 0);
	cfgUnparse(gdata.dbFile);
}

/* ------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------- */

typedef struct guiCtx{
	PROC_VARSET;
	btu8			*dataBase;
	btu8			*title;
	btu8			*text;
	btu8			*button1;
	btu8			*button2;
	btu8			*button3;
	guiClass_t		*description;
	guiObject_t		*dsObject;
	ipcSrvClient_t	*client;
	procCond_t		*dialogReadyCond;
	btbool			dialogReady;
	btbool			didSet;
	btbool			withEcho;
	array_t			*texts;
}guiCtx_t;

static blockpool_t *guimanPool = NULL;

static void	poolFreeBlock(void *block)
{
    poolReturnBlock(guimanPool, block);
}

/* ------------------------------------------------------------------------- */

static void	dialogGet(struct ipcSrvClient *client, bti32 uid, bti32 gid, void *dsource, ipcMsg_t *rq, ipcMsg_t *reply, void *caller)
{
guiCtx_t	*this = dsource;
array_t		*entries;
btint		cnt, i;
guiEntry_t	*entry;
void		*data;

	marshalError(reply, 0, 0);
	entries = this->description->entries;
	cnt = arrObjectCount(entries);
	for(i=0;i<cnt;i++){
		entry = arrObjectAtIndex(entries, i);
		data = this->dataBase + (btint)entry->extraInfo;
		switch(entry->type){
		case MAIN_GUI_ENTRY_STRING:
		case MAIN_GUI_ENTRY_PASSWORD:
		case MAIN_GUI_ENTRY_FILE:
			if(*(btu8 **)data == NULL){
				ipcAddAscii(reply, "");
			}else{
				ipcAddAscii(reply, *(btu8 **)data);
			}
			break;
		case MAIN_GUI_ENTRY_ENUM:
		case MAIN_GUI_ENTRY_NUMBER:
			ipcAddInt(reply, *(bti32 *)data);
			break;
		case MAIN_GUI_ENTRY_BOOL:
			ipcAddInt(reply, *(btbool *)data);
			break;
		case MAIN_GUI_ENTRY_SEPARATOR:
			/* no data */
			break;
		}
	}
	PROC_IMMEDIATE_RET(caller, NULL);
}

static void	dialogSet(struct ipcSrvClient *client, bti32 uid, bti32 gid, void *dsource, ipcMsg_t *rq, ipcMsg_t *reply, void *caller)
{
guiCtx_t	*this = dsource;
array_t		*entries;
btint		cnt, i;
guiEntry_t	*entry;
void		*data;
btbool		isDirty;
btu8		*s;
bti32		num;

	entries = this->description->entries;
	cnt = arrObjectCount(entries);
	for(i=0;i<cnt;i++){
		entry = arrObjectAtIndex(entries, i);
		data = this->dataBase + (btint)entry->extraInfo;
		isDirty = ipcGetInt(rq); 
		switch(entry->type){
		case MAIN_GUI_ENTRY_STRING:
		case MAIN_GUI_ENTRY_PASSWORD:
		case MAIN_GUI_ENTRY_FILE:
			s = ipcGetAscii(rq);
			if(isDirty){
				if(*(btu8 **)data != NULL){
					memFree(*(btu8 **)data);
				}
				if(s == NULL)
					s = "";
				*(btu8 **)data = utilStrDup(s);
			}
			break;
		case MAIN_GUI_ENTRY_ENUM:
		case MAIN_GUI_ENTRY_NUMBER:
			num = ipcGetInt(rq);
			if(isDirty){
				*(bti32 *)data = num;
			}
			break;
		case MAIN_GUI_ENTRY_BOOL:
			num = ipcGetInt(rq);
			if(isDirty){
				*(btbool *)data = num;
			}
			break;
		case MAIN_GUI_ENTRY_SEPARATOR:
			/* no data */
			break;
		}
	}
	marshalError(reply, 0, 0);
	this->didSet = 1;
	this->dialogReady = 1;
	procCondBroadcast(this->dialogReadyCond, NULL);
	PROC_IMMEDIATE_RET(caller, NULL);
}

static void	dialogCancel(struct ipcSrvClient *client, bti32 uid, bti32 gid, void *dsource, ipcMsg_t *rq, ipcMsg_t *reply, void *caller)
{
guiCtx_t	*this = dsource;

	this->didSet = 0;
	this->dialogReady = 1;
	procCondBroadcast(this->dialogReadyCond, NULL);
	PROC_IMMEDIATE_RET(caller, NULL);
}

static guiDsourceMethod_t	dialogMethods[] = {
	dialogGet,
	dialogSet,
	dialogCancel,
};


static void	pollForClientInvalidate(void *obj, void *arg)
{
guiCtx_t	*this = obj;

	if(ipcSrvClientIsInvalid(this->client)){
		procCondBroadcast(this->dialogReadyCond, NULL);
	}else{
		elpCallAtCancelPrev(elpNow() + 2000, pollForClientInvalidate, this);
	}
}

static void	guimanRunDialogProc(void *obj, void *arg)
{
guiCtx_t	*this = obj;
ipcMsg_t	*msg = arg;
guimanRes_t	res;
btu32		*flag = arg;

	PROC_BEGIN;
	this->dsObject = mainModRegisterDataSource(this, this->description, dialogMethods, ELEMENTS(dialogMethods));
	this->dialogReadyCond = procCondNew();
	msg = ipcMsgNew();
	ipcAddInt(msg, 0);	/* size */
	ipcAddInt(msg, 0);	/* protocol version */
	ipcAddInt(msg, 0);	/* sequence number */
	ipcAddInt(msg, MAIN_IPCCFN_RUN_DIALOG);
	ipcAddInt(msg, 0);	/* uid */
	ipcAddInt(msg, 0);	/* gid */
	ipcAddInt(msg, this->dsObject->id);	/* object ID */
	ipcAddInt(msg, *flag);	/* dialog flags */
	ipcSrvClientRetain(this->client);
	PROC_WAITFOR_X(1, ipcSrvRPC(mainModule->ipcModule, this->client, msg, this));
	if(msg == NULL){	/* client died */
		res.err.class = ERRCLS_FRAMEWORK;
		res.err.code = ERR_FRWclientcomm;
		goto doReturn;
	}
	ipcGetInt(msg);	/* uid */
	ipcGetInt(msg);	/* gid */
	res.err.class = ipcGetInt(msg);
	res.err.code = ipcGetInt(msg);
	if(res.err.class != 0){
		goto doReturn;
	}
	/* poll for client invalidation; this happens if the GUI quits */
	elpCallAtCancelPrev(elpNow() + 2000, pollForClientInvalidate, this);
	while(!this->dialogReady && !ipcSrvClientIsInvalid(this->client)){
		PROC_WAITFOR_X(2, procCondWait(this->dialogReadyCond, this));
	}
	/* stop polling now: */
	elpCancelCallAt(pollForClientInvalidate, this);
	/* allow callback to GUI before we run any button actions: */
	PROC_WAITFOR_X(3, elpCallCancelPrev(guimanRunDialogProc, this));
	res.err.class = res.err.code = 0;
doReturn:
	ipcSrvClientRelease(this->client);
	mainModUnregisterDataSource(this->dsObject);
	procCondFree(this->dialogReadyCond);
	res.didSet = this->didSet;
	PROC_RETURN(&res);
	PROC_END;
}

void	guimanRunDialog(guiClass_t *description, ipcSrvClient_t *client, void *dataBase, btbool runModal, btbool ontopOfFileselector, void *caller)
{
guiCtx_t	*ctx;
btu32		flag = 0;

	if(runModal)
		flag |= MAIN_DIALOG_FLG_MODAL;
	if(ontopOfFileselector)
		flag |= MAIN_DIALOG_FLG_OVER_FILESEL;
	if(guimanPool == NULL){
		guimanPool = poolNew(sizeof(guiCtx_t), sizeof(guiCtx_t));
	}
	PROC_INIT(ctx, guimanRunDialogProc, caller, guimanPool, poolFreeBlock);
	ctx->dataBase = dataBase;
	ctx->description = description;
	ctx->client = client;
	PROC_START(ctx, &flag);
}

/* ------------------------------------------------------------------------- */

static void	guimanRunFileselectorProc(void *obj, void *arg)
{
guiCtx_t	*this = obj;
ipcMsg_t	*msg = arg;
guimanRes_t	res;

	PROC_BEGIN;
	msg = ipcMsgNew();
	ipcAddInt(msg, 0);	/* size */
	ipcAddInt(msg, 0);	/* protocol version */
	ipcAddInt(msg, 0);	/* sequence number */
	ipcAddInt(msg, MAIN_IPCCFN_RUN_FILESELECTOR);
	ipcAddInt(msg, 0);	/* uid */
	ipcAddInt(msg, 0);	/* gid */
	ipcAddAscii(msg, this->title);
	ipcAddAscii(msg, this->text);
	PROC_WAITFOR_X(1, ipcSrvRPC(mainModule->ipcModule, this->client, msg, this));
	if(msg == NULL){	/* client died */
		res.err.class = ERRCLS_FRAMEWORK;
		res.err.code = ERR_FRWclientcomm;
		goto doReturn;
	}
	ipcGetInt(msg);	/* uid */
	ipcGetInt(msg);	/* gid */
	res.err.class = ipcGetInt(msg);
	res.err.code = ipcGetInt(msg);
	res.didSet = ipcGetInt(msg);
	if(res.didSet)
		res.file = ipcGetAscii(msg);
doReturn:
	PROC_RETURN(&res);
	PROC_END;
}

void	guimanRunFileselector(struct ipcSrvClient *client, btu8 *title, btu8 *startAtPath, void *caller)
{
guiCtx_t	*ctx;

	if(guimanPool == NULL){
		guimanPool = poolNew(sizeof(guiCtx_t), sizeof(guiCtx_t));
	}
	PROC_INIT(ctx, guimanRunFileselectorProc, caller, guimanPool, poolFreeBlock);
	ctx->title = title;
	ctx->text = startAtPath;
	ctx->client = client;
	PROC_START(ctx, NULL);
}

/* ------------------------------------------------------------------------- */

static void	guimanRunAlertProc(void *obj, void *arg)
{
guiCtx_t	*this = obj;
ipcMsg_t	*msg = arg;
guimanRes_t	res;

	PROC_BEGIN;
	msg = ipcMsgNew();
	ipcAddInt(msg, 0);	/* size */
	ipcAddInt(msg, 0);	/* protocol version */
	ipcAddInt(msg, 0);	/* sequence number */
	ipcAddInt(msg, MAIN_IPCCFN_RUN_ALERT);
	ipcAddInt(msg, 0);	/* uid */
	ipcAddInt(msg, 0);	/* gid */
	ipcAddAscii(msg, this->title);
	ipcAddAscii(msg, this->text);
	ipcAddAscii(msg, this->button1);
	ipcAddAscii(msg, this->button2);
	ipcAddAscii(msg, this->button3);
	PROC_WAITFOR_X(1, ipcSrvRPC(mainModule->ipcModule, this->client, msg, this));
	if(msg == NULL){	/* client died */
		res.err.class = ERRCLS_FRAMEWORK;
		res.err.code = ERR_FRWclientcomm;
		goto doReturn;
	}
	ipcGetInt(msg);	/* uid */
	ipcGetInt(msg);	/* gid */
	res.err.class = ipcGetInt(msg);
	res.err.code = ipcGetInt(msg);
	res.button = ipcGetInt(msg);
doReturn:
	PROC_RETURN(&res);
	PROC_END;
}

void	guimanRunAlert(struct ipcSrvClient *client, btu8 *title, btu8 *text, btu8 *button1, btu8 *button2, btu8 *button3, void *caller)
{
guiCtx_t	*ctx;

	if(guimanPool == NULL){
		guimanPool = poolNew(sizeof(guiCtx_t), sizeof(guiCtx_t));
	}
	PROC_INIT(ctx, guimanRunAlertProc, caller, guimanPool, poolFreeBlock);
	ctx->title = title;
	ctx->text = text;
	ctx->button1 = button1;
	ctx->button2 = button2;
	ctx->button3 = button3;
	ctx->client = client;
	PROC_START(ctx, NULL);
}

/* ------------------------------------------------------------------------- */

static void	guimanRunSelectionProc(void *obj, void *arg)
{
guiCtx_t	*this = obj;
ipcMsg_t	*msg = arg;
guimanRes_t	res;
btint		i;

	PROC_BEGIN;
	msg = ipcMsgNew();
	ipcAddInt(msg, 0);	/* size */
	ipcAddInt(msg, 0);	/* protocol version */
	ipcAddInt(msg, 0);	/* sequence number */
	ipcAddInt(msg, MAIN_IPCCFN_RUN_SELECTION);
	ipcAddInt(msg, 0);	/* uid */
	ipcAddInt(msg, 0);	/* gid */
	ipcAddAscii(msg, this->title);
	ipcAddAscii(msg, this->button1);
	ipcAddAscii(msg, this->button2);
	ipcAddInt(msg, arrObjectCount(this->texts));
	for(i=0;i<arrObjectCount(this->texts);i++){
		ipcAddAscii(msg, arrObjectAtIndex(this->texts, i));
	}
	PROC_WAITFOR_X(1, ipcSrvRPC(mainModule->ipcModule, this->client, msg, this));
	if(msg == NULL){	/* client died */
		res.err.class = ERRCLS_FRAMEWORK;
		res.err.code = ERR_FRWclientcomm;
		goto doReturn;
	}
	ipcGetInt(msg);	/* uid */
	ipcGetInt(msg);	/* gid */
	res.err.class = ipcGetInt(msg);
	res.err.code = ipcGetInt(msg);
	res.didSet = ipcGetInt(msg);
	if(res.didSet)
		res.file = ipcGetAscii(msg);
doReturn:
	PROC_RETURN(&res);
	PROC_END;
}

void	guimanRunSelection(struct ipcSrvClient *client, btu8 *title, btu8 *ok, btu8 *cancel, struct array *texts, void *caller)
{
guiCtx_t	*ctx;

	if(guimanPool == NULL){
		guimanPool = poolNew(sizeof(guiCtx_t), sizeof(guiCtx_t));
	}
	PROC_INIT(ctx, guimanRunSelectionProc, caller, guimanPool, poolFreeBlock);
	ctx->title = title;
	ctx->button1 = ok;
	ctx->button2 = cancel;
	ctx->client = client;
	ctx->texts = texts;
	PROC_START(ctx, NULL);
}

/* ------------------------------------------------------------------------- */

static void	guimanDisplayErrorProc(void *obj, void *arg)
{
guiCtx_t	*this = obj;
ipcMsg_t	*msg = arg;
btError		*err = arg;
guimanRes_t	res;

	PROC_BEGIN;
	msg = ipcMsgNew();
	ipcAddInt(msg, 0);	/* size */
	ipcAddInt(msg, 0);	/* protocol version */
	ipcAddInt(msg, 0);	/* sequence number */
	ipcAddInt(msg, MAIN_IPCCFN_DISPLAY_ERROR);
	ipcAddInt(msg, 0);	/* uid */
	ipcAddInt(msg, 0);	/* gid */
	marshalError(msg, err->class, err->code);
	PROC_WAITFOR_X(1, ipcSrvRPC(mainModule->ipcModule, this->client, msg, this));
	if(msg == NULL){	/* client died */
		res.err.class = ERRCLS_FRAMEWORK;
		res.err.code = ERR_FRWclientcomm;
		goto doReturn;
	}
	ipcGetInt(msg);	/* uid */
	ipcGetInt(msg);	/* gid */
	res.err.class = ipcGetInt(msg);
	res.err.code = ipcGetInt(msg);
doReturn:
	PROC_RETURN(&res);
	PROC_END;
}

void	guimanDisplayError(struct ipcSrvClient *client, btError *err, void *caller)
{
guiCtx_t	*ctx;

	if(guimanPool == NULL){
		guimanPool = poolNew(sizeof(guiCtx_t), sizeof(guiCtx_t));
	}
	PROC_INIT(ctx, guimanDisplayErrorProc, caller, guimanPool, poolFreeBlock);
	ctx->client = client;
	PROC_START(ctx, err);
}

/* ------------------------------------------------------------------------- */

static void	guimanAskDataProc(void *obj, void *arg)
{
guiCtx_t	*this = obj;
ipcMsg_t	*msg = arg;
guimanRes_t	res;

	PROC_BEGIN;
	msg = ipcMsgNew();
	ipcAddInt(msg, 0);	/* size */
	ipcAddInt(msg, 0);	/* protocol version */
	ipcAddInt(msg, 0);	/* sequence number */
	ipcAddInt(msg, MAIN_IPCCFN_ASK_DATA);
	ipcAddInt(msg, 0);	/* uid */
	ipcAddInt(msg, 0);	/* gid */
	ipcAddAscii(msg, this->text);
	ipcAddInt(msg, this->withEcho);
	PROC_WAITFOR_X(1, ipcSrvRPC(mainModule->ipcModule, this->client, msg, this));
	if(msg == NULL){	/* client died */
		res.err.class = ERRCLS_FRAMEWORK;
		res.err.code = ERR_FRWclientcomm;
		goto doReturn;
	}
	ipcGetInt(msg);	/* uid */
	ipcGetInt(msg);	/* gid */
	res.err.class = ipcGetInt(msg);
	res.err.code = ipcGetInt(msg);
	res.file = ipcGetAscii(msg);
doReturn:
	PROC_RETURN(&res);
	PROC_END;
}

void	guimanAskData(struct ipcSrvClient *client, btu8 *description, btbool withEcho, void *caller)
{
guiCtx_t	*ctx;

	if(guimanPool == NULL){
		guimanPool = poolNew(sizeof(guiCtx_t), sizeof(guiCtx_t));
	}
	PROC_INIT(ctx, guimanAskDataProc, caller, guimanPool, poolFreeBlock);
	ctx->client = client;
	ctx->text = description;
	ctx->withEcho = withEcho;
	PROC_START(ctx, NULL);
}

/* ------------------------------------------------------------------------- */
