#!/bin/sh
# Name: gtkinfo.sh
# Project: Sharity
# Author: Christian Starkjohann <cs@obdev.at>
# Creation Date: 1999-07-22
# Tabsize: 4
# Copyright: (c) 1999 by Christian Starkjohann, all rights reserved.
#     For details of the license see the file doc/License.txt.
# This Revision: $Id: gtkinfo.sh,v 1.4 2002/02/18 12:37:39 cs Exp $

# General Description:
# This script is a frontend to "gtk-config". It can rewrite the linker
# flags for static linking of the GTK libraries and it tries to search
# for the gtk-config utility intelligently.

whereis()
{
	echo $PATH | tr ':' '\012' | {
		while read path; do
			if [ -x "$path/$1" ]; then
				echo "$path/$1"
				break
			fi
		done
	}
}

findlib()
{
	thislib=`echo $lpath | tr ':' '\012' | {
		while read path; do
			if [ -f "$path/lib$1.a" ]; then
				echo "$path/lib$1.a"
				break
			fi
		done
	}`
	if [ -z "$thislib" ]; then
		echo "*** Could not find static library lib$1.a" 1>&2
		exit 1;
	fi
	echo "$thislib"
}

# first search explicitly for GTK+ 1.2
gtkcfg=`whereis gtk12-config`

# if no explicit 1.2 version was found, search for generic GTK+
if [ -z "$gtkcfg" ]; then
	gtkcfg=`whereis gtk-config`
	if [ -z "$gtkcfg" ]; then
		echo '*** Could not find GTK utility gtk-config in $PATH' 1>&2
		exit 1
	fi
fi

function="$1"
shift
syslibs="$*"

case "$function" in
--static-libs|--static-libs-aix)
	for i in `$gtkcfg --libs`; do
		case $i in
		-L*)
			lpath="$lpath:`echo $i | sed 's/^-L//'`"
			output="$output $i"
			;;
		-l*)
			library=`echo $i | sed 's/^-l//'`
			case $library in
			gtk* | gdk* | gmodule* | glib* | intl*)
				staticlib="$staticlib `findlib $library`"
				;;
			*)
				output="$output $i"
				;;
			esac
			;;
		*)
			output="$output $i"
			;;
		esac
	done
	if [ "$function" = --static-libs-aix ]; then
		echo -b static $staticlib -b dynamic $syslibs $output
	else
		echo $staticlib $syslibs $output
	fi
	;;
--libs)
	$gtkcfg --libs
	;;
--cflags)
	$gtkcfg --cflags
	;;
*)
	echo "usage: $0 --cflags | --libs | --static-libs [system-libs]" 1>&2
esac
