/*
 * Name: guiHelper.c
 * Project: CIFS Client GUI
 * Author: Christian Starkjohann <cs@obdev.at>
 * Creation Date: 1998-12-18
 * Tabsize: 4
 * Copyright: (c) 1998 by Christian Starkjohann, all rights reserved.
 *     For details of the license see the file doc/License.txt.
 * This Revision: $Id: guiHelper.c,v 1.6 2000/10/07 12:47:02 cs Exp $
 */

#include "stdheaders.h"
#include <gtk/gtk.h>
#include "basictypes.h"
#include "guiHelper.h"
#include "htmlDisplay.h"

static bti8	*appName = NULL;

static bti8	*helpIntro = "If you move the mouse cursor over a component with help associated, it will turn into a question mark. Clicking on the component displays the help text in this window.";

static bti8	guiRestoreCursor[] = "sharityGuiRestoreCursor";

/* ------------------------------------------------------------------------- */

void	guiSetAppName(bti8 *name)
{
	if(appName != NULL)
		g_free(appName);
	appName = g_strdup(name);
}

void	guiSetHelpIntroText(bti8 *text)
{
	helpIntro = g_strdup(text);
}

/* ------------------------------------------------------------------------- */

GtkWidget	*guiButtonBar(gint numButtons, ...)
{
va_list			vlist;
GtkWidget		*button, *box, **output, *obox;
gint			i;
gchar			*text;

	box = gtk_hbox_new(TRUE, 0);
	va_start(vlist, numButtons);
	for(i=0;i<numButtons;i++){
		text = va_arg(vlist, gchar *);
		output = va_arg(vlist, GtkWidget**);
		button = gtk_button_new_with_label(text);
    	gtk_container_set_border_width(GTK_CONTAINER(button), 5);
		if(output != NULL)
			*output = button;
		gtk_widget_show(button);
		gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
	}
	va_end(vlist);
	obox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_end(GTK_BOX(obox), box, FALSE, FALSE, 0);
	gtk_widget_show(box);
	gtk_widget_show(obox);
	return obox;
}

/* ------------------------------------------------------------------------- */

struct _GtkWidget	*guiAlignmentBox(btbool isVert, btint padding, ...)
{
va_list		vlist;
GtkWidget	*element, *box;
gint		fill;

	box = isVert ? gtk_vbox_new(FALSE, padding) : gtk_hbox_new(FALSE, padding);
	va_start(vlist, padding);
	while((element = va_arg(vlist, GtkWidget *)) != NULL){
		fill = va_arg(vlist, gint);
		gtk_box_pack_start(GTK_BOX(box), element, fill, fill, 0);
	}
	va_end(vlist);
	gtk_widget_show(box);
	return box;
}

/* ------------------------------------------------------------------------- */

#include "sharity.xpm"

char	**guiGetIconXpm(void)
{
	return sharity_xpm;
}

/* ------------------------------------------------------------------------- */

static gchar	*fontNames[GUI_FONT_HUGE+1] = {
	"-adobe-helvetica-%s-%s-normal--*-100-*-*-*-*-*-*",
	"-adobe-helvetica-%s-%s-normal--*-120-*-*-*-*-*-*",
	"-adobe-helvetica-%s-%s-normal--*-140-*-*-*-*-*-*",
	"-adobe-helvetica-%s-%s-normal--*-180-*-*-*-*-*-*",
	"-adobe-helvetica-%s-%s-normal--*-240-*-*-*-*-*-*",
};

struct _GdkFont	*guiGetFont(gint fontSize, btbool isBold, btbool isOblique)
{
static GdkFont	*fontCache[GUI_FONT_HUGE+1][2][2];
static btbool	initialized = 0;
gchar			buffer[256];
gint			boldIndex, obliqueIndex;

	if(!initialized){
		memset(fontCache, 0, sizeof(fontCache));
		initialized = 1;
	}
	boldIndex = isBold ? 1 : 0;	/* be sure to have 1 or 0 for indexing */
	obliqueIndex = isOblique ? 1 : 0;	/* be sure to have 1 or 0 for indexing */
	if(fontSize < 0)
		fontSize = 0;
	if(fontSize > GUI_FONT_HUGE)
		fontSize = GUI_FONT_HUGE;
	if(fontCache[fontSize][boldIndex][obliqueIndex] == NULL){
		sprintf(buffer, fontNames[fontSize], isBold ? "bold" : "medium", isOblique ? "o" : "r");
		fontCache[fontSize][boldIndex][obliqueIndex] = gdk_font_load(buffer);
		gdk_font_ref(fontCache[fontSize][boldIndex][obliqueIndex]);
	}
	return fontCache[fontSize][boldIndex][obliqueIndex];
}

struct _GtkWidget	*guiHeadlineLabel(bti8 *text)
{
GtkWidget		*label;
static GtkStyle	*boldFontStyle = NULL;

	if(boldFontStyle == NULL){
		boldFontStyle = gtk_style_copy(gtk_widget_get_default_style());
		gtk_style_ref(boldFontStyle);
		boldFontStyle->font = guiGetFont(GUI_FONT_LARGE, TRUE, FALSE);
	}
	label = gtk_label_new(text);
	if(boldFontStyle != NULL)
		gtk_widget_set_style(label, boldFontStyle);
	return label;
}

/* ------------------------------------------------------------------------- */

static btint	alertRval;

static void	alertQuitSignal(GtkWidget *sender, gpointer arg)
{
	alertRval = (gint)arg;
	gtk_main_quit();
}

static gint	alertQuitEvent(GtkWidget *sender, GdkEvent *event, gpointer arg)
{
	alertRval = (gint)arg;
	gtk_main_quit();
	return TRUE;
}

btint	guiRunAlert(bti8 *button0, bti8 *button1, bti8 *button2, bti8 *title, bti8 *format, ...)
{
va_list		vlist;
gchar		buffer[2048], *buttons[3], *p, *q, *lineStart;
GtkWidget	*window, *label, *vbox, *hbox, *obox, *button, *image, *header, *sep;
GdkPixmap	*pixmap;
GdkBitmap	*mask;
GtkStyle	*style;
gint		i, width, numLines, textHeight;
GdkColor	black;
GdkFont		*font;

	buttons[0] = button0;
	buttons[1] = button1;
	buttons[2] = button2;
	black.red = black.green = black.blue = 0;
	black.pixel = 0;
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	if(appName != NULL)
		sprintf(buffer, "%s Alert", appName);
	else
		sprintf(buffer, "Alert");
    gtk_window_set_title(GTK_WINDOW(window), buffer);
    gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(alertQuitEvent), (gpointer)-1);
    gtk_container_set_border_width(GTK_CONTAINER (window), 6);
	gtk_widget_realize(window);
    style = gtk_widget_get_style(window);
    pixmap = gdk_pixmap_create_from_xpm_d(window->window,  &mask, &style->bg[GTK_STATE_NORMAL], sharity_xpm);
    image = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(image);
	label = guiHeadlineLabel(title);
	gtk_misc_set_alignment(GTK_MISC(label), 0.1, 0.5);
	gtk_widget_show(label);
	header = guiAlignmentBox(FALSE, 0,
		image, FALSE,
		label, TRUE,
		NULL);
	sep = gtk_hseparator_new();
	gtk_widget_show(sep);
	hbox = gtk_hbox_new(TRUE, 0);
	for(i=0;i<3;i++){
		if(buttons[i] != NULL){
			button = gtk_button_new_with_label(buttons[i]);
			gtk_container_set_border_width(GTK_CONTAINER(button), 5);
			if(strlen(buttons[i]) < 6)
				gtk_widget_set_usize(button, 70, -1);
			gtk_widget_show(button);
			gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
			gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(alertQuitSignal), (gpointer)i);
			if(button2 != NULL){
				GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
			}
			if(i == 2)
				gtk_window_set_default(GTK_WINDOW(window), button);
		}
	}
	obox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_end(GTK_BOX(obox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);
	gtk_widget_show(obox);
	va_start(vlist, format);
	vsprintf(buffer, format, vlist);
	va_end(vlist);
	/* line wrapping code: */
	label = gtk_label_new("");
	font = gtk_widget_get_style(GTK_WIDGET(label))->font;
	p = lineStart = buffer;
	width = 0;
	numLines = 1;
	while(*p!=0){
		if(*p == '\n'){
			lineStart = p;
			width = 0;
			p++;
			numLines++;
			continue;
		}
		width += gdk_char_width(font, *p++);
		if(width >= 340){
			for(q=p;q>lineStart;q--){
				if(isspace((btu8)q[-1])){
					p = q;
					break;
				}
			}
			p[-1] = '\n';	/* replace space or last fitting char with \n */ 
			lineStart = p;
			width = 0;
			numLines++;
		}
	}
	if(numLines > 1)
		gtk_label_set_justify (GTK_LABEL(label), GTK_JUSTIFY_LEFT);
	gtk_label_set_text(GTK_LABEL(label), buffer);
	gtk_widget_show(label);
	vbox = guiAlignmentBox(TRUE, 0,
		header, FALSE,
		sep, FALSE,
		label, TRUE,
		obox, FALSE,
		NULL);
	gtk_container_add(GTK_CONTAINER(window), vbox);
	textHeight = (font->ascent + font->descent + 2) * numLines;
	gtk_widget_set_usize(window, 350, 130 + textHeight);
	gtk_widget_show(window);
	gtk_grab_add(window);
	gtk_main();
	gtk_grab_remove(window);
	gtk_widget_destroy(window);
	return alertRval;
}

/* ------------------------------------------------------------------------- */

static gint	fselRval;

static gint	fselReadySignal(gpointer rval)
{
	fselRval = (gint)rval;
	gtk_main_quit();
	return TRUE;
}

btu8	*guiRunFileselector(btu8 *title, btu8 *initialPath)
{
GtkWidget	*fsel;
gchar		*name = NULL;
btint		l;

	fsel = gtk_file_selection_new(title);
	gtk_signal_connect_object(GTK_OBJECT(fsel), "delete_event", GTK_SIGNAL_FUNC(fselReadySignal), (gpointer)0);
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(fsel)->ok_button), "clicked", GTK_SIGNAL_FUNC(fselReadySignal), (gpointer)1);
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(fsel)->cancel_button), "clicked", GTK_SIGNAL_FUNC(fselReadySignal), (gpointer)0);
	if(initialPath != NULL)
		gtk_file_selection_set_filename(GTK_FILE_SELECTION(fsel), initialPath);
    gtk_widget_show(fsel);
	gtk_grab_add(fsel);
	gtk_main();
	if(fselRval){
		name = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fsel));
		while(name[0] == '/' && name[1] == '/')	/* remove multiple leading slashes */
			name++;
		l = strlen(name);
		while(l > 1 && name[l-1] == '/')	/* remove trailing slashes */
			name[--l] = 0;
		name = g_strdup(name);
	}
	gtk_grab_remove(fsel);
	gtk_widget_destroy(fsel);
	return name;
}

/* ------------------------------------------------------------------------- */

static btbool	helpIsActive = 0;
static btbool	helpTextIsIntro;
static GList	*helpButtons = NULL;
static gint	helpMouseLeaves(GtkWidget *widget, GdkEventCrossing *event);

static void	displayHelp(bti8 *helpText);

static gint	helpWindowCloseEvent(GtkWidget *window)
{
	if(helpIsActive){
		helpTextIsIntro = 1;
		guiSetHelpMode(FALSE);
	}else{
		displayHelp(NULL);
	}
	return 1;
}

static void	helpWindowNew(GtkWidget **textWidget, GtkWidget **windowWidget)
{
GtkWidget	*window, *text, *scroller, *box, *content;
btu8		buffer[256];

	sprintf(buffer, "%s Help", appName);
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_usize(window, 300, 350);
	gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, FALSE);
    gtk_window_set_title(GTK_WINDOW(window), buffer);
    gtk_container_set_border_width(GTK_CONTAINER(window), 0);
	gtk_signal_connect_object(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(helpWindowCloseEvent), GTK_OBJECT(window));
	text = gtk_text_new(NULL, NULL);
	scroller = gtk_vscrollbar_new (GTK_TEXT(text)->vadj);
	box = guiAlignmentBox(FALSE, 0, text, TRUE, scroller, FALSE, NULL);
	gtk_widget_show(scroller);
	gtk_text_set_editable(GTK_TEXT(text), FALSE);
	gtk_text_set_word_wrap(GTK_TEXT(text), TRUE);
	gtk_widget_show(text);

	content = box;
	gtk_container_add(GTK_CONTAINER(window), content);
	*textWidget = text;
	*windowWidget = window;
}

static void	displayHelp(bti8 *helpText)
{
static GtkWidget	*window = NULL, *text;
static gint			winX, winY;
static btbool		winIsDisplayed = 0;
btbool				winIsNew;

	if(helpText != NULL){
		winIsNew = (window == NULL);
		if(winIsNew){
			helpWindowNew(&text, &window);
		}
		gtk_widget_realize(text);
		gtk_text_freeze(GTK_TEXT(text));
		gtk_text_set_point(GTK_TEXT(text), 0);
		gtk_text_forward_delete(GTK_TEXT(text), gtk_text_get_length(GTK_TEXT(text)));
		htmlInsertText(GTK_TEXT(text), helpText);
		gtk_text_thaw(GTK_TEXT(text));
		gtk_widget_show(window);
		gdk_window_raise(GTK_WIDGET(window)->window);
		winIsDisplayed = 1;
	}else{
		if(window != NULL && winIsDisplayed){
			gdk_window_get_position(GTK_WIDGET(window)->window, &winX, &winY);
			gtk_widget_hide(window);
			winIsDisplayed = 0;
			gtk_widget_set_uposition(window, winX, winY);
		}
	}
}

static gint	helpClick(GtkWidget *widget, GdkEvent *event, bti8 *helpText)
{
	if(helpIsActive && event->type == GDK_BUTTON_PRESS){
		helpTextIsIntro = 0;
		displayHelp(helpText);
		helpMouseLeaves(widget, NULL);
		return TRUE;
	}
	return FALSE;
}

/* ------------------------------------------------------------------------- */

static gint	helpMouseLeaves(GtkWidget *widget, GdkEventCrossing *event)
{
GdkCursor	*cursor;

	if(gtk_object_get_data(GTK_OBJECT(widget), guiRestoreCursor) != NULL){
  		cursor = gdk_cursor_new(GDK_TOP_LEFT_ARROW);
		gdk_window_set_cursor(GTK_WIDGET(widget)->window, cursor);
		gdk_cursor_destroy(cursor);
		gtk_object_set_data(GTK_OBJECT(widget), guiRestoreCursor, NULL);
	}
	return FALSE;
}

static gint	helpMouseEnters(GtkWidget *widget, GdkEventCrossing *event)
{
GdkCursor	*cursor;

	if(helpIsActive){
  		cursor = gdk_cursor_new(GDK_QUESTION_ARROW);
		gdk_window_set_cursor(GTK_WIDGET(widget)->window, cursor);
		gdk_cursor_destroy(cursor);
		gtk_object_set_data(GTK_OBJECT(widget), guiRestoreCursor, guiRestoreCursor);
		gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "enter_notify_event");
		return TRUE;
	}else{
		helpMouseLeaves(widget, NULL);
	}
	return FALSE;
}

/* ------------------------------------------------------------------------- */

void	guiHelpifyWidget(struct _GtkWidget *widget, bti8 *helpText)
{
	gtk_signal_connect(GTK_OBJECT(widget), "event", GTK_SIGNAL_FUNC(helpClick), helpText);
	gtk_signal_connect(GTK_OBJECT(widget), "enter_notify_event", GTK_SIGNAL_FUNC(helpMouseEnters), NULL);
	gtk_signal_connect(GTK_OBJECT(widget), "leave_notify_event", GTK_SIGNAL_FUNC(helpMouseLeaves), NULL);
}

void	guiSetHelpMode(btbool helpMode)
{
GList	*l;

	if(!helpIsActive != !helpMode){
		helpIsActive = helpMode;
		if(helpIsActive){
			helpTextIsIntro = 1;
			displayHelp(helpIntro);
		}else if(helpTextIsIntro){
			displayHelp(NULL);	/* hide if no useful info */
		}
		for(l=helpButtons;l!=NULL;l=g_list_next(l)){
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(l->data), helpIsActive);
		}
	}
}

static void	helpClicked(GtkWidget *sender, void *dummy)
{
	guiSetHelpMode(GTK_TOGGLE_BUTTON(sender)->active);
}

void	guiRegisterHelpTogglebutton(struct _GtkWidget *helpButton)
{
	gtk_signal_connect(GTK_OBJECT(helpButton), "clicked", GTK_SIGNAL_FUNC(helpClicked), NULL);
	helpButtons = g_list_append(helpButtons, helpButton);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(helpButton), helpIsActive);
}

void	guiUnregisterHelpTogglebutton(struct _GtkWidget *helpButton)
{
	helpButtons = g_list_remove(helpButtons, helpButton);
}

/* ------------------------------------------------------------------------- */

static gint	selectionRval;

static gint	selectionQuitSignal(gpointer *x)
{
	selectionRval = (gint)x;
	gtk_main_quit();
	return TRUE;
}

static void	selectionSelectSignal(GtkWidget *sender, GtkWidget *entry)
{
GList		*selection;
GtkWidget	*item;
gchar		*dirName;

	selection = GTK_LIST(sender)->selection;
	if(selection != NULL && selection->data != NULL){
		item = selection->data;
		dirName = gtk_object_get_user_data(GTK_OBJECT(item));
		if(dirName != NULL){
			gtk_entry_set_text(GTK_ENTRY(entry), dirName);
		}
	}
}

static void	selectionClickSignal(GtkWidget *item, GdkEventButton *event)
{
	if(GTK_IS_LIST_ITEM(item) && (event->type==GDK_2BUTTON_PRESS || event->type==GDK_3BUTTON_PRESS)){
		selectionRval = 1;
		gtk_main_quit();
	}
}

bti8	*guiRunSelectionBox(bti8 *title, bti8 *okButton, bti8 *cancelButton, struct _GList *entries)
{
GtkWidget	*list, *scrollWin, *cancel, *add, *entry, *content, *window;
GList		*l;
gchar		*rval = NULL;

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window), title);
	gtk_signal_connect_object(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(selectionQuitSignal), (gpointer)0);
    gtk_container_set_border_width(GTK_CONTAINER(window), 6);

	scrollWin = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollWin), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    gtk_widget_set_usize(scrollWin, -1, 150);
	gtk_widget_show(scrollWin);
    
    list = gtk_list_new();
	entry = gtk_entry_new();
	gtk_signal_connect(GTK_OBJECT(list), "selection_changed", GTK_SIGNAL_FUNC(selectionSelectSignal), entry);
	gtk_list_set_selection_mode(GTK_LIST(list), GTK_SELECTION_BROWSE);
    gtk_widget_show(list);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollWin), list);
	gtk_widget_show(GTK_WIDGET(entry));
	content = guiAlignmentBox(TRUE, 5,
		scrollWin, TRUE,
		entry, FALSE,
		guiButtonBar(2, cancelButton, &cancel, okButton, &add), FALSE,
		NULL);
	GTK_WIDGET_SET_FLAGS(cancel, GTK_CAN_DEFAULT);
	GTK_WIDGET_SET_FLAGS(add, GTK_CAN_DEFAULT);
	gtk_window_set_default(GTK_WINDOW(window), add);
	gtk_signal_connect_object(GTK_OBJECT(cancel), "clicked", GTK_SIGNAL_FUNC(selectionQuitSignal), (gpointer)0);
	gtk_signal_connect_object(GTK_OBJECT(add), "clicked", GTK_SIGNAL_FUNC(selectionQuitSignal), (gpointer)1);
    for(l=entries;l!=NULL;l=g_list_next(l)) {
		GtkWidget	*listItem;
		listItem = gtk_list_item_new_with_label(l->data);
		gtk_object_set_user_data(GTK_OBJECT(listItem), l->data);	/* have name as user data */
		gtk_container_add(GTK_CONTAINER(list), listItem);
		gtk_signal_connect(GTK_OBJECT(listItem), "button_press_event", GTK_SIGNAL_FUNC(selectionClickSignal), NULL);
		gtk_widget_show(listItem);
    }
	gtk_container_add(GTK_CONTAINER(window), content);
	gtk_grab_add(window);	/* run modal */
	gtk_widget_show(window);
	gtk_main();
	if(selectionRval != 0){
		rval = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
	}
	gtk_grab_remove(window);
	gtk_widget_destroy(window);
	return rval;
}

/* ------------------------------------------------------------------------- */
